@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  * 
- * Copyright Ericsson AB 2003-2020. All Rights Reserved.
+ * Copyright Ericsson AB 2003-2021. All Rights Reserved.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -67,7 +67,6 @@
 #endif
 
 #define MIN_MBC_SZ		(16*1024)
-#define MIN_MBC_FIRST_FREE_SZ	(4*1024)
 
 #define TREE_NODE_FLG		(((Uint) 1) << 0)
 #define RED_FLG			(((Uint) 1) << 1)
@@ -306,7 +305,6 @@ erts_aoffalc_start(AOFFAllctr_t *alc,
     alc->crr_order                      = aoffinit->crr_order;
     allctr->mbc_header_size		= sizeof(AOFF_Carrier_t);
     allctr->min_mbc_size		= MIN_MBC_SZ;
-    allctr->min_mbc_first_free_size	= MIN_MBC_FIRST_FREE_SZ;
     allctr->min_block_size              = sizeof(AOFF_RBTree_t);
 
     allctr->vsn_str			= ERTS_ALC_AOFF_ALLOC_VSN_STR;
@@ -603,9 +601,11 @@ rbt_delete(AOFF_RBTree_t** root, AOFF_RBTree_t* del)
     /* Find node to splice out */
     if (!z->left || !z->right)
 	y = z;
-    else
+    else {
 	/* Set y to z:s successor */
-	for(y = z->right; y->left; y = y->left);
+	for(y = z->right; y->left; y = y->left)
+	    ;
+    }
     /* splice out y */
     x = y->left ? y->left : y->right;
     spliced_is_black = IS_BLACK(y);
@@ -1073,7 +1073,8 @@ static Block_t *aoff_first_fblk_in_mbc(Allctr_t *allctr, Carrier_t *carrier)
         AOFF_RBTree_t *blk;
 
         /* Descend to the rightmost block of the tree. */
-        for (blk = crr->root; blk->right; blk = blk->right);
+        for (blk = crr->root; blk->right; blk = blk->right)
+            ;
 
         return (Block_t*)blk;
     }
@@ -1093,7 +1094,8 @@ static Block_t *aoff_next_fblk_in_mbc(Allctr_t *allctr, Carrier_t *carrier,
 
     if (blk->left) {
         /* Descend to the rightmost block of the left subtree. */
-        for (blk = blk->left; blk->right; blk = blk->right);
+        for (blk = blk->left; blk->right; blk = blk->right)
+            ;
 
         return (Block_t*)blk;
     }
@@ -1277,10 +1279,10 @@ static void print_tree(AOFF_RBTree_t*);
 
 /*
  * Checks that the order between parent and children are correct,
- * and that the Red-Black Tree properies are satisfied. if size > 0,
+ * and that the Red-Black Tree properties are satisfied. if size > 0,
  * check_tree() returns the node that satisfies "address order first fit"
  *
- * The Red-Black Tree properies are:
+ * The Red-Black Tree properties are:
  *   1. Every node is either red or black.
  *   2. Every leaf (NIL) is black.
  *   3. If a node is red, then both its children are black.
