@@ -96,12 +96,16 @@ static erts_lc_lock_order_t erts_lock_order[] = {
     {	"proc_msgq",				"pid"			},
     {	"proc_btm",				"pid"			},
     {	"dist_entry_links",			"address"		},
+    {   "nif_load",                             NULL                    },
     {   "update_persistent_term_permission",    NULL                    },
     {   "persistent_term_delete_permission",    NULL                    },
-    {   "code_write_permission",                NULL                    },
+    {   "code_stage_permission",                NULL                    },
+    {   "code_mod_permission",                  NULL                    },
     {	"purge_state",		      		NULL			},
     {	"proc_status",				"pid"			},
     {	"proc_trace",				"pid"			},
+    {   "trace_session_list",                   NULL                    },
+    {   "trace_cleaner",                        NULL                    },
     {	"node_table",				NULL			},
     {	"dist_table",				NULL			},
     {	"sys_tracers",				NULL			},
@@ -109,6 +113,7 @@ static erts_lc_lock_order_t erts_lock_order[] = {
     {	"fun_tab",				NULL			},
     {	"environ",				NULL			},
     {	"release_literal_areas",		NULL			},
+    {   "on_halt",                              NULL                    },
     {	"drv_ev_state_grow",			NULL,   		},
     {	"drv_ev_state",				"address"		},
     {	"safe_hash",				"address"		},
@@ -145,7 +150,6 @@ static erts_lc_lock_order_t erts_lock_order[] = {
     {	"instr_x",				NULL			},
     {	"instr",				NULL			},
     {   "dyn_lock_check",                       NULL                    },
-    {   "nif_load",                             NULL                    },
     {	"alcu_allocator",			"index"			},
     {	"mseg",					NULL			},
     {	"get_time",				NULL			},
@@ -427,7 +431,7 @@ new_locked_lock(lc_thread_t* thr,
 }
 
 static void
-raw_print_lock(char *prefix, Sint16 id, Wterm extra, erts_lock_flags_t flags,
+raw_print_lock(char *prefix, Sint16 id, Eterm extra, erts_lock_flags_t flags,
 	       const char* file, unsigned int line, char *suffix)
 {
     char *lname = (1 <= id && id < ERTS_LOCK_ORDER_SIZE
@@ -448,7 +452,7 @@ raw_print_lock(char *prefix, Sint16 id, Wterm extra, erts_lock_flags_t flags,
 }
 
 static void
-print_lock2(char *prefix, Sint16 id, Wterm extra, erts_lock_flags_t flags, char *suffix)
+print_lock2(char *prefix, Sint16 id, Eterm extra, erts_lock_flags_t flags, char *suffix)
 {
   raw_print_lock(prefix, id, extra, flags, NULL, 0, suffix);
 }
