@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  * 
- * Copyright Ericsson AB 1999-2016. All Rights Reserved.
+ * Copyright Ericsson AB 1999-2024. All Rights Reserved.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,7 +20,9 @@
 
 #ifndef _ERL_PROCESS_DICT_H
 #define _ERL_PROCESS_DICT_H
+
 #include "sys.h"
+#include "erl_term_hashing.h"
 
 typedef struct proc_dict {
     unsigned int sizeMask;
@@ -40,10 +42,11 @@ void erts_erase_dicts(struct process *p);
 void erts_dictionary_dump(fmtfn_t to, void *to_arg, ProcDict *pd);
 void erts_deep_dictionary_dump(fmtfn_t to, void *to_arg,
 			       ProcDict* pd, void (*cb)(fmtfn_t, void *, Eterm obj));
-Eterm erts_dictionary_copy(struct process *p, ProcDict *pd);
+Eterm erts_dictionary_copy(ErtsHeapFactory *hfact, ProcDict *pd, Uint reserve_size);
 
 Eterm erts_pd_hash_get(struct process *p, Eterm id);
-Uint32 erts_pd_make_hx(Eterm key);
-Eterm erts_pd_hash_get_with_hx(Process *p, Uint32 hx, Eterm id);
+
+erts_ihash_t erts_pd_make_hx(Eterm key);
+Eterm erts_pd_hash_get_with_hx(Process *p, erts_ihash_t hx, Eterm id);
 
 #endif
