@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2011-2018. All Rights Reserved.
+ * Copyright Ericsson AB 2011-2021. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -127,6 +127,7 @@ void erts_thr_progress_init(int no_schedulers, int managed, int unmanaged);
 ErtsThrPrgrData *erts_thr_progress_register_managed_thread(
     ErtsSchedulerData *esdp, ErtsThrPrgrCallbacks *, int, int);
 void erts_thr_progress_register_unmanaged_thread(ErtsThrPrgrCallbacks *);
+void erts_thr_progress_unregister_unmanaged_thread(void);
 void erts_thr_progress_active(ErtsThrPrgrData *, int on);
 void erts_thr_progress_wakeup(ErtsThrPrgrData *,
 			      ErtsThrPrgrVal value);
@@ -156,7 +157,7 @@ ERTS_GLB_INLINE ErtsThrPrgrVal erts_thr_progress_current_to_later__(ErtsThrPrgrV
 ERTS_GLB_INLINE ErtsThrPrgrVal erts_thr_progress_later(ErtsSchedulerData *);
 ERTS_GLB_INLINE ErtsThrPrgrVal erts_thr_progress_current(void);
 ERTS_GLB_INLINE int erts_thr_progress_has_passed__(ErtsThrPrgrVal val1, ErtsThrPrgrVal val2);
-ERTS_GLB_INLINE int erts_thr_progress_has_reached_this(ErtsThrPrgrVal this, ErtsThrPrgrVal val);
+ERTS_GLB_INLINE int erts_thr_progress_has_reached_this(ErtsThrPrgrVal this_, ErtsThrPrgrVal val);
 ERTS_GLB_INLINE int erts_thr_progress_equal(ErtsThrPrgrVal val1,
 					    ErtsThrPrgrVal val2);
 ERTS_GLB_INLINE int erts_thr_progress_cmp(ErtsThrPrgrVal val1, ErtsThrPrgrVal val2);
@@ -194,7 +195,7 @@ erts_thr_prgr_read_mb__(ERTS_THR_PRGR_ATOMIC *atmc)
 ERTS_GLB_INLINE int
 erts_thr_progress_is_managed_thread(void)
 {
-    ErtsThrPrgrData *tpd = erts_tsd_get(erts_thr_prgr_data_key__);
+    ErtsThrPrgrData *tpd = (ErtsThrPrgrData*)erts_tsd_get(erts_thr_prgr_data_key__);
     return tpd && tpd->is_managed;
 }
 
@@ -221,7 +222,7 @@ erts_thr_progress_unmanaged_continue(ErtsThrPrgrDelayHandle handle)
 ERTS_GLB_INLINE int
 erts_thr_progress_lc_is_delaying(void)
 {
-    ErtsThrPrgrData *tpd = erts_tsd_get(erts_thr_prgr_data_key__);
+    ErtsThrPrgrData *tpd = (ErtsThrPrgrData *)erts_tsd_get(erts_thr_prgr_data_key__);
     return tpd && tpd->is_delaying;
 }
 
@@ -250,7 +251,7 @@ erts_thr_progress_later(ErtsSchedulerData *esdp)
 	ERTS_THR_MEMORY_BARRIER;
     }
     else {
-	tpd = erts_tsd_get(erts_thr_prgr_data_key__);
+	tpd = (ErtsThrPrgrData *)erts_tsd_get(erts_thr_prgr_data_key__);
 	if (tpd && tpd->is_managed)
 	    goto managed_thread;
 	val = erts_thr_prgr_read_mb__(&erts_thr_prgr__.current);
@@ -287,11 +288,11 @@ erts_thr_progress_has_passed__(ErtsThrPrgrVal val1, ErtsThrPrgrVal val0)
 }
 
 ERTS_GLB_INLINE int
-erts_thr_progress_has_reached_this(ErtsThrPrgrVal this, ErtsThrPrgrVal val)
+erts_thr_progress_has_reached_this(ErtsThrPrgrVal this_, ErtsThrPrgrVal val)
 {
-    if (this == val)
+    if (this_ == val)
 	return 1;
-    return erts_thr_progress_has_passed__(this, val);
+    return erts_thr_progress_has_passed__(this_, val);
 }
 
 ERTS_GLB_INLINE int
