@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1996-2022. All Rights Reserved.
+ * Copyright Ericsson AB 1996-2023. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -53,6 +53,8 @@
 #define MAX_REG      1024            /* Max number of x(N) registers used */
 #define REG_MASK     (MAX_REG - 1)
 
+#define MAX_BIF_ARITY 4 /* Maximum allowed arguments for C-based BIFs. */
+
 /*
  * Guard BIFs and the new trapping length/1 implementation need 3 extra
  * registers in the register array.
@@ -298,8 +300,8 @@ extern void** beam_ops;
 
 #ifndef BEAMASM
 
-#define BeamIsReturnTimeTrace(w) \
-    BeamIsOpCode(*(const BeamInstr*)(w), op_i_return_time_trace)
+#define BeamIsReturnCallAccTrace(w) \
+    BeamIsOpCode(*(const BeamInstr*)(w), op_i_call_trace_return)
 #define BeamIsReturnToTrace(w) \
     BeamIsOpCode(*(const BeamInstr*)(w), op_i_return_to_trace)
 #define BeamIsReturnTrace(w) \
@@ -307,8 +309,8 @@ extern void** beam_ops;
 
 #else /* BEAMASM */
 
-#define BeamIsReturnTimeTrace(w) \
-    ((w) == beam_return_time_trace)
+#define BeamIsReturnCallAccTrace(w) \
+    ((w) == beam_call_trace_return)
 #define BeamIsReturnToTrace(w) \
     ((w) == beam_return_to_trace)
 #define BeamIsReturnTrace(w) \
@@ -316,9 +318,14 @@ extern void** beam_ops;
 
 #endif /* BEAMASM */
 
+/* Stack frame sizes (not including CP_SIZE) */
+#define BEAM_RETURN_CALL_ACC_TRACE_FRAME_SZ 3
+#define BEAM_RETURN_TO_TRACE_FRAME_SZ   1
+#define BEAM_RETURN_TRACE_FRAME_SZ      3
+
 #if ERTS_GLB_INLINE_INCL_FUNC_DEF
 ERTS_GLB_INLINE
-int erts_cp_size()
+int erts_cp_size(void)
 {
     if (erts_frame_layout == ERTS_FRAME_LAYOUT_RA) {
         return 1;
