@@ -30,6 +30,15 @@ extern "C"
 #include "erl_msacc.h"
 }
 
+void BeamModuleAssembler::ubif_comment(const ArgWord &Bif) {
+    if (logger.file()) {
+        ErtsCodeMFA *mfa = ubif2mfa((void *)Bif.get());
+        if (mfa) {
+            comment("UBIF: %T/%d", mfa->function, mfa->arity);
+        }
+    }
+}
+
 /* ARG2 = argument vector, ARG4 (!) = bif function pointer
  *
  * Result is returned in ARG1 (will be THE_NON_VALUE if the BIF call failed). */
@@ -100,10 +109,11 @@ void BeamModuleAssembler::emit_i_bif1(const ArgSource &Src1,
                                       const ArgLabel &Fail,
                                       const ArgWord &Bif,
                                       const ArgRegister &Dst) {
-    auto src1 = load_source(Src1, TMP1);
+    auto src1 = load_source(Src1);
 
     a.str(src1.reg, getXRef(0));
 
+    ubif_comment(Bif);
     emit_i_bif(Fail, Bif, Dst);
 }
 
@@ -116,6 +126,7 @@ void BeamModuleAssembler::emit_i_bif2(const ArgSource &Src1,
 
     a.stp(src1.reg, src2.reg, getXRef(0));
 
+    ubif_comment(Bif);
     emit_i_bif(Fail, Bif, Dst);
 }
 
@@ -131,6 +142,7 @@ void BeamModuleAssembler::emit_i_bif3(const ArgSource &Src1,
     a.stp(src1.reg, src2.reg, getXRef(0));
     a.str(src3.reg, getXRef(2));
 
+    ubif_comment(Bif);
     emit_i_bif(Fail, Bif, Dst);
 }
 
@@ -157,10 +169,11 @@ void BeamModuleAssembler::emit_i_bif(const ArgLabel &Fail,
 void BeamModuleAssembler::emit_nofail_bif1(const ArgSource &Src1,
                                            const ArgWord &Bif,
                                            const ArgRegister &Dst) {
-    auto src1 = load_source(Src1, TMP1);
+    auto src1 = load_source(Src1);
 
     a.str(src1.reg, getXRef(0));
 
+    ubif_comment(Bif);
     mov_arg(ARG4, Bif);
     fragment_call(ga->get_i_bif_guard_shared());
     mov_arg(Dst, ARG1);
@@ -174,6 +187,7 @@ void BeamModuleAssembler::emit_nofail_bif2(const ArgSource &Src1,
 
     a.stp(src1.reg, src2.reg, getXRef(0));
 
+    ubif_comment(Bif);
     mov_arg(ARG4, Bif);
     fragment_call(ga->get_i_bif_guard_shared());
     mov_arg(Dst, ARG1);
@@ -182,20 +196,29 @@ void BeamModuleAssembler::emit_nofail_bif2(const ArgSource &Src1,
 void BeamModuleAssembler::emit_i_length_setup(const ArgLabel &Fail,
                                               const ArgWord &Live,
                                               const ArgSource &Src) {
-    mov_arg(TMP1, Src);
-    mov_imm(TMP2, make_small(0));
-
     /* Store trap state after the currently live registers. There are
      * 3 extra registers beyond the ordinary ones that we're free to
      * use for whatever purpose. */
     ERTS_CT_ASSERT(ERTS_X_REGS_ALLOCATED - MAX_REG >= 3);
-    mov_arg(ArgXRegister(Live.get() + 0), TMP1);
-    mov_arg(ArgXRegister(Live.get() + 1), TMP2);
+    auto trap_reg1 = ArgXRegister(Live.get() + 0);
+    auto trap_reg2 = ArgXRegister(Live.get() + 1);
+    auto trap_reg3 = ArgXRegister(Live.get() + 2);
 
-    /* Store original argument. This is only needed for exceptions and can be
-     * safely skipped in guards. */
-    if (Fail.get() == 0) {
-        mov_arg(ArgXRegister(Live.get() + 2), TMP1);
+    auto src = load_source(Src, TMP1);
+    auto dst1 = init_destination(trap_reg1, src.reg);
+    auto dst2 = init_destination(trap_reg2, TMP2);
+
+    mov_imm(dst2.reg, make_small(0));
+    mov_var(dst1, src);
+
+    /* Store original argument. This is only needed for exceptions and
+     * can be safely skipped in guards. */
+    if (Fail.get() != 0) {
+        flush_vars(dst1, dst2);
+    } else {
+        auto dst3 = init_destination(trap_reg3, src.reg);
+        mov_var(dst3, src);
+        flush_vars(dst1, dst2, dst3);
     }
 }
 
@@ -240,7 +263,7 @@ void BeamGlobalAssembler::emit_i_length_common(Label fail, int state_size) {
         a.add(ARG2, ARG2, imm(state_size));
 
         a.str(ZERO, arm::Mem(c_p, offsetof(Process, current)));
-        a.str(ARG2, arm::Mem(c_p, offsetof(Process, arity)));
+        a.strb(ARG2.w(), arm::Mem(c_p, offsetof(Process, arity)));
 
         /* We'll find our way back through the entry address (ARG3). */
         a.b(labels[context_switch_simplified]);
@@ -319,6 +342,7 @@ static Eterm debug_call_light_bif(Process *c_p,
                                   ErtsBifFunc vbf) {
     Eterm result;
 
+    ERTS_ASSERT_TRACER_REFS(&c_p->common);
     ERTS_UNREQ_PROC_MAIN_LOCK(c_p);
     {
         ERTS_CHK_MBUF_SZ(c_p);
@@ -332,6 +356,7 @@ static Eterm debug_call_light_bif(Process *c_p,
     }
     PROCESS_MAIN_CHK_LOCKS(c_p);
     ERTS_REQ_PROC_MAIN_LOCK(c_p);
+    ERTS_ASSERT_TRACER_REFS(&c_p->common);
 
     return result;
 }
@@ -346,9 +371,6 @@ static Eterm debug_call_light_bif(Process *c_p,
  * ARG8 = BIF pointer
  */
 void BeamGlobalAssembler::emit_call_light_bif_shared() {
-    /* We use the HTOP, FCALLS, and XREG1 registers as they are not
-     * used on the runtime-stack and are caller save. */
-
     arm::Mem entry_mem = TMP_MEM1q, export_mem = TMP_MEM2q,
              mbuf_mem = TMP_MEM3q;
 
@@ -376,7 +398,7 @@ void BeamGlobalAssembler::emit_call_light_bif_shared() {
 
         emit_enter_runtime_frame();
         emit_enter_runtime<Update::eReductions | Update::eStack |
-                           Update::eHeap | Update::eXRegs>();
+                           Update::eHeap | Update::eXRegs>(MAX_BIF_ARITY);
 
 #ifdef ERTS_MSACC_EXTENDED_STATES
         {
@@ -449,7 +471,8 @@ void BeamGlobalAssembler::emit_call_light_bif_shared() {
          * after seeing a later timestamp from its own call to
          * erlang:monotonic_time/0. */
         emit_leave_runtime<Update::eReductions | Update::eCodeIndex |
-                           Update::eHeap | Update::eStack | Update::eXRegs>();
+                           Update::eHeap | Update::eStack | Update::eXRegs>(
+                MAX_BIF_ARITY);
         emit_leave_runtime_frame();
 
         /* ERTS_IS_GC_DESIRED_INTERNAL */
@@ -517,7 +540,7 @@ void BeamGlobalAssembler::emit_call_light_bif_shared() {
         {
             emit_enter_runtime_frame();
             emit_enter_runtime<Update::eReductions | Update::eStack |
-                               Update::eHeap | Update::eXRegs>();
+                               Update::eHeap | Update::eXRegs>(MAX_BIF_ARITY);
 
             a.mov(ARG3, ARG1);
 
@@ -525,11 +548,11 @@ void BeamGlobalAssembler::emit_call_light_bif_shared() {
             a.ldr(ARG2, mbuf_mem);
             load_x_reg_array(ARG4);
             a.ldr(ARG5, export_mem);
-            a.ldr(ARG5, arm::Mem(ARG5, offsetof(Export, info.mfa.arity)));
+            a.ldrb(ARG5.w(), arm::Mem(ARG5, offsetof(Export, info.mfa.arity)));
             runtime_call<5>(erts_gc_after_bif_call_lhf);
 
             emit_leave_runtime<Update::eReductions | Update::eStack |
-                               Update::eHeap | Update::eXRegs>();
+                               Update::eHeap | Update::eXRegs>(MAX_BIF_ARITY);
             emit_leave_runtime_frame();
 
             a.b(check_bif_return);
@@ -544,9 +567,9 @@ void BeamGlobalAssembler::emit_call_light_bif_shared() {
 
     a.bind(yield);
     {
-        a.ldr(ARG2, arm::Mem(ARG4, offsetof(Export, info.mfa.arity)));
+        a.ldrb(ARG2.w(), arm::Mem(ARG4, offsetof(Export, info.mfa.arity)));
         lea(ARG4, arm::Mem(ARG4, offsetof(Export, info.mfa)));
-        a.str(ARG2, arm::Mem(c_p, offsetof(Process, arity)));
+        a.strb(ARG2.w(), arm::Mem(c_p, offsetof(Process, arity)));
         a.str(ARG4, arm::Mem(c_p, offsetof(Process, current)));
 
         /* We'll find our way back through ARG3 (entry address). */
@@ -557,6 +580,7 @@ void BeamGlobalAssembler::emit_call_light_bif_shared() {
 void BeamModuleAssembler::emit_call_light_bif(const ArgWord &Bif,
                                               const ArgExport &Exp) {
     Label entry = a.newLabel();
+    BeamFile_ImportEntry *e = &beam->imports.entries[Exp.get()];
 
     a.bind(entry);
 
@@ -564,6 +588,9 @@ void BeamModuleAssembler::emit_call_light_bif(const ArgWord &Bif,
     mov_arg(ARG8, Bif);
     a.adr(ARG3, entry);
 
+    if (logger.file()) {
+        comment("BIF: %T:%T/%d", e->module, e->function, e->arity);
+    }
     fragment_call(ga->get_call_light_bif_shared());
 }
 
@@ -621,6 +648,12 @@ void BeamGlobalAssembler::emit_bif_nif_epilogue(void) {
     a.mov(XREG0, ARG1);
 
     emit_leave_erlang_frame();
+
+    if (erts_alcu_enable_code_atags) {
+        /* See emit_i_test_yield. */
+        a.str(a64::x30, arm::Mem(c_p, offsetof(Process, i)));
+    }
+
     a.ret(a64::x30);
 
     a.bind(check_trap);
@@ -683,8 +716,8 @@ void BeamGlobalAssembler::emit_call_bif_shared(void) {
     emit_enter_runtime_frame();
     a.str(ARG2, arm::Mem(c_p, offsetof(Process, current)));
     /* `call_bif` wants arity in ARG5. */
-    a.ldr(ARG5, arm::Mem(ARG2, offsetof(ErtsCodeMFA, arity)));
-    a.str(ARG5, arm::Mem(c_p, offsetof(Process, arity)));
+    a.ldr(ARG5.w(), arm::Mem(ARG2, offsetof(ErtsCodeMFA, arity)));
+    a.strb(ARG5.w(), arm::Mem(c_p, offsetof(Process, arity)));
     a.str(ARG3, arm::Mem(c_p, offsetof(Process, i)));
 
     /* The corresponding leave can be found in the epilogue. */
@@ -776,6 +809,10 @@ void BeamModuleAssembler::emit_call_bif_mfa(const ArgAtom &M,
 
     a.adr(ARG3, current_label);
     a.sub(ARG2, ARG3, imm(sizeof(ErtsCodeMFA)));
+    comment("HBIF: %T:%T/%d",
+            e->info.mfa.module,
+            e->info.mfa.function,
+            A.get());
     a.mov(ARG4, imm(func));
 
     a.b(resolve_fragment(ga->get_call_bif_shared(), disp128MB));
@@ -860,6 +897,11 @@ void BeamGlobalAssembler::emit_dispatch_nif(void) {
 void BeamGlobalAssembler::emit_call_nif_yield_helper() {
     Label yield = a.newLabel();
 
+    if (erts_alcu_enable_code_atags) {
+        /* See emit_i_test_yield. */
+        a.str(ARG3, arm::Mem(c_p, offsetof(Process, i)));
+    }
+
     a.subs(FCALLS, FCALLS, imm(1));
     a.b_le(yield);
     a.b(labels[call_nif_shared]);
@@ -869,8 +911,8 @@ void BeamGlobalAssembler::emit_call_nif_yield_helper() {
         int mfa_offset = sizeof(ErtsCodeMFA);
         int arity_offset = offsetof(ErtsCodeMFA, arity) - mfa_offset;
 
-        a.ldur(TMP1, arm::Mem(ARG3, arity_offset));
-        a.str(TMP1, arm::Mem(c_p, offsetof(Process, arity)));
+        a.ldur(TMP1.w(), arm::Mem(ARG3, arity_offset));
+        a.strb(TMP1.w(), arm::Mem(c_p, offsetof(Process, arity)));
 
         a.sub(TMP1, ARG3, imm(mfa_offset));
         a.str(TMP1, arm::Mem(c_p, offsetof(Process, current)));
@@ -968,14 +1010,14 @@ void BeamModuleAssembler::emit_i_load_nif() {
 
     a.bind(entry);
 
-    emit_enter_runtime<Update::eStack | Update::eHeap | Update::eXRegs>(2);
+    emit_enter_runtime<Update::eHeapAlloc | Update::eXRegs>(2);
 
     a.mov(ARG1, c_p);
     a.adr(ARG2, current_label);
     load_x_reg_array(ARG3);
     runtime_call<3>(beam_jit_load_nif);
 
-    emit_leave_runtime<Update::eStack | Update::eHeap | Update::eXRegs>(2);
+    emit_leave_runtime<Update::eHeapAlloc | Update::eXRegs>(2);
 
     a.cmp(ARG1, imm(RET_NIF_yield));
     a.b_eq(schedule);
