@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2020-2021. All Rights Reserved.
+ * Copyright Ericsson AB 2020-2024. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -71,7 +71,7 @@ struct emulator_info {
     const void *normal_exit;
 };
 
-struct module_info {
+struct erlang_module_info {
     uint64_t base_address;
     uint32_t range_count;
     uint32_t code_size;
@@ -106,7 +106,7 @@ struct debug_info {
     enum debug_info_header header;
     union {
         struct emulator_info emu;
-        struct module_info mod;
+        struct erlang_module_info mod;
     } payload;
 };
 
@@ -131,35 +131,31 @@ static void beamasm_init_late_gdb() {
     entry->symfile_size = sizeof(struct debug_info);
 
     /* Insert into linked list */
-    entry->next_entry = __jit_debug_descriptor.first_entry;
-    if (entry->next_entry) {
-        entry->next_entry->prev_entry = entry;
-    } else {
-        entry->prev_entry = nullptr;
+    erts_mtx_lock(&__jit_debug_descriptor.mutex);
+
+    if (__jit_debug_descriptor.first_entry) {
+        ASSERT(__jit_debug_descriptor.first_entry->prev_entry == nullptr);
+        __jit_debug_descriptor.first_entry->prev_entry = entry;
     }
 
-    /* Insert into linked list */
-    erts_mtx_lock(&__jit_debug_descriptor.mutex);
+    entry->prev_entry = nullptr;
     entry->next_entry = __jit_debug_descriptor.first_entry;
-    if (entry->next_entry) {
-        entry->next_entry->prev_entry = entry;
-    } else {
-        entry->prev_entry = nullptr;
-    }
+    __jit_debug_descriptor.first_entry = entry;
 
     /* Register with gdb */
+    ASSERT(__jit_debug_descriptor.relevant_entry == nullptr);
     __jit_debug_descriptor.action_flag = JIT_REGISTER_FN;
-    __jit_debug_descriptor.first_entry = entry;
     __jit_debug_descriptor.relevant_entry = entry;
     __jit_debug_register_code();
+    __jit_debug_descriptor.relevant_entry = nullptr;
+
     erts_mtx_unlock(&__jit_debug_descriptor.mutex);
 }
 
-static void beamasm_update_gdb_info(
-        std::string module_name,
-        ErtsCodePtr base_address,
-        size_t code_size,
-        const std::vector<BeamAssembler::AsmRange> &ranges) {
+static void *beamasm_insert_gdb_info(std::string module_name,
+                                     ErtsCodePtr base_address,
+                                     size_t code_size,
+                                     const std::vector<AsmRange> &ranges) {
     Sint symfile_size = sizeof(struct debug_info) + module_name.size() + 1;
 
     for (const auto &range : ranges) {
@@ -222,21 +218,58 @@ static void beamasm_update_gdb_info(
 
     /* Insert into linked list */
     erts_mtx_lock(&__jit_debug_descriptor.mutex);
-    entry->next_entry = __jit_debug_descriptor.first_entry;
-    if (entry->next_entry) {
-        entry->next_entry->prev_entry = entry;
-    } else {
-        entry->prev_entry = nullptr;
+
+    if (__jit_debug_descriptor.first_entry) {
+        ASSERT(__jit_debug_descriptor.first_entry->prev_entry == nullptr);
+        __jit_debug_descriptor.first_entry->prev_entry = entry;
     }
 
-    /* register with gdb */
-    __jit_debug_descriptor.action_flag = JIT_REGISTER_FN;
+    entry->prev_entry = nullptr;
+    entry->next_entry = __jit_debug_descriptor.first_entry;
     __jit_debug_descriptor.first_entry = entry;
+
+    /* register with gdb */
+    ASSERT(__jit_debug_descriptor.relevant_entry == nullptr);
+    __jit_debug_descriptor.action_flag = JIT_REGISTER_FN;
     __jit_debug_descriptor.relevant_entry = entry;
     __jit_debug_register_code();
+    __jit_debug_descriptor.relevant_entry = nullptr;
+
     erts_mtx_unlock(&__jit_debug_descriptor.mutex);
+
+    /* Return a reference to our debugger entry so we can remove it when
+     * unloading. */
+    return entry;
 }
 
+static void beamasm_delete_gdb_info(void *metadata) {
+    jit_code_entry *entry = (jit_code_entry *)metadata;
+
+    erts_mtx_lock(&__jit_debug_descriptor.mutex);
+
+    /* Unlink current module from the entry list. */
+    if (entry->prev_entry) {
+        entry->prev_entry->next_entry = entry->next_entry;
+    } else {
+        __jit_debug_descriptor.first_entry = entry->next_entry;
+    }
+
+    if (entry->next_entry) {
+        entry->next_entry->prev_entry = entry->prev_entry;
+    }
+
+    /* unregister with gdb  */
+    ASSERT(__jit_debug_descriptor.relevant_entry == nullptr);
+    __jit_debug_descriptor.action_flag = JIT_UNREGISTER_FN;
+    __jit_debug_descriptor.relevant_entry = entry;
+    __jit_debug_register_code();
+    __jit_debug_descriptor.relevant_entry = nullptr;
+
+    erts_mtx_unlock(&__jit_debug_descriptor.mutex);
+
+    free((void *)entry->symfile_addr);
+    free(entry);
+}
 #endif /* HAVE_GDB_SUPPORT */
 
 #ifdef HAVE_LINUX_PERF_SUPPORT
@@ -336,7 +369,7 @@ public:
         return true;
     }
 
-    void update(const std::vector<BeamAssembler::AsmRange> &ranges) {
+    void update(const std::vector<AsmRange> &ranges) {
         struct JitCodeLoadRecord {
             RecordHeader header;
             Uint32 pid;
@@ -353,7 +386,7 @@ public:
         record.pid = getpid();
         record.tid = erts_thr_self();
 
-        for (const BeamAssembler::AsmRange &range : ranges) {
+        for (const AsmRange &range : ranges) {
             /* Line entries must be written first, if present. */
             if (!range.lines.empty()) {
                 struct JitCodeDebugEntry {
@@ -434,7 +467,12 @@ class JitPerfMap {
 public:
     bool init() {
         char name[MAXPATHLEN];
-        snprintf(name, sizeof(name), "/tmp/perf-%i.map", getpid());
+        size_t namesz = sizeof(name);
+
+        if (erts_sys_explicit_host_getenv("ERL_SYM_MAP_FILE", name, &namesz) !=
+            1) {
+            snprintf(name, sizeof(name), "/tmp/perf-%i.map", getpid());
+        }
         file = fopen(name, "w");
         if (!file) {
             int saved_errno = errno;
@@ -449,7 +487,7 @@ public:
         return true;
     }
 
-    void update(const std::vector<BeamAssembler::AsmRange> &ranges) {
+    void update(const std::vector<AsmRange> &ranges) {
         for (const auto &range : ranges) {
             char *start = (char *)range.start, *stop = (char *)range.stop;
             ptrdiff_t size = stop - start;
@@ -491,7 +529,7 @@ public:
                               ERTS_LOCK_FLAGS_CATEGORY_GENERIC);
     }
 
-    void update(const std::vector<BeamAssembler::AsmRange> &ranges) {
+    void update(const std::vector<AsmRange> &ranges) {
         if (modes) {
             erts_mtx_lock(&mutex);
 #    ifdef HAVE_LINUX_PERF_DUMP_SUPPORT
@@ -526,16 +564,26 @@ void beamasm_metadata_late_init() {
 #endif
 }
 
-void beamasm_metadata_update(
-        std::string module_name,
-        ErtsCodePtr base_address,
-        size_t code_size,
-        const std::vector<BeamAssembler::AsmRange> &ranges) {
+void *beamasm_metadata_insert(std::string module_name,
+                              ErtsCodePtr base_address,
+                              size_t code_size,
+                              const std::vector<AsmRange> &ranges) {
 #ifdef HAVE_LINUX_PERF_SUPPORT
     perf.update(ranges);
 #endif
 
 #ifdef HAVE_GDB_SUPPORT
-    beamasm_update_gdb_info(module_name, base_address, code_size, ranges);
+    return beamasm_insert_gdb_info(module_name,
+                                   base_address,
+                                   code_size,
+                                   ranges);
+#else
+    return NULL;
+#endif
+}
+
+void beamasm_unregister_metadata(void *metadata) {
+#ifdef HAVE_GDB_SUPPORT
+    beamasm_delete_gdb_info(metadata);
 #endif
 }
