@@ -25,58 +25,58 @@ using namespace asmjit;
 extern "C"
 {
 #include "erl_map.h"
+#include "erl_term_hashing.h"
 #include "beam_common.h"
 }
 
-static const Uint32 INTERNAL_HASH_SALT = 3432918353;
-static const Uint32 HCONST = 0x9E3779B9;
-
-/*
- * ARG4 = lower 32
- * ARG5 = upper 32
+/* ARG2 = term
  *
- * Helper function for calculating the internal hash of keys before looking
- * them up in a map.
+ * Helper for calculating the internal hash of keys before looking them up in a
+ * map. This is a manual expansion of `erts_internal_hash`, and all changes to
+ * that function must be mirrored here.
  *
- * This is essentially just a manual expansion of the `UINT32_HASH_2` macro.
- * Whenever the internal hash algorithm is updated, this and all of its users
- * must follow suit.
- *
- * Result is returned in ARG3. */
+ * Result in ARG3. */
 void BeamGlobalAssembler::emit_internal_hash_helper() {
-    x86::Gp hash = ARG3d, lower = ARG4d, upper = ARG5d;
+    x86::Gp key = ARG2, key_hash = ARG3;
 
-    a.mov(hash, imm(INTERNAL_HASH_SALT));
-    a.add(lower, imm(HCONST));
-    a.add(upper, imm(HCONST));
+    /* Unsigned multiplication instructions on x86 either use RDX as an
+     * implicit source or clobber it. Sigh. */
+    if (key == x86::rdx) {
+        a.mov(TMP_MEM1q, x86::rdx);
+    } else {
+        ASSERT(key_hash == x86::rdx);
+    }
 
-    using rounds =
-            std::initializer_list<std::tuple<x86::Gp, x86::Gp, x86::Gp, int>>;
-    for (const auto &round : rounds{{lower, upper, hash, 13},
-                                    {upper, hash, lower, -8},
-                                    {hash, lower, upper, 13},
-                                    {lower, upper, hash, 12},
-                                    {upper, hash, lower, -16},
-                                    {hash, lower, upper, 5},
-                                    {lower, upper, hash, 3},
-                                    {upper, hash, lower, -10},
-                                    {hash, lower, upper, 15}}) {
-        const auto &[r_a, r_b, r_c, shift] = round;
+    /* key_hash = key ^ (key >> 33); */
+    a.mov(ARG4, ARG2);
+    a.shr(ARG4, imm(33));
+    a.mov(x86::rdx, ARG2);
+    a.xor_(x86::rdx, ARG4);
 
-        a.sub(r_a, r_b);
-        a.sub(r_a, r_c);
+    /* `RDX * ARG6` storing a 128 bit result in ARG4:RDX. We only want the
+     * lower 64 bits in RDX.
+     *
+     * key_hash *= 0xFF51AFD7ED558CCDull */
+    mov_imm(ARG6, 0xFF51AFD7ED558CCDull);
+    a.mulx(ARG4, x86::rdx, ARG6);
 
-        /* We have no use for the type constant anymore, reuse its register for
-         * the `a ^= r_c << shift` expression. */
-        a.mov(ARG6d, r_c);
+    /* key_hash ^= key_hash >> 33; */
+    a.mov(ARG4, x86::rdx);
+    a.shr(ARG4, imm(33));
+    a.xor_(x86::rdx, ARG4);
 
-        if (shift > 0) {
-            a.shr(ARG6d, imm(shift));
-        } else {
-            a.shl(ARG6d, imm(-shift));
-        }
+    /* key_hash *= 0xC4CEB9FE1A85EC53ull */
+    mov_imm(ARG6, 0xC4CEB9FE1A85EC53ull);
+    a.mulx(ARG4, x86::rdx, ARG6);
 
-        a.xor_(r_a, ARG6d);
+    /* key_hash ^= key_hash >> 33; */
+    a.mov(ARG4, x86::rdx);
+    a.shr(ARG4, imm(33));
+    a.xor_(x86::rdx, ARG4);
+
+    if (key == x86::rdx) {
+        a.mov(key_hash, x86::rdx);
+        a.mov(key, TMP_MEM1q);
     }
 
 #ifdef DBG_HASHMAP_COLLISION_BONANZA
@@ -89,14 +89,12 @@ void BeamGlobalAssembler::emit_internal_hash_helper() {
     runtime_call<2>(erts_dbg_hashmap_collision_bonanza);
     emit_leave_runtime();
 
-    a.mov(ARG3d, RETd);
+    a.mov(ARG3, RET);
 
     a.mov(ARG1, TMP_MEM1q);
     a.mov(ARG2, TMP_MEM2q);
     a.mov(RET, TMP_MEM3q);
 #endif
-
-    a.ret();
 }
 
 /* ARG1 = hash map root, ARG2 = key, ARG3 = key hash, RETd = node header
@@ -105,7 +103,7 @@ void BeamGlobalAssembler::emit_internal_hash_helper() {
 void BeamGlobalAssembler::emit_hashmap_get_element() {
     Label node_loop = a.newLabel();
 
-    x86::Gp node = ARG1, key = ARG2, key_hash = ARG3d, header_val = RETd,
+    x86::Gp node = ARG1, key = ARG2, key_hash = ARG3, header_val = RETd,
             index = ARG4d, depth = ARG5d;
 
     const int header_shift =
@@ -123,7 +121,7 @@ void BeamGlobalAssembler::emit_hashmap_get_element() {
 
         /* Find out which child we should follow, and shift the hash for the
          * next round. */
-        a.mov(index, key_hash);
+        a.mov(index, key_hash.r32());
         a.and_(index, imm(0xF));
         a.shr(key_hash, imm(4));
         a.inc(depth);
@@ -157,13 +155,13 @@ void BeamGlobalAssembler::emit_hashmap_get_element() {
         emit_ptr_val(node, node);
 
         /* Have we found our leaf? */
-        a.test(node.r32(), imm(_TAG_PRIMARY_MASK - TAG_PRIMARY_LIST));
+        a.test(node.r8(), imm(_TAG_PRIMARY_MASK - TAG_PRIMARY_LIST));
         a.short_().je(leaf_node);
 
         /* Nope, we have to search another node. */
         a.mov(header_val, emit_boxed_val(node, 0, sizeof(Uint32)));
 
-        /* After 8 nodes we've run out of the 32 bits we started with
+        /* After 8/16 nodes we've run out of the hash bits we've started with
          * and we end up in a collision node. */
         a.test(depth, imm(HAMT_MAX_LEVEL - 1));
         a.short_().jnz(node_loop);
@@ -238,13 +236,13 @@ void BeamGlobalAssembler::emit_flatmap_get_element() {
 
 void BeamGlobalAssembler::emit_new_map_shared() {
     emit_enter_frame();
-    emit_enter_runtime<Update::eReductions | Update::eStack | Update::eHeap>();
+    emit_enter_runtime<Update::eReductions | Update::eHeapAlloc>();
 
     a.mov(ARG1, c_p);
     load_x_reg_array(ARG2);
     runtime_call<5>(erts_gc_new_map);
 
-    emit_leave_runtime<Update::eReductions | Update::eStack | Update::eHeap>();
+    emit_leave_runtime<Update::eReductions | Update::eHeapAlloc>();
     emit_leave_frame();
 
     a.ret();
@@ -266,37 +264,80 @@ void BeamModuleAssembler::emit_new_map(const ArgRegister &Dst,
     mov_arg(Dst, RET);
 }
 
-void BeamGlobalAssembler::emit_i_new_small_map_lit_shared() {
-    emit_enter_frame();
-    emit_enter_runtime<Update::eReductions | Update::eStack | Update::eHeap>();
-
-    a.mov(ARG1, c_p);
-    load_x_reg_array(ARG2);
-    runtime_call<5>(erts_gc_new_small_map_lit);
-
-    emit_leave_runtime<Update::eReductions | Update::eStack | Update::eHeap>();
-    emit_leave_frame();
-
-    a.ret();
-}
-
 void BeamModuleAssembler::emit_i_new_small_map_lit(const ArgRegister &Dst,
                                                    const ArgWord &Live,
                                                    const ArgLiteral &Keys,
                                                    const ArgWord &Size,
                                                    const Span<ArgVal> &args) {
-    Label data = embed_vararg_rodata(args, CP_SIZE);
-
     ASSERT(Size.get() == args.size());
 
-    ASSERT(Keys.isLiteral());
-    mov_arg(ARG3, Keys);
-    mov_imm(ARG4, Live.get());
-    a.lea(ARG5, x86::qword_ptr(data));
+    emit_gc_test(ArgWord(0),
+                 ArgWord(args.size() + MAP_HEADER_FLATMAP_SZ + 1),
+                 Live);
 
-    fragment_call(ga->get_i_new_small_map_lit_shared());
+    std::vector<ArgVal> data;
+    data.reserve(args.size() + MAP_HEADER_FLATMAP_SZ + 1);
+    data.push_back(ArgWord(MAP_HEADER_FLATMAP));
+    data.push_back(Size);
+    data.push_back(Keys);
 
-    mov_arg(Dst, RET);
+    for (auto arg : args) {
+        data.push_back(arg);
+    }
+
+    size_t size = data.size();
+    unsigned i;
+
+    mov_arg(x86::qword_ptr(HTOP), data[0]);
+
+    /* Starting from 1 instead of 0 gives more opportunities for
+     * applying the MMX optimizations. */
+    for (i = 1; i < size - 1; i += 2) {
+        x86::Mem dst_ptr0 = x86::qword_ptr(HTOP, i * sizeof(Eterm));
+        x86::Mem dst_ptr1 = x86::qword_ptr(HTOP, (i + 1) * sizeof(Eterm));
+        auto first = data[i];
+        auto second = data[i + 1];
+
+        switch (ArgVal::memory_relation(first, second)) {
+        case ArgVal::consecutive: {
+            x86::Mem src_ptr = getArgRef(first, 16);
+
+            comment("(initializing two elements at once)");
+            dst_ptr0.setSize(16);
+            vmovups(x86::xmm0, src_ptr);
+            vmovups(dst_ptr0, x86::xmm0);
+            break;
+        }
+        case ArgVal::reverse_consecutive: {
+            if (!hasCpuFeature(CpuFeatures::X86::kAVX)) {
+                mov_arg(dst_ptr0, first);
+                mov_arg(dst_ptr1, second);
+            } else {
+                x86::Mem src_ptr = getArgRef(second, 16);
+
+                comment("(initializing with two swapped elements at once)");
+                dst_ptr0.setSize(16);
+                a.vpermilpd(x86::xmm0, src_ptr, 1); /* Load and swap */
+                a.vmovups(dst_ptr0, x86::xmm0);
+            }
+            break;
+        }
+        case ArgVal::none:
+            mov_arg(dst_ptr0, first);
+            mov_arg(dst_ptr1, second);
+            break;
+        }
+    }
+
+    if (i < size) {
+        x86::Mem dst_ptr = x86::qword_ptr(HTOP, i * sizeof(Eterm));
+        mov_arg(dst_ptr, data[i]);
+    }
+
+    a.lea(ARG1, x86::byte_ptr(HTOP, TAG_PRIMARY_BOXED));
+    a.add(HTOP, imm(size * sizeof(Eterm)));
+
+    mov_arg(Dst, ARG1);
 }
 
 /* ARG1 = map, ARG2 = key
@@ -324,9 +365,11 @@ void BeamGlobalAssembler::emit_i_get_map_element_shared() {
 
     a.bind(generic);
     {
+        emit_enter_frame();
         emit_enter_runtime();
         runtime_call<2>(get_map_element);
         emit_leave_runtime();
+        emit_leave_frame();
 
         emit_test_the_non_value(RET);
 
@@ -339,13 +382,9 @@ void BeamGlobalAssembler::emit_i_get_map_element_shared() {
 
     a.bind(hashmap);
     {
-        /* Calculate the internal hash of ARG2 before diving into the HAMT. */
-        a.mov(ARG5, ARG2);
-        a.shr(ARG5, imm(32));
-        a.mov(ARG4d, ARG2d);
-
-        a.call(labels[internal_hash_helper]);
-
+        /* Calculate the internal hash of the key before diving into the
+         * HAMT. */
+        emit_internal_hash_helper();
         emit_hashmap_get_element();
     }
 }
@@ -357,7 +396,7 @@ void BeamModuleAssembler::emit_i_get_map_element(const ArgLabel &Fail,
     mov_arg(ARG1, Src);
     mov_arg(ARG2, Key);
 
-    if (masked_types(Key, BEAM_TYPE_MASK_IMMEDIATE) != BEAM_TYPE_NONE &&
+    if (maybe_one_of<BeamTypeId::MaybeImmediate>(Key) &&
         hasCpuFeature(CpuFeatures::X86::kBMI2)) {
         safe_fragment_call(ga->get_i_get_map_element_shared());
         a.jne(resolve_beam_label(Fail));
@@ -523,13 +562,30 @@ void BeamModuleAssembler::emit_i_get_map_element_hash(const ArgLabel &Fail,
 /* ARG3 = live registers, ARG4 = update vector size, ARG5 = update vector. */
 void BeamGlobalAssembler::emit_update_map_assoc_shared() {
     emit_enter_frame();
-    emit_enter_runtime<Update::eReductions | Update::eStack | Update::eHeap>();
+    emit_enter_runtime<Update::eReductions | Update::eHeapAlloc>();
 
     a.mov(ARG1, c_p);
     load_x_reg_array(ARG2);
     runtime_call<5>(erts_gc_update_map_assoc);
 
-    emit_leave_runtime<Update::eReductions | Update::eStack | Update::eHeap>();
+    emit_leave_runtime<Update::eReductions | Update::eHeapAlloc>();
+    emit_leave_frame();
+
+    a.ret();
+}
+
+/* ARG2 = key
+ * ARG3 = value
+ * ARG4 = map
+ */
+void BeamGlobalAssembler::emit_update_map_single_assoc_shared() {
+    emit_enter_frame();
+    emit_enter_runtime<Update::eHeapAlloc>();
+
+    a.mov(ARG1, c_p);
+    runtime_call<4>(erts_maps_put);
+
+    emit_leave_runtime<Update::eHeapAlloc>();
     emit_leave_frame();
 
     a.ret();
@@ -540,16 +596,23 @@ void BeamModuleAssembler::emit_update_map_assoc(const ArgSource &Src,
                                                 const ArgWord &Live,
                                                 const ArgWord &Size,
                                                 const Span<ArgVal> &args) {
-    Label data = embed_vararg_rodata(args, CP_SIZE);
-
     ASSERT(Size.get() == args.size());
 
-    mov_arg(getXRef(Live.get()), Src);
+    if (args.size() == 2) {
+        mov_arg(ARG2, args[0]);
+        mov_arg(ARG3, args[1]);
+        mov_arg(ARG4, Src);
+        safe_fragment_call(ga->get_update_map_single_assoc_shared());
+    } else {
+        Label data = embed_vararg_rodata(args, CP_SIZE);
 
-    mov_imm(ARG3, Live.get());
-    mov_imm(ARG4, args.size());
-    a.lea(ARG5, x86::qword_ptr(data));
-    fragment_call(ga->get_update_map_assoc_shared());
+        mov_arg(getXRef(Live.get()), Src);
+
+        mov_imm(ARG3, Live.get());
+        mov_imm(ARG4, args.size());
+        a.lea(ARG5, x86::qword_ptr(data));
+        fragment_call(ga->get_update_map_assoc_shared());
+    }
 
     mov_arg(Dst, RET);
 }
@@ -559,13 +622,13 @@ void BeamModuleAssembler::emit_update_map_assoc(const ArgSource &Src,
  * Result is returned in RET, error is indicated by ZF. */
 void BeamGlobalAssembler::emit_update_map_exact_guard_shared() {
     emit_enter_frame();
-    emit_enter_runtime<Update::eReductions | Update::eStack | Update::eHeap>();
+    emit_enter_runtime<Update::eReductions | Update::eHeapAlloc>();
 
     a.mov(ARG1, c_p);
     load_x_reg_array(ARG2);
     runtime_call<5>(erts_gc_update_map_exact);
 
-    emit_leave_runtime<Update::eReductions | Update::eStack | Update::eHeap>();
+    emit_leave_runtime<Update::eReductions | Update::eHeapAlloc>();
     emit_leave_frame();
 
     emit_test_the_non_value(RET);
@@ -579,13 +642,13 @@ void BeamGlobalAssembler::emit_update_map_exact_body_shared() {
     Label error = a.newLabel();
 
     emit_enter_frame();
-    emit_enter_runtime<Update::eReductions | Update::eStack | Update::eHeap>();
+    emit_enter_runtime<Update::eReductions | Update::eHeapAlloc>();
 
     a.mov(ARG1, c_p);
     load_x_reg_array(ARG2);
     runtime_call<5>(erts_gc_update_map_exact);
 
-    emit_leave_runtime<Update::eReductions | Update::eStack | Update::eHeap>();
+    emit_leave_runtime<Update::eReductions | Update::eHeapAlloc>();
     emit_leave_frame();
 
     emit_test_the_non_value(RET);
@@ -600,28 +663,70 @@ void BeamGlobalAssembler::emit_update_map_exact_body_shared() {
     }
 }
 
+/* ARG2 = key
+ * ARG3 = value
+ * ARG4 = map
+ *
+ * Does not return on error. */
+void BeamGlobalAssembler::emit_update_map_single_exact_body_shared() {
+    Label error = a.newLabel();
+
+    a.mov(TMP_MEM2q, ARG2);
+
+    emit_enter_frame();
+    emit_enter_runtime<Update::eHeapAlloc>();
+
+    a.mov(ARG1, c_p);
+    a.lea(ARG5, TMP_MEM1q);
+    runtime_call<5>(erts_maps_update);
+
+    emit_leave_runtime<Update::eHeapAlloc>();
+    emit_leave_frame();
+
+    a.test(RETd, RETd);
+    a.short_().je(error);
+
+    a.mov(RET, TMP_MEM1q);
+    a.ret();
+
+    a.bind(error);
+    {
+        a.mov(RET, TMP_MEM2q);
+        a.mov(x86::qword_ptr(c_p, offsetof(Process, freason)), imm(BADKEY));
+        a.mov(x86::qword_ptr(c_p, offsetof(Process, fvalue)), RET);
+        mov_imm(ARG4, 0);
+        a.jmp(labels[raise_exception]);
+    }
+}
+
 void BeamModuleAssembler::emit_update_map_exact(const ArgSource &Src,
                                                 const ArgLabel &Fail,
                                                 const ArgRegister &Dst,
                                                 const ArgWord &Live,
                                                 const ArgWord &Size,
                                                 const Span<ArgVal> &args) {
-    Label data = embed_vararg_rodata(args, CP_SIZE);
-
     ASSERT(Size.get() == args.size());
 
-    /* We _KNOW_ Src is a map */
-    mov_arg(getXRef(Live.get()), Src);
-
-    mov_imm(ARG3, Live.get());
-    mov_imm(ARG4, args.size());
-    a.lea(ARG5, x86::qword_ptr(data));
-
-    if (Fail.get() != 0) {
-        fragment_call(ga->get_update_map_exact_guard_shared());
-        a.je(resolve_beam_label(Fail));
+    if (args.size() == 2 && Fail.get() == 0) {
+        mov_arg(ARG2, args[0]);
+        mov_arg(ARG3, args[1]);
+        mov_arg(ARG4, Src);
+        fragment_call(ga->get_update_map_single_exact_body_shared());
     } else {
-        fragment_call(ga->get_update_map_exact_body_shared());
+        Label data = embed_vararg_rodata(args, CP_SIZE);
+
+        mov_arg(getXRef(Live.get()), Src);
+
+        mov_imm(ARG3, Live.get());
+        mov_imm(ARG4, args.size());
+        a.lea(ARG5, x86::qword_ptr(data));
+
+        if (Fail.get() != 0) {
+            fragment_call(ga->get_update_map_exact_guard_shared());
+            a.je(resolve_beam_label(Fail));
+        } else {
+            fragment_call(ga->get_update_map_exact_body_shared());
+        }
     }
 
     mov_arg(Dst, RET);
