@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  * 
- * Copyright Ericsson AB 1997-2016. All Rights Reserved.
+ * Copyright Ericsson AB 1997-2024. All Rights Reserved.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -318,8 +318,12 @@ reg_from_erlang(ErlDrvData clientData, char* buf, ErlDrvSizeT count)
 	}
 	break;
     case CMD_DELETE_KEY:
-	result = RegDeleteKey(rp->hkey, NULL);
-	reply(rp, result);
+	{
+	    char* key = buf;
+
+	    result = RegDeleteKey(rp->hkey, key);
+	    reply(rp, result);
+	}
 	break;
     case CMD_DELETE_VALUE:
 	result = RegDeleteValue(rp->hkey, buf);
@@ -370,8 +374,11 @@ fix_value_result(RegPort* rp, LONG result, DWORD type,
     switch (type) {
     case REG_SZ:
     case REG_EXPAND_SZ:
-	valueSize--;		/* No reason to send the '\0' to Erlang. */
-	break;
+        /* No reason to send the trailing '\0', if present, to Erlang. */
+        if (valueSize > 0 && value[valueSize - 1] == '\0') {
+            valueSize--;
+        }
+        break;
     case REG_DWORD_LITTLE_ENDIAN:
     case REG_DWORD_BIG_ENDIAN:
 	/*
