@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson 2017-2020. All Rights Reserved.
+ * Copyright Ericsson 2017-2023. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,6 +30,7 @@
 
 #include "erl_driver.h"
 #include "prim_file_nif.h"
+#include "prim_file_nif_dyncall.h"
 
 /* NIF interface declarations */
 static int load(ErlNifEnv *env, void** priv_data, ERL_NIF_TERM load_info);
@@ -101,11 +102,17 @@ static ERL_NIF_TERM read_file_nif(ErlNifEnv *env, int argc, const ERL_NIF_TERM a
 static ERL_NIF_TERM open_nif(ErlNifEnv *env, int argc, const ERL_NIF_TERM argv[]);
 static ERL_NIF_TERM close_nif(ErlNifEnv *env, int argc, const ERL_NIF_TERM argv[]);
 
+static ERL_NIF_TERM file_desc_to_ref_nif(ErlNifEnv *env, int argc, const ERL_NIF_TERM argv[]);
+
 /* Internal ops */
 static ERL_NIF_TERM delayed_close_nif(ErlNifEnv *env, int argc, const ERL_NIF_TERM argv[]);
 static ERL_NIF_TERM get_handle_nif(ErlNifEnv *env, int argc, const ERL_NIF_TERM argv[]);
 static ERL_NIF_TERM altname_nif(ErlNifEnv *env, int argc, const ERL_NIF_TERM argv[]);
 
+/* Helper functions */
+
+static ERL_NIF_TERM create_ref_or_error_tuple(ErlNifEnv *env, efile_data_t *d);
+
 /* All file handle operations are passed through a wrapper that handles state
  * transitions, marking it as busy during the course of the operation, and
  * closing on completion if the owner died in the middle of an operation.
@@ -123,16 +130,16 @@ static ERL_NIF_TERM altname_nif(ErlNifEnv *env, int argc, const ERL_NIF_TERM arg
  * The states may transition as follows:
  *
  * IDLE ->
- *      BUSY (file_handle_wrapper) |
+ *      BUSY (file_handle_wrapper | dyncall_dup) |
  *      CLOSED (owner_death_callback)
  *
  * BUSY ->
- *      IDLE (file_handle_wrapper)
+ *      IDLE (file_handle_wrapper | dyncall_dup)
  *      CLOSED (close_nif_impl)
  *      CLOSE_PENDING (owner_death_callback)
  *
  * CLOSE_PENDING ->
- *      CLOSED (file_handle_wrapper)
+ *      CLOSED (file_handle_wrapper | dyncall_dup)
  *
  * Should the owner of a file die, we can't close it immediately as that could
  * potentially block a normal scheduler. When entering the CLOSED state from
@@ -205,6 +212,7 @@ static ErlNifFunc nif_funcs[] = {
     {"get_handle_nif", 1, get_handle_nif},
     {"delayed_close_nif", 1, delayed_close_nif, ERL_NIF_DIRTY_JOB_IO_BOUND},
     {"altname_nif", 1, altname_nif, ERL_NIF_DIRTY_JOB_IO_BOUND},
+    {"file_desc_to_ref_nif", 1, file_desc_to_ref_nif, ERL_NIF_DIRTY_JOB_IO_BOUND},
 };
 
 ERL_NIF_INIT(prim_file, nif_funcs, load, NULL, upgrade, unload)
@@ -213,6 +221,8 @@ static ErlNifPid erts_prim_file_pid;
 
 static void owner_death_callback(ErlNifEnv* env, void* obj, ErlNifPid* pid, ErlNifMonitor* mon);
 
+static ErlNifResourceDynCall dyncall;
+
 static int load(ErlNifEnv *env, void** priv_data, ERL_NIF_TERM prim_file_pid)
 {
     ErlNifResourceTypeInit callbacks;
@@ -257,11 +267,13 @@ static int load(ErlNifEnv *env, void** priv_data, ERL_NIF_TERM prim_file_pid)
     am_cur = enif_make_atom(env, "cur");
     am_eof = enif_make_atom(env, "eof");
 
-    callbacks.down = owner_death_callback;
-    callbacks.dtor = NULL;
-    callbacks.stop = NULL;
+    callbacks.down      = owner_death_callback;
+    callbacks.dtor      = NULL;
+    callbacks.stop      = NULL;
+    callbacks.dyncall   = dyncall;
+    callbacks.members   = 4;
 
-    efile_resource_type = enif_open_resource_type_x(env, "efile", &callbacks,
+    efile_resource_type = enif_init_resource_type(env, "efile", &callbacks,
         ERL_NIF_RT_CREATE, NULL);
 
     *priv_data = NULL;
@@ -321,8 +333,9 @@ static ERL_NIF_TERM file_handle_wrapper(file_op_impl_t operation, ErlNifEnv *env
         ASSERT(previous_state != EFILE_STATE_IDLE);
 
         if(previous_state == EFILE_STATE_CLOSE_PENDING) {
-            /* This is the only point where a change from CLOSE_PENDING is
-             * possible, and we're running synchronously, so we can't race with
+            /* Here and in dyncall_dup are the only points
+             * where a change from CLOSE_PENDING is possible,
+             * and we're running synchronously, so we can't race with
              * anything else here. */
             posix_errno_t ignored;
 
@@ -407,6 +420,56 @@ static void owner_death_callback(ErlNifEnv* env, void* obj, ErlNifPid* pid, ErlN
     }
 }
 
+static void dyncall_dup(ErlNifEnv* env, efile_data_t* d, struct prim_file_nif_dyncall_dup *dc_dup) {
+
+    enum efile_state_t previous_state;
+
+    previous_state = erts_atomic32_cmpxchg_acqb(&d->state,
+        EFILE_STATE_BUSY, EFILE_STATE_IDLE);
+
+    if(previous_state == EFILE_STATE_IDLE) {
+
+        dc_dup->result = (int)
+            efile_dup_handle(env, d, &dc_dup->handle);
+
+        previous_state = erts_atomic32_cmpxchg_relb(&d->state,
+            EFILE_STATE_IDLE, EFILE_STATE_BUSY);
+
+        ASSERT(previous_state != EFILE_STATE_IDLE);
+
+        if(previous_state == EFILE_STATE_CLOSE_PENDING) {
+            /* Here and in file_handle_wrapper are the only points
+             * where a change from CLOSE_PENDING is possible,
+             * and we're running synchronously, so we can't race with
+             * anything else here. */
+            posix_errno_t ignored;
+
+            erts_atomic32_set_acqb(&d->state, EFILE_STATE_CLOSED);
+            efile_close(d, &ignored);
+        }
+    } else {
+        /* CLOSE_PENDING should be impossible at this point since it requires
+         * a transition from BUSY; the only valid state here is CLOSED. */
+        ASSERT(previous_state == EFILE_STATE_CLOSED);
+
+        dc_dup->result = EINVAL;
+    }
+}
+
+static void dyncall(ErlNifEnv* env, void* obj, void* data) {
+    efile_data_t *d = (efile_data_t*)obj;
+    struct prim_file_nif_dyncall *dc = (struct prim_file_nif_dyncall *)data;
+
+    switch (dc->op) {
+    case prim_file_nif_dyncall_dup:
+        dyncall_dup(env, d, (struct prim_file_nif_dyncall_dup *)dc);
+        return;
+    default:
+        dc->result = ENOTSUP;
+        return;
+    }
+}
+
 static ERL_NIF_TERM efile_filetype_to_atom(enum efile_filetype_t type) {
     switch(type) {
         case EFILE_FILETYPE_DEVICE: return am_device;
@@ -472,28 +535,10 @@ static enum efile_modes_t efile_translate_modelist(ErlNifEnv *env, ERL_NIF_TERM
     return modes;
 }
 
-static ERL_NIF_TERM open_nif(ErlNifEnv *env, int argc, const ERL_NIF_TERM argv[]) {
-    posix_errno_t posix_errno;
-    efile_data_t *d;
-
+static ERL_NIF_TERM create_ref_or_error_tuple(ErlNifEnv *env, efile_data_t *d) {
     ErlNifPid controlling_process;
-    enum efile_modes_t modes;
     ERL_NIF_TERM result;
-    efile_path_t path;
-
-    ASSERT(argc == 2);
-    if(!enif_is_list(env, argv[1])) {
-        return enif_make_badarg(env);
-    }
-
-    modes = efile_translate_modelist(env, argv[1]);
-
-    if((posix_errno = efile_marshal_path(env, argv[0], &path))) {
-        return posix_error_to_tuple(env, posix_errno);
-    } else if((posix_errno = efile_open(&path, modes, efile_resource_type, &d))) {
-        return posix_error_to_tuple(env, posix_errno);
-    }
-
+    
     enif_self(env, &controlling_process);
 
     if(enif_monitor_process(env, d, &controlling_process, &d->monitor)) {
@@ -521,6 +566,52 @@ static ERL_NIF_TERM open_nif(ErlNifEnv *env, int argc, const ERL_NIF_TERM argv[]
     return enif_make_tuple2(env, am_ok, result);
 }
 
+static ERL_NIF_TERM open_nif(ErlNifEnv *env, int argc, const ERL_NIF_TERM argv[]) {
+    posix_errno_t posix_errno;
+    efile_data_t *d;
+
+    enum efile_modes_t modes;
+    efile_path_t path;
+
+    ASSERT(argc == 2);
+    if(!enif_is_list(env, argv[1])) {
+        return enif_make_badarg(env);
+    }
+
+    modes = efile_translate_modelist(env, argv[1]);
+
+    if((posix_errno = efile_marshal_path(env, argv[0], &path))) {
+        return posix_error_to_tuple(env, posix_errno);
+    } else if((posix_errno = efile_open(&path, modes, efile_resource_type, &d))) {
+        return posix_error_to_tuple(env, posix_errno);
+    }
+
+    return create_ref_or_error_tuple(env, d);
+}
+
+static ERL_NIF_TERM file_desc_to_ref_nif(ErlNifEnv *env, int argc, const ERL_NIF_TERM argv[]) {
+    posix_errno_t posix_errno;
+    efile_data_t *d;
+
+    int fd;
+
+    ASSERT(argc == 1);
+
+    if(!enif_is_number(env, argv[0])) {
+        return enif_make_badarg(env);
+    }
+
+    if(!enif_get_int(env, argv[0], &fd)) {
+        return enif_make_badarg(env);
+    }
+
+    if((posix_errno = efile_from_fd(fd, efile_resource_type, &d))) {
+        return posix_error_to_tuple(env, posix_errno);
+    }
+
+    return create_ref_or_error_tuple(env, d);
+}
+
 static ERL_NIF_TERM close_nif(ErlNifEnv *env, int argc, const ERL_NIF_TERM argv[]) {
     enum efile_state_t previous_state;
     efile_data_t *d;
@@ -893,6 +984,7 @@ static ERL_NIF_TERM ipread_s32bu_p32bu_nif_impl(efile_data_t *d, ErlNifEnv *env,
 }
 
 static ERL_NIF_TERM get_handle_nif_impl(efile_data_t *d, ErlNifEnv *env, int argc, const ERL_NIF_TERM argv[]) {
+
     ASSERT(argc == 0);
 
     return efile_get_handle(env, d);
@@ -905,9 +997,12 @@ static ERL_NIF_TERM build_file_info(ErlNifEnv *env, efile_fileinfo_t *info) {
         enif_make_uint64(env, info->size),
         efile_filetype_to_atom(info->type),
         efile_access_to_atom(info->access),
-        enif_make_int64(env, MAX(EFILE_MIN_FILETIME, info->a_time)),
-        enif_make_int64(env, MAX(EFILE_MIN_FILETIME, info->m_time)),
-        enif_make_int64(env, MAX(EFILE_MIN_FILETIME, info->c_time)),
+        enif_make_int64(env,
+            MIN(EFILE_MAX_FILETIME, MAX(EFILE_MIN_FILETIME, info->a_time))),
+        enif_make_int64(env,
+            MIN(EFILE_MAX_FILETIME, MAX(EFILE_MIN_FILETIME, info->m_time))),
+        enif_make_int64(env,
+            MIN(EFILE_MAX_FILETIME, MAX(EFILE_MIN_FILETIME, info->c_time))),
         enif_make_uint(env, info->mode),
         enif_make_uint(env, info->links),
         enif_make_uint(env, info->major_device),
