@@ -10,27 +10,19 @@
 #endif
 
 typedef unsigned char byte;
+int write_exact(byte *buf, int len);
 
-int read_cmd(byte *buf)
+int read_exact(byte *buf, int len);
+
+int write_exact(byte *buf, int len)
 {
-  int len;
-  if (read_exact(buf, 4) != 4)
-      return(-1);
-
-  len = (buf[0] << 24) | (buf[1] << 16) | (buf[2] << 8) | buf[3];
-  return read_exact(buf, len);
-}
-
-int write_cmd(byte *buf, int len)
-{
-  byte li[4];
-  li[0] = (len >> 24) & 0xff;
-  li[1] = (len >> 16) & 0xff;
-  li[2] = (len >> 8)  & 0xff;
-  li[3] = len  & 0xff;
-  write_exact(&li, 4);
-
-  return write_exact(buf, len);
+  int i, wrote = 0;
+  do {
+    if ((i = write(1, buf+wrote, len-wrote)) < 0)
+      return (i);
+    wrote += i;
+  } while (wrote<len);
+  return len;
 }
 
 int read_exact(byte *buf, int len)
@@ -46,15 +38,26 @@ int read_exact(byte *buf, int len)
   return len;
 }
 
-int write_exact(byte *buf, int len)
+int write_cmd(byte *buf, int len)
 {
-  int i, wrote = 0;
-  do {
-    if ((i = write(1, buf+wrote, len-wrote)) < 0)
-      return (i);
-    wrote += i;
-  } while (wrote<len);
-  return len;
+  byte li[4];
+  li[0] = (len >> 24) & 0xff;
+  li[1] = (len >> 16) & 0xff;
+  li[2] = (len >> 8)  & 0xff;
+  li[3] = len  & 0xff;
+  write_exact(li, 4);
+
+  return write_exact(buf, len);
+}
+
+int read_cmd(byte *buf)
+{
+  int len;
+  if (read_exact(buf, 4) != 4)
+      return(-1);
+
+  len = (buf[0] << 24) | (buf[1] << 16) | (buf[2] << 8) | buf[3];
+  return read_exact(buf, len);
 }
 
 byte static_buf[31457280]; // 30 mb
