@@ -177,6 +177,7 @@ eval({op,_,Op,A0}, LFH) ->
 eval({op,_,Op,A0,B0}, LFH) ->
     [A,B] = eval_list([A0,B0], LFH),
     Res = eval_op(Op, A, B),
+    ok = eval_op_guard(Op, A, B, Res),
     erlang:garbage_collect(),
     Res;
 eval({integer,_,I}, _) ->
@@ -207,6 +208,18 @@ eval_op('bxor', A, B) -> A bxor B;
 eval_op('bsl', A, B) -> A bsl B;
 eval_op('bsr', A, B) -> A bsr B.
 
+eval_op_guard('-', A, B, Res) when Res =:= A - B -> ok;
+eval_op_guard('+', A, B, Res) when Res =:= A + B -> ok;
+eval_op_guard('*', A, B, Res) when Res =:= A * B -> ok;
+eval_op_guard('div', A, B, Res) when Res =:= A div B -> ok;
+eval_op_guard('rem', A, B, Res) when Res =:= A rem B -> ok;
+eval_op_guard('band', A, B, Res) when Res =:= A band B -> ok;
+eval_op_guard('bor', A, B, Res) when Res =:= A bor B -> ok;
+eval_op_guard('bxor', A, B, Res) when Res =:= A bxor B -> ok;
+eval_op_guard('bsl', A, B, Res) when Res =:= A bsl B -> ok;
+eval_op_guard('bsr', A, B, Res) when Res =:= A bsr B -> ok;
+eval_op_guard(Op, A, B, Res) -> {error,{Op,A,B,Res}}.
+
 test_squaring(I) ->
     %% Multiplying an integer by itself is specially optimized, so we
     %% should take special care to test squaring.  The optimization
@@ -214,7 +227,7 @@ test_squaring(I) ->
     Sqr = I * I,
 
     %% This expression will be multiplied in the usual way, because
-    %% the the two operands for '*' are stored at different addresses.
+    %% the two operands for '*' are stored at different addresses.
     Sqr = I * ((I + id(1)) - id(1)),
 
     ok.
@@ -520,12 +533,13 @@ properties(_Config) ->
     _ = [begin
              A = id(rand_int()),
              B = id(rand_int()),
-             io:format("~.36#\n~.36#\n", [A,B]),
-             test_properties(A, B)
+             C = id(rand_int()),
+             io:format("~.36#\n~.36#\n~.36#\n", [A,B,C]),
+             test_properties(A, B, C)
          end || _ <- lists:seq(1, 1000)],
     ok.
 
-test_properties(A, B) ->
+test_properties(A, B, C) ->
     SquaredA = id(A * A),
     SquaredB = id(B * B),
 
@@ -543,6 +557,11 @@ test_properties(A, B) ->
     A = id(Sum - B),
     B = id(Sum - A),
     0 = Sum - A - B,
+    C = id(A + B + C) - Sum,
+
+    PS = id(A * B + C),
+    PS = P + C,
+    ok = test_mul_add_guard(A, B, C, PS),
 
     NegA = id(-A),
     A = -NegA,
@@ -563,6 +582,7 @@ test_properties(A, B) ->
 
     ok.
 
+test_mul_add_guard(A, B, C, Res) when Res =:= A * B + C -> ok.
 
 rand_int() ->
     Sz = max(floor(rand:normal() * 512 + 256), 7),
