@@ -1,7 +1,8 @@
+
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1998-2017. All Rights Reserved.
+%% Copyright Ericsson AB 1998-2024. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -22,43 +23,74 @@
 
 -include_lib("common_test/include/ct.hrl").
 
--export([all/0, suite/0]).
--export([trace_bif/1, trace_bif_timestamp/1, trace_on_and_off/1, 
+-export([all/0, suite/0, groups/0,
+         init_per_suite/1, end_per_suite/1,
+         init_per_group/2, end_per_group/2,
+	 init_per_testcase/2,end_per_testcase/2]).
+
+-export([trace_bif/1, trace_bif_timestamp/1, trace_on_and_off/1,
 	 trace_bif_local/1,
-	 trace_bif_timestamp_local/1, trace_bif_return/1, not_run/1,
+	 trace_bif_timestamp_local/1, trace_bif_return/1,
 	 trace_info_old_code/1]).
 
 -export([bif_process/0]).
 
 suite() -> [{ct_hooks,[ts_install_cth]}].
 
-all() -> 
-    case test_server:is_native(trace_bif_SUITE) of
-	true -> [not_run];
-	false ->
-	    [trace_bif, trace_bif_timestamp, trace_on_and_off,
-	     trace_bif_local, trace_bif_timestamp_local,
-	     trace_bif_return, trace_info_old_code]
-    end.
+all() ->
+    trace_sessions:all().
+
+groups() ->
+    trace_sessions:groups(testcases()).
+
+testcases() ->
+    [trace_bif, trace_bif_timestamp, trace_on_and_off,
+     trace_bif_local, trace_bif_timestamp_local,
+     trace_bif_return, trace_info_old_code].
+
+init_per_suite(Config) ->
+    trace_sessions:init_per_suite(Config, ?MODULE).
+
+end_per_suite(Config) ->
+    trace_sessions:end_per_suite(Config).
+
+init_per_group(Group, Config) ->
+    trace_sessions:init_per_group(Group, Config).
+
+end_per_group(Group, Config) ->
+    trace_sessions:end_per_group(Group, Config).
 
 
-not_run(Config) when is_list(Config) -> 
-    {skipped,"Native code"}.
+init_per_testcase(Func, Config) when is_atom(Func), is_list(Config) ->
+    Config.
+
+end_per_testcase(_Func, Config) ->
+    trace_sessions:end_per_testcase(Config).
+
+erlang_trace(A,B,C) ->
+    trace_sessions:erlang_trace(A,B,C).
+
+erlang_trace_pattern(A,B,C) ->
+    trace_sessions:erlang_trace_pattern(A,B,C).
+
+erlang_trace_info(A,B) ->
+    trace_sessions:erlang_trace_info(A,B).
+
 
 %% Tests switching tracing on and off.
 trace_on_and_off(Config) when is_list(Config) ->
     Pid = spawn_link(?MODULE, bif_process, []),
     Self = self(),
-    1 = erlang:trace(Pid, true, [call,timestamp]),
-    {flags, Flags} = erlang:trace_info(Pid,flags),
+    1 = erlang_trace(Pid, true, [call,timestamp]),
+    {flags, Flags} = erlang_trace_info(Pid,flags),
     [call,timestamp] = lists:sort(Flags),
-    {tracer, Self} = erlang:trace_info(Pid,tracer),
-    1 = erlang:trace(Pid, false, [timestamp]),
-    {flags,[call]} =  erlang:trace_info(Pid,flags),
-    {tracer, Self} = erlang:trace_info(Pid,tracer),
-    1 = erlang:trace(Pid, false, [call]),
-    {flags,[]} =  erlang:trace_info(Pid,flags),
-    {tracer, []} = erlang:trace_info(Pid,tracer),
+    {tracer, Self} = erlang_trace_info(Pid,tracer),
+    1 = erlang_trace(Pid, false, [timestamp]),
+    {flags,[call]} =  erlang_trace_info(Pid,flags),
+    {tracer, Self} = erlang_trace_info(Pid,tracer),
+    1 = erlang_trace(Pid, false, [call]),
+    {flags,[]} =  erlang_trace_info(Pid,flags),
+    {tracer, []} = erlang_trace_info(Pid,tracer),
     unlink(Pid),
     exit(Pid,kill),
     ok.
@@ -73,8 +105,8 @@ trace_bif_local(Config) when is_list(Config) ->
 
 do_trace_bif(Flags) ->
     Pid = spawn_link(?MODULE, bif_process, []),
-    1 = erlang:trace(Pid, true, [call]),
-    erlang:trace_pattern({erlang,'_','_'}, [], Flags),
+    1 = erlang_trace(Pid, true, [call]),
+    erlang_trace_pattern({erlang,'_','_'}, [], Flags),
     Pid ! {do_bif, time, []},
     receive_trace_msg({trace,Pid,call,{erlang,time, []}}),
     Pid ! {do_bif, statistics, [runtime]},
@@ -89,8 +121,8 @@ do_trace_bif(Flags) ->
     receive_trace_msg({trace,Pid,call,
                        {erlang,statistics, [runtime]}}),
 
-    1 = erlang:trace(Pid, false, [call]),
-    erlang:trace_pattern({erlang,'_','_'}, false, Flags),
+    1 = erlang_trace(Pid, false, [call]),
+    erlang_trace_pattern({erlang,'_','_'}, false, Flags),
     unlink(Pid),
     exit(Pid, die),
     ok.
@@ -124,8 +156,8 @@ trace_bif_timestamp_local(Config) when is_list(Config) ->
 do_trace_bif_timestamp(Flags, TsType, TsFlags) ->
     io:format("Testing with TsType=~p TsFlags=~p~n", [TsType, TsFlags]),
     Pid = spawn_link(?MODULE, bif_process, []),
-    1 = erlang:trace(Pid, true, [call]++TsFlags),
-    erlang:trace_pattern({erlang,'_','_'}, [], Flags),
+    1 = erlang_trace(Pid, true, [call]++TsFlags),
+    erlang_trace_pattern({erlang,'_','_'}, [], Flags),
 
     Ts0 = make_ts(TsType),
     Pid ! {do_bif, time, []},
@@ -150,7 +182,7 @@ do_trace_bif_timestamp(Flags, TsType, TsFlags) ->
     check_ts(TsType, Ts4, make_ts(TsType)),
 
     %% We should be able to turn off the timestamp.
-    1 = erlang:trace(Pid, false, TsFlags),
+    1 = erlang_trace(Pid, false, TsFlags),
 
     Pid ! {do_statistics_bif},
     receive_trace_msg({trace,Pid,call,
@@ -160,8 +192,8 @@ do_trace_bif_timestamp(Flags, TsType, TsFlags) ->
     receive_trace_msg({trace,Pid,call,
                        {erlang,statistics, [runtime]}}),
 
-    1 = erlang:trace(Pid, false, [call]),
-    erlang:trace_pattern({erlang,'_','_'}, false, Flags),
+    1 = erlang_trace(Pid, false, [call]),
+    erlang_trace_pattern({erlang,'_','_'}, false, Flags),
 
     unlink(Pid),
     exit(Pid, die),
@@ -183,8 +215,8 @@ trace_bif_return(Config) when is_list(Config) ->
 do_trace_bif_return(TsType, TsFlags) ->
     io:format("Testing with TsType=~p TsFlags=~p~n", [TsType, TsFlags]),
     Pid = spawn_link(?MODULE, bif_process, []),
-    1 = erlang:trace(Pid, true, [call,return_to]++TsFlags),
-    erlang:trace_pattern({erlang,'_','_'}, [{'_',[],[{return_trace}]}], 
+    1 = erlang_trace(Pid, true, [call,return_to]++TsFlags),
+    erlang_trace_pattern({erlang,'_','_'}, [{'_',[],[{return_trace}]}],
                          [local]),
 
     Ts0 = make_ts(TsType),
@@ -235,7 +267,7 @@ do_trace_bif_return(TsType, TsFlags) ->
                                            {?MODULE, bif_process,0}},
                                           Ts11, TsType),
     check_ts(TsType, Ts12, make_ts(TsType)),
-    erlang:trace_pattern({erlang,'_','_'}, false, [local]),
+    erlang_trace_pattern({erlang,'_','_'}, false, [local]),
     ok.
 
 
@@ -348,7 +380,8 @@ trace_info_old_code(Config) when is_list(Config) ->
     {ok,M,Mbin} = compile:forms(AbsForms),
     {module,M} = code:load_binary(M, Fname, Mbin),
     true  = erlang:delete_module(M),
-    {traced,undefined} = erlang:trace_info(MFA, traced),
+    {traced,undefined} = erlang_trace_info(MFA, traced),
+    erlang:purge_module(M),
     ok.
 
 a(L) ->
