@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1999-2016. All Rights Reserved.
+%% Copyright Ericsson AB 1999-2024. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -21,8 +21,12 @@
 
 -module(trace_port_SUITE).
 
--export([all/0, suite/0,
-	 call_trace/1,
+-export([all/0, suite/0, groups/0,
+         init_per_suite/1, end_per_suite/1,
+         init_per_group/2, end_per_group/2,
+         init_per_testcase/2, end_per_testcase/2]).
+
+-export([call_trace/1,
 	 return_trace/1,
 	 send/1,
 	 receive_trace/1,
@@ -33,65 +37,100 @@
 	 default_tracer/1,
 	 tracer_port_crash/1]).
 
+%% Needed for apply
+-export([erlang_trace/3, erlang_trace_info/2, erlang_trace_pattern/3]).
+
+
 -include_lib("common_test/include/ct.hrl").
 
 suite() ->
     [{ct_hooks,[ts_install_cth]},
-     {timetrap, {seconds, 30}}].
+     {timetrap, {seconds, 20}}]. %{minutes, 2}}].
 
 all() ->
+    trace_sessions:all().
+
+groups() ->
+    trace_sessions:groups(testcases()).
+
+testcases() ->
     [call_trace, return_trace, send, receive_trace,
      receive_trace_non_scheduler,
      process_events, schedule, gc,
      default_tracer, tracer_port_crash].
 
+init_per_suite(Config) ->
+    trace_sessions:init_per_suite(Config, ?MODULE).
+
+end_per_suite(Config) ->
+    trace_sessions:end_per_suite(Config).
+
+init_per_group(Group, Config) ->
+    trace_sessions:init_per_group(Group, Config).
+
+end_per_group(Group, Config) ->
+    trace_sessions:end_per_group(Group, Config).
+
+
+init_per_testcase(Func, Config) when is_atom(Func), is_list(Config) ->
+    trace_sessions:init_per_testcase(Config).
+
+end_per_testcase(_Func, Config) ->
+    trace_sessions:end_per_testcase(Config).
+
+
+erlang_trace(A,B,C) ->
+    trace_sessions:erlang_trace(A,B,C).
+
+erlang_trace_pattern(A,B,C) ->
+    trace_sessions:erlang_trace_pattern(A,B,C).
+
+erlang_trace_info(A,B) ->
+    trace_sessions:erlang_trace_info(A,B).
+
+
+
 %% Test sending call trace messages to a port.
 call_trace(Config) when is_list(Config) ->
-    case test_server:is_native(?MODULE) orelse
-         test_server:is_native(lists) of
-        true -> 
-            {skip,"Native code"};
-        false ->
-            start_tracer(Config),
-            Self = self(),
-            trace_func({lists,reverse,1}, []),
-            trace_pid(Self, true, [call]),
-            trace_info(Self, flags),
-            trace_info(Self, tracer),
-            [b,a] = lists:reverse([a,b]),
-            expect({trace,Self,call,{lists,reverse,[[a,b]]}}),
+    start_tracer(Config),
+    Self = self(),
+    trace_func({lists,reverse,1}, []),
+    trace_pid(Self, true, [call]),
+    trace_info(Self, flags),
+    trace_info(Self, tracer),
+    [b,a] = lists:reverse([a,b]),
+    expect({trace,Self,call,{lists,reverse,[[a,b]]}}),
 
-            trace_pid(Self, true, [timestamp]),
-            trace_info(Self, flags),
-            Huge = huge_data(),
-            lists:reverse(Huge),
-            expect({trace_ts,Self,call,{lists,reverse,[Huge]},ts}),
+    trace_pid(Self, true, [timestamp]),
+    trace_info(Self, flags),
+    Huge = huge_data(),
+    lists:reverse(Huge),
+    expect({trace_ts,Self,call,{lists,reverse,[Huge]},ts}),
 
-            trace_pid(Self, true, [arity]),
-            trace_info(Self, flags),
-            [y,x] = lists:reverse([x,y]),
-            expect({trace_ts,Self,call,{lists,reverse,1},ts}),
+    trace_pid(Self, true, [arity]),
+    trace_info(Self, flags),
+    [y,x] = lists:reverse([x,y]),
+    expect({trace_ts,Self,call,{lists,reverse,1},ts}),
 
-            trace_pid(Self, false, [timestamp]),
-            trace_info(Self, flags),
-            [z,y,x] = lists:reverse([x,y,z]),
-            expect({trace,Self,call,{lists,reverse,1}}),
+    trace_pid(Self, false, [timestamp]),
+    trace_info(Self, flags),
+    [z,y,x] = lists:reverse([x,y,z]),
+    expect({trace,Self,call,{lists,reverse,1}}),
 
-            %% OTP-7399. Delayed sub-binary creation optimization.
-            trace_pid(Self, false, [arity]),
-            trace_info(Self, flags),
-            trace_func({?MODULE,bs_sum_c,2}, [], [local]),
-            26 = bs_sum_c(<<3:4,5:4,7:4,11:4>>, 0),
-            trace_func({?MODULE,bs_sum_c,2}, false, [local]),
-            expect({trace,Self,call,{?MODULE,bs_sum_c,[<<3:4,5:4,7:4,11:4>>,0]}}),
-            expect({trace,Self,call,{?MODULE,bs_sum_c,[<<5:4,7:4,11:4>>,3]}}),
-            expect({trace,Self,call,{?MODULE,bs_sum_c,[<<7:4,11:4>>,8]}}),
-            expect({trace,Self,call,{?MODULE,bs_sum_c,[<<11:4>>,15]}}),
-            expect({trace,Self,call,{?MODULE,bs_sum_c,[<<>>,26]}}),
+    %% OTP-7399. Delayed sub-binary creation optimization.
+    trace_pid(Self, false, [arity]),
+    trace_info(Self, flags),
+    trace_func({?MODULE,bs_sum_c,2}, [], [local]),
+    26 = bs_sum_c(<<3:4,5:4,7:4,11:4>>, 0),
+    trace_func({?MODULE,bs_sum_c,2}, false, [local]),
+    expect({trace,Self,call,{?MODULE,bs_sum_c,[<<3:4,5:4,7:4,11:4>>,0]}}),
+    expect({trace,Self,call,{?MODULE,bs_sum_c,[<<5:4,7:4,11:4>>,3]}}),
+    expect({trace,Self,call,{?MODULE,bs_sum_c,[<<7:4,11:4>>,8]}}),
+    expect({trace,Self,call,{?MODULE,bs_sum_c,[<<11:4>>,15]}}),
+    expect({trace,Self,call,{?MODULE,bs_sum_c,[<<>>,26]}}),
 
-            trace_func({lists,reverse,1}, false),
-            ok
-    end.
+    trace_func({lists,reverse,1}, false),
+    ok.
 
 bs_sum_c(<<H:4,T/bits>>, Acc) -> bs_sum_c(T, H+Acc);
 bs_sum_c(<<>>, Acc) -> Acc.
@@ -99,38 +138,32 @@ bs_sum_c(<<>>, Acc) -> Acc.
 
 %% Test the new return trace.
 return_trace(Config) when is_list(Config) ->
-    case test_server:is_native(?MODULE) orelse
-         test_server:is_native(lists) of
-        true -> 
-            {skip,"Native code"};
-        false ->
-            start_tracer(Config),
-            Self = self(),
-            MFA = {lists,reverse,1},
+    start_tracer(Config),
+    Self = self(),
+    MFA = {lists,reverse,1},
 
-            %% Plain (no timestamp, small data).
+    %% Plain (no timestamp, small data).
 
-            trace_func(MFA, [{['$1'],[],[{return_trace},
-                                         {message,false}]}]),
-            trace_pid(Self, true, [call]),
-            trace_info(Self, flags),
-            trace_info(Self, tracer),
-            trace_info(MFA, match_spec),
-            {traced,global} = trace_info(MFA, traced),
-            [b,a] = lists:reverse([a,b]),
-            expect({trace,Self,return_from,MFA,[b,a]}),
+    trace_func(MFA, [{['$1'],[],[{return_trace},
+                                 {message,false}]}]),
+    trace_pid(Self, true, [call]),
+    trace_info(Self, flags),
+    trace_info(Self, tracer),
+    trace_info(MFA, match_spec),
+    {traced,global} = trace_info(MFA, traced),
+    [b,a] = lists:reverse([a,b]),
+    expect({trace,Self,return_from,MFA,[b,a]}),
 
-            %% Timestamp, huge data.
-            trace_pid(Self, true, [timestamp]),
-            Result = lists:reverse(huge_data()),
-            expect({trace_ts,Self,return_from,MFA,Result,ts}),
+    %% Timestamp, huge data.
+    trace_pid(Self, true, [timestamp]),
+    Result = lists:reverse(huge_data()),
+    expect({trace_ts,Self,return_from,MFA,Result,ts}),
 
-            %% Turn off trace.
-            trace_func(MFA, false),
-            trace_info(MFA, match_spec),
-            {traced,false} = trace_info(MFA, traced),
-            ok
-    end.
+    %% Turn off trace.
+    trace_func(MFA, false),
+    trace_info(MFA, match_spec),
+    {traced,false} = trace_info(MFA, traced),
+    ok.
 
 %% Test sending send trace messages to a port.
 send(Config) when is_list(Config) ->
@@ -190,17 +223,18 @@ receive_trace(Config) when is_list(Config) ->
 receive_trace_non_scheduler(Config) when is_list(Config) ->
     start_tracer(Config),
     S = self(),
-    Receiver = spawn(
+    Receiver = spawn_link(
                  fun() ->
                          receive
                              go ->
-                                 Ref = S ! erlang:trace_delivered(all),
+                                 Ref = erlang:trace_delivered(all),
+                                 S ! {sync,Ref},
                                  receive {trace_delivered, Ref, all} -> ok end
                          end
                  end),
     trac(Receiver, true, ['receive']),
     Receiver ! go,
-    Ref = receive R -> R end,
+    Ref = receive {sync,R} -> R end,
     expect({trace,Receiver,'receive',go}),
     expect({trace,Receiver,'receive',{trace_delivered, all, Ref}}),
 
@@ -274,69 +308,61 @@ default_tracer(Config) when is_list(Config) ->
     TracerMonitor = erlang:monitor(process, Tracer),
     Port = get(tracer_port),
     %%
-    N = erlang:trace(all, true, [send, {tracer, Port}]),
-    {flags, [send]} = erlang:trace_info(self(), flags),
-    {tracer, Port} = erlang:trace_info(self(), tracer),
-    {flags, [send]} = erlang:trace_info(new, flags),
-    {tracer, Port} = erlang:trace_info(new, tracer),
+    N = erlang_trace(all, true, [send, {tracer, Port}]),
+    {flags, [send]} = erlang_trace_info(self(), flags),
+    {tracer, Port} = erlang_trace_info(self(), tracer),
+    {flags, [send]} = erlang_trace_info(new, flags),
+    {tracer, Port} = erlang_trace_info(new, tracer),
     G1 = fun_spawn(fun general/0),
-    {flags, [send]} = erlang:trace_info(G1, flags),
-    {tracer, Port} = erlang:trace_info(G1, tracer),
+    {flags, [send]} = erlang_trace_info(G1, flags),
+    {tracer, Port} = erlang_trace_info(G1, tracer),
     unlink(Tracer),
     exit(Port, done),
-    receive
-        {'DOWN', TracerMonitor, process, Tracer, TracerExitReason} ->
-            done = TracerExitReason
-    end,
-    {flags, []} = erlang:trace_info(self(), flags),
-    {tracer, []} = erlang:trace_info(self(), tracer),
-    {flags, []} = erlang:trace_info(new, flags),
-    {tracer, []} = erlang:trace_info(new, tracer),
-    M = erlang:trace(all, false, [all]),
-    {flags, []} = erlang:trace_info(self(), flags),
-    {tracer, []} = erlang:trace_info(self(), tracer),
-    {flags, []} = erlang:trace_info(G1, flags),
-    {tracer, []} = erlang:trace_info(G1, tracer),
+    done = receive
+               {'DOWN', TracerMonitor, process, Tracer, TracerExitReason} ->
+                   TracerExitReason
+           end,
+    {flags, []} = erlang_trace_info(self(), flags),
+    {tracer, []} = erlang_trace_info(self(), tracer),
+    {flags, []} = erlang_trace_info(new, flags),
+    {tracer, []} = erlang_trace_info(new, tracer),
+    M = erlang_trace(all, false, [all]),
+    {flags, []} = erlang_trace_info(self(), flags),
+    {tracer, []} = erlang_trace_info(self(), tracer),
+    {flags, []} = erlang_trace_info(G1, flags),
+    {tracer, []} = erlang_trace_info(G1, tracer),
     G1 ! {apply,{erlang,exit,[normal]}},
     io:format("~p = ~p.~n", [M, N]),
     M = N - 1, % G1 has been started, but Tracer and Port have died
     ok.
 
 tracer_port_crash(Config) when is_list(Config) ->
-    case test_server:is_native(?MODULE) orelse
-         test_server:is_native(lists) of
-        true -> 
-            {skip,"Native code"};
-        false ->
-            Tr = start_tracer(Config),
-            Port = get(tracer_port),
-            Tracee = spawn(fun () ->
-                                   register(trace_port_linker, self()),
-                                   link(Port),
-                                   receive go -> ok end,
-                                   lists:reverse([1,b,c]),
-                                   receive die -> ok end
-                           end),
-            Tr ! {unlink_tracer_port, self()},
-            receive {unlinked_tracer_port, Tr} -> ok end,
-            port_control(Port, $c, []), %% Make port commands crash tracer port...
-            trace_func({lists,reverse,1}, []),
-            trace_pid(Tracee, true, [call]),
-            trace_info(Tracee, flags),
-            trace_info(self(), tracer),
-            Tracee ! go,
-            receive after 1000 -> ok end,
-            case whereis(trace_port_linker) of
-                undefined ->
-                    ok;
-                Id ->
-                    %		    erts_debug:set_internal_state(available_internal_state, true),
-                    %		    erts_debug:set_internal_state(abort, {trace_port_linker, Id})
-                    ct:fail({trace_port_linker, Id})
-            end,
-            undefined = process_info(Tracee),
-            ok
-    end.
+    Tr = start_tracer(Config),
+    Port = get(tracer_port),
+    Tracee = spawn(fun () ->
+                           register(trace_port_linker, self()),
+                           link(Port),
+                           receive go -> ok end,
+                           lists:reverse([1,b,c]),
+                           receive die -> ok end
+                   end),
+    Tr ! {unlink_tracer_port, self()},
+    receive {unlinked_tracer_port, Tr} -> ok end,
+    port_control(Port, $c, []), %Make port commands crash tracer port...
+    trace_func({lists,reverse,1}, []),
+    trace_pid(Tracee, true, [call]),
+    trace_info(Tracee, flags),
+    trace_info(self(), tracer),
+    Tracee ! go,
+    receive after 1000 -> ok end,
+    case whereis(trace_port_linker) of
+        undefined ->
+            ok;
+        Id ->
+            ct:fail({trace_port_linker, Id})
+    end,
+    undefined = process_info(Tracee),
+    ok.
 
 %%% Help functions.
 
@@ -349,15 +375,6 @@ huge_data(N) ->
     P = huge_data(N div 2),
     [16#1234566,P|P].
 
-expect() ->
-    receive
-        Other ->
-            ok = io:format("Unexpected; got ~p", [Other]),
-            ct:fail({unexpected, Other})
-    after 200 ->
-              ok
-    end.
-
 expect({trace_ts,E1,E2,info,ts}=Message) ->
     receive
         {trace_ts,E1,E2,_Info,_Ts}=MessageTs ->
@@ -421,7 +438,7 @@ expect(Message) ->
 
 trac(What, On, Flags0) ->
     Flags = [{tracer,get(tracer_port)}|Flags0],
-    get(tracer) ! {apply,self(),{erlang,trace,[What,On,Flags]}},
+    get(tracer) ! {apply,self(),{?MODULE,erlang_trace,[What,On,Flags]}},
     Res = receive
               {apply_result,Result} -> Result
           end,
@@ -430,7 +447,7 @@ trac(What, On, Flags0) ->
     Res.
 
 trace_info(What, Key) ->
-    get(tracer) ! {apply,self(),{erlang,trace_info,[What,Key]}},
+    get(tracer) ! {apply,self(),{?MODULE,erlang_trace_info,[What,Key]}},
     Res = receive
               {apply_result,Result} -> Result
           end,
@@ -439,7 +456,7 @@ trace_info(What, Key) ->
     Res.
 
 trace_func(MFA, MatchProg) ->
-    get(tracer) ! {apply,self(),{erlang,trace_pattern,[MFA,MatchProg]}},
+    get(tracer) ! {apply,self(),{?MODULE,erlang_trace_pattern,[MFA,MatchProg,[]]}},
     Res = receive
               {apply_result,Result} -> Result
           end,
@@ -447,7 +464,7 @@ trace_func(MFA, MatchProg) ->
     Res.
 
 trace_func(MFA, MatchProg, Flags) ->
-    get(tracer) ! {apply,self(),{erlang,trace_pattern,[MFA,MatchProg,Flags]}},
+    get(tracer) ! {apply,self(),{?MODULE,erlang_trace_pattern,[MFA,MatchProg,Flags]}},
     Res = receive
               {apply_result,Result} -> Result
           end,
@@ -456,7 +473,7 @@ trace_func(MFA, MatchProg, Flags) ->
 
 trace_pid(Pid, On, Flags0) ->
     Flags = [{tracer,get(tracer_port)}|Flags0],
-    get(tracer) ! {apply,self(),{erlang,trace,[Pid,On,Flags]}},
+    get(tracer) ! {apply,self(),{?MODULE,erlang_trace,[Pid,On,Flags]}},
     Res = receive
               {apply_result,Result} -> Result
           end,
