@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  * 
- * Copyright Ericsson AB 1998-2021. All Rights Reserved.
+ * Copyright Ericsson AB 1998-2022. All Rights Reserved.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -535,10 +535,14 @@ static BOOL start_a_service(ServerInfo *srvi){
 	  HANDLE hJob = CreateJobObject(NULL, NULL);
 	  JOBOBJECT_EXTENDED_LIMIT_INFORMATION jeli = { 0 };
 	  /*
-	   * Causes all processes associated with the job to terminate when the
-	   * last handle to the job is closed.
-	   */
-	  jeli.BasicLimitInformation.LimitFlags = JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE;
+        * JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE
+        * Causes all processes associated with the job to terminate when the
+        * last handle to the job is closed.
+        *
+        * JOB_OBJECT_LIMIT_BREAKAWAY_OK
+        * Sometimes we want to break out, for example to start programs in another windows session
+      */
+      jeli.BasicLimitInformation.LimitFlags = JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE | JOB_OBJECT_LIMIT_BREAKAWAY_OK;
 	  SetInformationJobObject(hJob, JobObjectExtendedLimitInformation, &jeli, sizeof(jeli));
 	  if (AssignProcessToJobObject(hJob, GetCurrentProcess()) == FALSE) {
 	      log_error(L"Could not AssignProcessToJobObject");
@@ -920,7 +924,7 @@ static VOID WINAPI service_main_loop(DWORD argc, wchar_t **argv){
 		    L"The service is not restarted, ignoring OnFail option.");
 	} else {
 	  log_error(L"Erlang machine seems to die "
-		    L"continously, not restarted.");
+		    L"continuously, not restarted.");
 	}
 	CloseHandle(eventStop);
 	set_stopped(ERROR_PROCESS_ABORTED);
