@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2010-2021. All Rights Reserved.
+ * Copyright Ericsson AB 2010-2023. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -107,6 +107,8 @@ x86_init(void)
 
     ethr_x86_cpuid__(&eax, &ebx, &ecx, &edx);
 
+    ethr_runtime__.conf.have_cpuid = (eax > 0);
+
     if (eax > 0
 	&& (ETHR_IS_X86_VENDOR("GenuineIntel", ebx, ecx, edx)
 	    || ETHR_IS_X86_VENDOR("AuthenticAMD", ebx, ecx, edx)
@@ -752,7 +754,7 @@ ETHR_IMPL_NORETURN__ ethr_fatal_error__(const char *file,
 }
 
 ETHR_IMPL_NORETURN__
-ethr_assert_failed(const char *file, int line, const char *func, char *a)
+ethr_assert_failed(const char *file, int line, const char *func, const char *a)
 {
     fprintf(stderr, "%s:%d: %s(): Assertion failed: %s\n", file, line, func, a);
     ethr_abort__();
