@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2009-2021. All Rights Reserved.
+ * Copyright Ericsson AB 2009-2024. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -52,6 +52,7 @@
 
 #include <sched.h>
 #include <errno.h>
+#include <limits.h>
 
 #define ETHR_YIELD_AFTER_BUSY_LOOPS 50
 
@@ -92,6 +93,7 @@ wait__(ethr_event *e, int spincount, ethr_sint64_t timeout)
 #ifdef ETHR_HAVE_ETHR_GET_MONOTONIC_TIME
     ethr_sint64_t start = 0; /* SHUT UP annoying faulty warning... */
 #endif
+    int timeout_res = ETIMEDOUT;
 
     if (spincount < 0)
 	ETHR_FATAL_ERROR__(EINVAL);
@@ -131,6 +133,7 @@ wait__(ethr_event *e, int spincount, ethr_sint64_t timeout)
 	}
 
 	if (timeout >= 0) {
+            ethr_sint64_t sec, nsec;
 #ifdef ETHR_HAVE_ETHR_GET_MONOTONIC_TIME
 	    time = timeout - (ethr_get_monotonic_time() - start);
 #endif
@@ -141,8 +144,18 @@ wait__(ethr_event *e, int spincount, ethr_sint64_t timeout)
 		    goto return_event_on;
 		return ETIMEDOUT;
 	    }
-	    ts.tv_sec = time / (1000*1000*1000);
-	    ts.tv_nsec = time % (1000*1000*1000);
+            sec = time / (1000*1000*1000);
+            nsec = time % (1000*1000*1000);
+            if (sizeof(ts.tv_sec) == 8
+                || sec <= (ethr_sint64_t) INT_MAX) {
+                ts.tv_sec = sec;
+                ts.tv_nsec = nsec;
+            }
+            else {
+                ts.tv_sec = INT_MAX;
+                ts.tv_nsec = 0;
+                timeout_res = EINTR;
+            }
 	}
 
 	if (val != ETHR_EVENT_OFF_WAITER__) {
@@ -160,8 +173,10 @@ wait__(ethr_event *e, int spincount, ethr_sint64_t timeout)
 			   ETHR_EVENT_OFF_WAITER__,
 			   tsp);
 	switch (res) {
-	case EINTR:
 	case ETIMEDOUT:
+            res = timeout_res;
+            /* Fall through... */
+	case EINTR:
 	    return res;
 	case 0:
 	case EWOULDBLOCK:
@@ -189,6 +204,7 @@ return_event_on:
 #include <sys/select.h>
 #include <errno.h>
 #include <string.h>
+#include <limits.h>
 
 #include "erl_misc_utils.h"
 
@@ -358,6 +374,7 @@ wait__(ethr_event *e, int spincount, ethr_sint64_t timeout)
 #ifdef ETHR_HAVE_ETHR_GET_MONOTONIC_TIME
     ethr_sint64_t timeout_time = 0; /* SHUT UP annoying faulty warning... */
 #endif
+    int timeout_res = ETIMEDOUT;
 
     val = ethr_atomic32_read(&e->state);
     if (val == ETHR_EVENT_ON__)
@@ -452,6 +469,7 @@ wait__(ethr_event *e, int spincount, ethr_sint64_t timeout)
 #ifdef ETHR_HAVE_PTHREAD_COND_TIMEDWAIT_MONOTONIC
 	    if (timeout > 0) {
 		if (time != timeout_time) {
+                    ethr_sint64_t sec, nsec;
 		    time = timeout_time;
 
 #if ERTS_USE_PREMATURE_TIMEOUT
@@ -467,11 +485,22 @@ wait__(ethr_event *e, int spincount, ethr_sint64_t timeout)
 			time -= ERTS_PREMATURE_TIMEOUT(rtmo, 1000*1000*1000);
 		    }
 #endif
-
-		    cond_timeout.tv_sec = time / (1000*1000*1000);
-		    cond_timeout.tv_nsec = time % (1000*1000*1000);
+                    sec = time / (1000*1000*1000);
+                    nsec = time % (1000*1000*1000);
+                    if (sizeof(cond_timeout.tv_sec) == 8
+                        || sec <= (ethr_sint64_t) INT_MAX) {
+                        cond_timeout.tv_sec = sec;
+                        cond_timeout.tv_nsec = nsec;
+                    }
+                    else {
+                        cond_timeout.tv_sec = INT_MAX;
+                        cond_timeout.tv_nsec = 0;
+                        timeout_res = EINTR;
+                    }
 		}
 		res = pthread_cond_timedwait(&e->cnd, &e->mtx, &cond_timeout);
+                if (res == ETIMEDOUT)
+                    res = timeout_res;
 		if (res == EINTR
 		    || (res == ETIMEDOUT
 #if ERTS_USE_PREMATURE_TIMEOUT
