@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2014-2016. All Rights Reserved.
+%% Copyright Ericsson AB 2014-2021. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -67,7 +67,7 @@ roundtrip(T, V) ->
 roundtrip_error(T, V) ->
     try asn1_test_lib:roundtrip('InfObjExtract', T, V) of
 	ok ->
-	    test_server:fail()
+	    ct:fail(expected_exception)
     catch
 	_:_ ->
 	    ok
