@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2006-2017. All Rights Reserved.
+%% Copyright Ericsson AB 2006-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -18,11 +18,13 @@
 %% %CopyrightEnd%
 %%
 
-%%% @doc Common Test Framework functions handling test specifications.
-%%%
-%%% <p>This module exports functions that are used within CT to
-%%% scan and parse test specifikations.</p>
 -module(ct_testspec).
+-moduledoc """
+Parsing of test specifications for `Common Test`.
+
+This module exports help functions for parsing of test specifications.
+""".
+-moduledoc(#{since => "OTP 19.3"}).
 
 -export([prepare_tests/1, prepare_tests/2, 
 	 collect_tests_from_list/2, collect_tests_from_list/3,
@@ -51,6 +53,7 @@
 %%% Version 1 - extract and return all tests and skips for Node 
 %%%             (incl all_nodes)
 %%%-------------------------------------------------------------------
+-doc false.
 prepare_tests(TestSpec,Node) when is_record(TestSpec,testspec),
 				  is_atom(Node) ->
     case lists:keysearch(Node,1,prepare_tests(TestSpec)) of
@@ -67,6 +70,7 @@ prepare_tests(TestSpec,Node) when is_record(TestSpec,testspec),
 %%% and the tuples in the Skip list will have the form 
 %%% {Dir,Suites,Comment} or {Dir,Suite,Cases,Comment}. 
 %%%-------------------------------------------------------------------
+-doc false.
 prepare_tests(TestSpec) when is_record(TestSpec,testspec) ->
     Tests = TestSpec#testspec.tests,
     %% Sort Tests into "flat" Run and Skip lists (not sorted per node).
@@ -263,9 +267,11 @@ get_skipped_cases1(_,_,_,[]) ->
 
 %%% collect_tests_from_file reads a testspec file and returns a record
 %%% containing the data found.
+-doc false.
 collect_tests_from_file(Specs,Relaxed) ->
     collect_tests_from_file(Specs,[node()],Relaxed).
 
+-doc false.
 collect_tests_from_file(Specs,Nodes,Relaxed) when is_list(Nodes) ->
     NodeRefs = lists:map(fun(N) -> {undefined,N} end, Nodes),
     %% [Spec1,Spec2,...] means create one testpec record per Spec file
@@ -413,9 +419,11 @@ create_spec(Terms,TestSpec,JoinedByPrev,Relaxed) ->
     TS#testspec{tests=lists:flatten(Tests),
 		logdir=LogDirs1}.
 
+-doc false.
 collect_tests_from_list(Terms,Relaxed) ->
     collect_tests_from_list(Terms,[node()],Relaxed).
 
+-doc false.
 collect_tests_from_list(Terms,Nodes,Relaxed) when is_list(Nodes) ->
     {ok,Cwd} = file:get_cwd(),
     NodeRefs = lists:map(fun(N) -> {undefined,N} end, Nodes),
@@ -809,6 +817,20 @@ list_nodes(#testspec{nodes=NodeRefs}) ->
 %%% and tests to run/skip.
 %%% [Spec1,Spec2,...] means create separate tests per spec
 %%% [[Spec1,Spec2,...]] means merge all specs into one
+-doc """
+Parse the given test specification files and return the tests to run and skip.
+
+[](){: #add_nodes-1 }
+
+If `SpecsIn=[Spec1,Spec2,...]`, separate tests will be created per
+specification. If `SpecsIn=[[Spec1,Spec2,...]]`, all specifications will be
+merge into one test.
+
+For each test, a `{Specs,Tests}` element is returned, where `Specs` is a list of
+all included test specifications, and `Tests` specifies actual tests to run/skip
+per node.
+""".
+-doc(#{since => <<"OTP 19.3">>}).
 -spec get_tests(Specs) -> {ok,[{Specs,Tests}]} | {error,Reason} when
       Specs :: [string()] | [[string()]],
       Tests :: {Node,Run,Skip},
@@ -985,6 +1007,9 @@ add_tests([{event_handler,Node,HOrHs,Args}|Ts],Spec) ->
 add_tests([{enable_builtin_hooks,Bool}|Ts],Spec) ->
     add_tests(Ts, Spec#testspec{enable_builtin_hooks = Bool});
 
+add_tests([{ct_hooks_order,Order}|Ts],Spec) ->
+    add_tests(Ts, Spec#testspec{ct_hooks_order = Order});
+
 add_tests([{release_shell,Bool}|Ts],Spec) ->
     add_tests(Ts, Spec#testspec{release_shell = Bool});
 
@@ -1083,7 +1108,7 @@ add_tests([],Spec) ->				% done
 %% have added something of his/her own, which we'll let pass if relaxed
 %% mode is enabled.
 check_term(Term) when is_tuple(Term) ->
-    Size = size(Term),
+    Size = tuple_size(Term),
     [Name|_] = tuple_to_list(Term),
     Valid = valid_terms(),
     case lists:member({Name,Size},Valid) of
@@ -1097,7 +1122,7 @@ check_term(Term) when is_tuple(Term) ->
 		    case get(relaxed) of
 			true ->
 			    %% warn if name resembles a CT term
-			    case resembles_ct_term(Name,size(Term)) of
+                            case resembles_ct_term(Name,tuple_size(Term)) of
 				true ->
 				    io:format("~nSuspicious term, "
 					      "please check:~n"
@@ -1158,7 +1183,7 @@ handle_data(verbosity,Node,VLvls,_Spec) when is_list(VLvls) ->
     VLvls1 = lists:map(fun(VLvl = {_Cat,_Lvl}) -> VLvl;
 			  (Lvl) -> {'$unspecified',Lvl} end, VLvls),
     [{Node,VLvls1}];
-handle_data(multiply_timetraps,Node,Mult,_Spec) when is_integer(Mult) ->
+handle_data(multiply_timetraps,Node,Mult,_Spec) when is_number(Mult) ->
     [{Node,Mult}];
 handle_data(scale_timetraps,Node,Scale,_Spec) when Scale == true;
                                                    Scale == false ->
@@ -1211,6 +1236,7 @@ per_node([],_,_,_) ->
     [].
 
 %% Change the testspec record "back" to a list of tuples
+-doc false.
 testspec_rec2list(Rec) ->
     {Terms,_} = lists:mapfoldl(fun(unknown, Pos) ->
 				       {element(Pos, Rec),Pos+1};
@@ -1221,6 +1247,7 @@ testspec_rec2list(Rec) ->
 
 %% Extract one or more values from a testspec record and
 %% return the result as a list of tuples
+-doc false.
 testspec_rec2list(Field, Rec) when is_atom(Field) ->
     [Term] = testspec_rec2list([Field], Rec),
     Term;
@@ -1425,7 +1452,12 @@ skip_groups1(Suite,Groups,Cmt,Suites0) ->
 	    GrAndCases1 = GrAndCases0 ++ SkipGroups,
 	    insert_in_order({Suite,GrAndCases1},Suites0,replace);
 	false ->
-	    insert_in_order({Suite,SkipGroups},Suites0,replace)
+	    case Suites0 of
+		[{all,_}=All|Skips]->
+		    [All|Skips++[{Suite,SkipGroups}]];
+                _ ->
+                    insert_in_order({Suite,SkipGroups},Suites0,replace)
+            end
     end.
 
 skip_cases(Node,Dir,Suite,Cases,Cmt,Tests,false) when is_list(Cases) ->
@@ -1591,6 +1623,7 @@ valid_terms() ->
      {event_handler,4},
      {ct_hooks,2},
      {ct_hooks,3},
+     {ct_hooks_order,2},
      {enable_builtin_hooks,2},
      {release_shell,2},
      {multiply_timetraps,2},
