@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2008-2023. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -46,6 +46,8 @@
 -export([unique_timestamp/0]).
 
 -export([rm_dir/1]).
+%% Tracing
+-export([handle_trace/3]).
 
 -include_lib("kernel/include/file.hrl").
 
@@ -522,7 +524,7 @@ verify_events(TEvs, Evs, Node, Config) ->
 
 verify_events1([TestEv|_], [{TEH,#event{name=stop_logging,node=Node,data=_}}|_], Node, _)
   when element(1,TestEv) == TEH, element(2,TestEv) =/= stop_logging ->
-    test_server:format("Failed to find ~tp in the list of events!~n", [TestEv]),
+    test_server:format("(I) Failed to find ~tp in the list of events!~n", [TestEv]),
     exit({event_not_found,TestEv});
 
 verify_events1(TEvs = [TestEv | TestEvs], Evs = [_|Events], Node, Config) ->
@@ -545,7 +547,7 @@ verify_events1(TEvs = [TestEv | TestEvs], Evs = [_|Events], Node, Config) ->
     end;
 
 verify_events1([TestEv|_], [], _, _) ->
-    test_server:format("Failed to find ~tp in the list of events!~n", [TestEv]),
+    test_server:format("(II) Failed to find ~tp in the list of events!~n", [TestEv]),
     exit({event_not_found,TestEv});
 
 verify_events1([], Evs, _, Config) ->
@@ -1217,6 +1219,9 @@ result_match({failed,{timetrap_timeout,{'$approx',Num}}},
 result_match({user_timetrap_error,{Why,'_'}},
 	     {user_timetrap_error,{Why,_Stack}}) ->
     true;
+result_match({SkipOrFail,{ErrorInd,{thrown,{Why,'_'}}}},
+         {SkipOrFail,{ErrorInd,{thrown,{Why,_Stack}}}}) ->
+    true;
 result_match(Result, Result) ->
     true;
 result_match(_, _) ->
@@ -1466,3 +1471,14 @@ slave_stop(Node) ->
 	    receive {nodedown, Node} -> ok after 0 -> ok end %flush
     end,
     ok.
+
+%%%################################################################
+%%%#
+%%%# Tracing
+%%%#
+handle_trace(ct,
+             {return_from, {?MODULE, start_slave, 3}, Return},
+             Stack) ->
+    {io_lib:format("CT Node = ~p",
+                   [proplists:get_value(ct_node, Return, not_found)]), Stack}.
+
