@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2018. All Rights Reserved.
+%% Copyright Ericsson AB 2018-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -33,6 +33,7 @@
 %%
 
 -module(beam_ssa_share).
+-moduledoc false.
 -export([module/2,block/2]).
 
 -include("beam_ssa.hrl").
@@ -51,17 +52,31 @@ module(#b_module{body=Fs0}=Module, _Opts) ->
       Blocks0 :: beam_ssa:block_map(),
       Blk :: beam_ssa:b_blk().
 
-block(#b_blk{last=Last0}=Blk, Blocks) ->
+block(#b_blk{is=Is0,last=Last0}=Blk, Blocks) when is_map(Blocks) ->
     case share_terminator(Last0, Blocks) of
-        none -> Blk;
-        Last -> Blk#b_blk{last=beam_ssa:normalize(Last)}
+        none ->
+            Blk;
+        #b_br{succ=Same,fail=Same}=Last ->
+            %% The terminator was reduced from a two-way branch to a
+            %% one-way branch.
+            case reverse(Is0) of
+                [#b_set{op={succeeded,guard},args=[Dst]},#b_set{dst=Dst}|Is] ->
+                    %% A {succeeded,guard} instruction must not be followed by
+                    %% a one-way branch. We must remove both the succeeded
+                    %% instruction and the instruction preceding it.
+                    Blk#b_blk{is=reverse(Is),last=beam_ssa:normalize(Last)};
+                _ ->
+                    Blk#b_blk{last=beam_ssa:normalize(Last)}
+            end;
+        Last ->
+            Blk#b_blk{last=beam_ssa:normalize(Last)}
     end.
 
 %%%
 %%% Local functions.
 %%%
 
-function(#b_function{anno=Anno,bs=Blocks0}=F) ->
+function(#b_function{anno=Anno,bs=Blocks0}=F) when is_map(Blocks0) ->
     try
         PO = reverse(beam_ssa:rpo(Blocks0)),
         {Blocks1,Changed} = blocks(PO, Blocks0, false),
@@ -117,8 +132,8 @@ share_terminator(_Last, _Blocks) -> none.
 %% possible if the blocks are not equivalent, as that is the common
 %% case.
 
-are_equivalent(_Succ, _, ?BADARG_BLOCK, _, _Blocks) ->
-    %% ?BADARG_BLOCK is special. Sharing could be incorrect.
+are_equivalent(_Succ, _, ?EXCEPTION_BLOCK, _, _Blocks) ->
+    %% ?EXCEPTION_BLOCK is special. Sharing could be incorrect.
     false;
 are_equivalent(_Succ, #b_blk{is=Is1,last=#b_ret{arg=RetVal1}=Ret1},
          _Fail, #b_blk{is=Is2,last=#b_ret{arg=RetVal2}=Ret2}, _Blocks) ->
@@ -136,7 +151,7 @@ are_equivalent(_Succ, #b_blk{is=Is1,last=#b_ret{arg=RetVal1}=Ret1},
                     false
             end;
         {#b_var{},#b_var{}} ->
-            %% The return values are varibles. We must canonicalize
+            %% The return values are variables. We must canonicalize
             %% the blocks (including returns) and compare them.
             Can1 = canonical_is(Is1 ++ [Ret1]),
             Can2 = canonical_is(Is2 ++ [Ret2]),
@@ -240,6 +255,12 @@ share_switch_2([[{_,_}|_]=Prep|T], Blocks, Acc0) ->
     share_switch_2(T, Blocks, Acc);
 share_switch_2([], _, Acc) -> Acc.
 
+canonical_block({?EXCEPTION_BLOCK,_VarMap}, _Blocks) ->
+    %% Never ever share the ?EXCEPTION_BLOCK with another block.
+    %% Unless the entire switch or br is optimized away, a
+    %% {f,0} can be emitted where it is not allowed and a later
+    %% pass will crash.
+    {{none,?EXCEPTION_BLOCK},done};
 canonical_block({L,VarMap0}, Blocks) ->
     #b_blk{is=Is,last=Last0} = map_get(L, Blocks),
     case canonical_terminator(L, Last0, Blocks) of
@@ -264,8 +285,9 @@ canonical_block({L,VarMap0}, Blocks) ->
 %%    * Variables defined in the instruction sequence are replaced with
 %%    {var,0}, {var,1}, and so on. Free variables are not changed.
 %%
-%%    * `location` annotations that would produce a `line` instruction are
-%%    kept. All other annotations are cleared.
+%%    * `location` annotations that would produce `line` or
+%%    `executable_line` instructions are kept. All other annotations
+%%    are cleared.
 %%
 %%    * Instructions are repackaged into tuples instead of into the
 %%    usual records. The main reason is to avoid violating the types for
@@ -277,32 +299,27 @@ canonical_is(Is) ->
     Can.
 
 canonical_is([#b_set{op=Op,dst=Dst,args=Args0}=I|Is], VarMap0, Acc) ->
-    Args = [canonical_arg(Arg, VarMap0) || Arg <-Args0],
+    Args = [canonical_arg(Arg, VarMap0) || Arg <- Args0],
     Var = {var,map_size(VarMap0)},
     VarMap = VarMap0#{Dst=>Var},
-    LineAnno = case Op of
-                   bs_match ->
-                       %% The location annotation for a bs_match instruction
-                       %% is only used in warnings, never to emit a `line`
-                       %% instruction. Therefore, it should not be included.
-                       [];
-                   _ ->
-                       %% The location annotation will be used in a `line`
-                       %% instruction. It must be included.
-                       beam_ssa:get_anno(location, I, none)
-               end,
+    LineAnno =
+        case {Op,Is} of
+            {executable_line, _} ->
+                %% The location annotation will be used in a
+                %% `executable_line` instruction.
+                beam_ssa:get_anno(location, I, none);
+            {_, [#b_set{op={succeeded,body},args=[Dst]}|_]} ->
+                %% The location annotation will be used in a `line`
+                %% instruction.
+                beam_ssa:get_anno(location, I, none);
+            {_, _} ->
+                %% The location annotation will not be included in
+                %% any BEAM instruction.
+                []
+        end,
     canonical_is(Is, VarMap, {Op,LineAnno,Var,Args,Acc});
-canonical_is([#b_ret{arg=Arg}], VarMap, Acc0) ->
-    Acc1 = case Acc0 of
-               {call,_Anno,Var,[#b_local{}|_]=Args,PrevAcc} ->
-                   %% This is a tail-recursive call to a local function.
-                   %% There will be no line instruction generated;
-                   %% thus, the annotation is not significant.
-                   {call,[],Var,Args,PrevAcc};
-               _ ->
-                   Acc0
-           end,
-    {{ret,canonical_arg(Arg, VarMap),Acc1},VarMap};
+canonical_is([#b_ret{arg=Arg}], VarMap, Acc) ->
+    {{ret,canonical_arg(Arg, VarMap),Acc},VarMap};
 canonical_is([#b_br{bool=#b_var{}=Arg,fail=Fail}], VarMap, Acc) ->
     %% A previous buggy version of this code omitted the canonicalized
     %% argument in the return value. Unfortunately, that worked most
@@ -311,6 +328,19 @@ canonical_is([#b_br{bool=#b_var{}=Arg,fail=Fail}], VarMap, Acc) ->
     {{br,canonical_arg(Arg, VarMap),succ,Fail,Acc},VarMap};
 canonical_is([#b_br{succ=Succ}], VarMap, Acc) ->
     {{br,Succ,Acc},VarMap};
+canonical_is([{tail_br,Arg0,Ret0}], VarMap, Acc0) ->
+    Arg = canonical_arg(Arg0, VarMap),
+    Ret = canonical_arg(Ret0, VarMap),
+    case Acc0 of
+        {{succeeded,body},_,Arg,[Ret],{call,_,Ret,CallArgs,PrevAcc}} ->
+            %% This is a tail-recursive call to a local function.
+            %% There will be no line instruction generated; thus, the
+            %% annotation is not significant.
+            {{tail_call,Ret,CallArgs,PrevAcc},VarMap};
+        _ ->
+            %% Not a tail-recursive call.
+            {{br_ret,Arg,Ret,Acc0},VarMap}
+    end;
 canonical_is([], VarMap, Acc) ->
     {Acc,VarMap}.
 
@@ -324,6 +354,14 @@ canonical_terminator(L, #b_br{bool=#b_literal{val=true},succ=Succ}=Br, Blocks) -
         [_|_]=Phis ->
             {Phis ++ [Br],done}
     end;
+canonical_terminator(_L, #b_br{bool=#b_var{}=Arg,succ=Succ,fail=?EXCEPTION_BLOCK}=Br,
+                     Blocks) ->
+    case Blocks of
+        #{Succ := #b_blk{is=[],last=#b_ret{arg=Ret}}} ->
+            {[{tail_br,Arg,Ret}],done};
+        #{} ->
+            {[Br],Succ}
+    end;
 canonical_terminator(_L, #b_br{bool=#b_var{},succ=Succ}=Br, _Blocks) ->
     {[Br],Succ};
 canonical_terminator(_, _, _) -> none.
@@ -331,11 +369,16 @@ canonical_terminator(_, _, _) -> none.
 canonical_terminator_phis([#b_set{op=phi,args=PhiArgs}=Phi|Is], L) ->
     {Value,L} = keyfind(L, 2, PhiArgs),
     [Phi#b_set{op=copy,args=[Value]}|canonical_terminator_phis(Is, L)];
-canonical_terminator_phis([#b_set{op=peek_message}=I|_], L) ->
-    %% We could get stuck into an infinite loop if we allowed the
-    %% comparisons to continue into this block. Force an unequal
-    %% compare with all other predecessors of this block.
-    [I#b_set{op=copy,args=[#b_literal{val=L}]}];
+canonical_terminator_phis([#b_set{}=I|_], L) ->
+    case beam_ssa:is_loop_header(I) of
+        true ->
+            %% We could get stuck into an infinite loop if we allowed the
+            %% comparisons to continue into this loop. Force an unequal
+            %% compare with all other predecessors of this block.
+            [I#b_set{op=copy,args=[#b_literal{val=L}]}];
+        false ->
+            []
+    end;
 canonical_terminator_phis(_, _) -> [].
 
 canonical_arg(#b_var{}=Var, VarMap) ->
@@ -368,7 +411,9 @@ shortcut_nonempty_block(L, Blocks) ->
 
 is_forbidden(L, Blocks) ->
     case map_get(L, Blocks) of
-        #b_blk{is=[#b_set{op=phi}|_]} -> true;
-        #b_blk{is=[#b_set{op=peek_message}|_]} -> true;
+        #b_blk{is=[#b_set{op=phi}|_]} ->
+            true;
+        #b_blk{is=[#b_set{}=I|_]} ->
+            beam_ssa:is_loop_header(I);
         #b_blk{} -> false
     end.
