@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1999-2018. All Rights Reserved.
+%% Copyright Ericsson AB 1999-2024. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 %% Purpose : Core Erlang (naive) prettyprinter
 
 -module(core_pp).
+-moduledoc false.
 
 -export([format/1,format_all/1]).
 
@@ -101,6 +102,8 @@ format_anno_list([H], Ctxt) ->
 
 strip_line([A | As]) when is_integer(A) ->
     strip_line(As);
+strip_line([{A,C} | As]) when is_integer(A), is_integer(C) ->
+    strip_line(As);
 strip_line([{file,_File} | As]) ->
     strip_line(As);
 strip_line([A | As]) ->
@@ -110,6 +113,8 @@ strip_line([]) ->
 
 get_line([L | _As]) when is_integer(L) ->
     L;
+get_line([{L, _Column} | _As]) when is_integer(L) ->
+    L;
 get_line([_ | As]) ->
     get_line(As);
 get_line([]) ->
@@ -130,11 +135,10 @@ format_1(#c_literal{anno=A,val=Bitstring}, Ctxt) when is_bitstring(Bitstring) ->
     Segs = segs_from_bitstring(Bitstring),
     format_1(#c_binary{anno=A,segments=Segs}, Ctxt);
 format_1(#c_literal{anno=A,val=M},Ctxt) when is_map(M) ->
-    Pairs = maps:to_list(M),
     Op = #c_literal{val=assoc},
     Cpairs = [#c_map_pair{op=Op,
 			  key=#c_literal{val=K},
-			  val=#c_literal{val=V}} || {K,V} <- Pairs],
+			  val=#c_literal{val=V}} || K := V <- M],
     format_1(#c_map{anno=A,arg=#c_literal{val=#{}},es=Cpairs},Ctxt);
 format_1(#c_literal{val=F},_Ctxt) when is_function(F) ->
     {module,M} = erlang:fun_info(F, module),
@@ -355,7 +359,9 @@ format_1(#c_module{name=N,exports=Es,attrs=As,defs=Ds}, Ctxt) ->
      format_funcs(Ds, Ctxt),
      nl_indent(Ctxt)
      | "end"
-    ].
+    ];
+format_1(#c_opaque{val=V}, Ctxt) ->
+    ["%% Opaque: ", format_1(#c_literal{val=V}, Ctxt)].
 
 format_funcs(Fs, Ctxt) ->
     format_vseq(Fs,
