@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2003-2022. All Rights Reserved.
+%% Copyright Ericsson AB 2003-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -90,22 +90,22 @@ opt_opts(Mod) ->
                      ({feature,_,enable}) -> true;
                      ({feature,_,disable}) -> true;
                      (inline) -> true;
+                     (line_coverage) -> true;
+                     (no_badrecord) -> true;
                      (no_bs_create_bin) -> true;
                      (no_bsm_opt) -> true;
+                     (no_bs_match) -> true;
                      (no_copt) -> true;
                      (no_fun_opt) -> true;
-                     (no_init_yregs) -> true;
-                     (no_make_fun3) -> true;
+                     (no_min_max_bifs) -> true;
                      (no_module_opt) -> true;
                      (no_postopt) -> true;
                      (no_recv_opt) -> true;
                      (no_share_opt) -> true;
-                     (no_shared_fun_wrappers) -> true;
                      (no_ssa_opt_float) -> true;
                      (no_ssa_opt_ranges) -> true;
                      (no_ssa_opt) -> true;
                      (no_stack_trimming) -> true;
-                     (no_swap) -> true;
                      (no_type_opt) -> true;
                      (_) -> false
                 end, Opts).
@@ -119,11 +119,13 @@ get_data_dir(Config) ->
     Opts = [{return,list}],
     Suffixes = ["_no_opt_SUITE",
                 "_no_copt_SUITE",
+                "_no_copt_ssa_SUITE",
                 "_post_opt_SUITE",
                 "_inline_SUITE",
                 "_no_module_opt_SUITE",
                 "_no_type_opt_SUITE",
-                "_no_ssa_opt_SUITE"],
+                "_no_ssa_opt_SUITE",
+                "_cover_SUITE"],
     lists:foldl(fun(Suffix, Acc) ->
                         Opts = [{return,list}],
                         re:replace(Acc, Suffix, "_SUITE", Opts)
@@ -138,11 +140,13 @@ is_cloned_mod(Mod) ->
 
 is_cloned_mod_1("_no_opt_SUITE") -> true;
 is_cloned_mod_1("_no_copt_SUITE") -> true;
+is_cloned_mod_1("_no_copt_ssa_SUITE") -> true;
 is_cloned_mod_1("_no_ssa_opt_SUITE") -> true;
 is_cloned_mod_1("_no_type_opt_SUITE") -> true;
 is_cloned_mod_1("_post_opt_SUITE") -> true;
 is_cloned_mod_1("_inline_SUITE") -> true;
 is_cloned_mod_1("_no_module_opt_SUITE") -> true;
+is_cloned_mod_1("_cover_SUITE") -> true;
 is_cloned_mod_1([_|T]) -> is_cloned_mod_1(T);
 is_cloned_mod_1([]) -> false.
 
@@ -159,14 +163,15 @@ highest_opcode(Beam) ->
 
 p_run(Test, List) ->
     %% Limit the number of parallel processes to avoid running out of
-    %% memory.
-    S = case {erlang:system_info(schedulers),erlang:system_info(wordsize)} of
-            {S0,4} ->
-                min(S0, 2);
-            {S0,8} ->
-                min(S0, 8)
+    %% virtual address space or memory. This is especially important
+    %% on 32-bit Windows, where only 2 GB of virtual address space is
+    %% available.
+    N = case {erlang:system_info(schedulers),erlang:system_info(wordsize)} of
+            {_,4} ->
+                1;
+            {N0,8} ->
+                min(N0, 8)
         end,
-    N = S + 1,
     p_run(Test, List, N).
 
 p_run(Test, List, N) ->
