@@ -20,6 +20,7 @@
 %% @author Kostis Sagonas <kostis@it.uu.se>
 
 -module(erl_bif_types).
+-moduledoc false.
 
 -define(BITS, 128). %This is only in bsl to convert answer to pos_inf/neg_inf.
 -export([type/3, type/4, type/5, arg_types/3,
@@ -67,10 +68,10 @@
 		    t_is_cons/2,
 		    t_is_float/2,
 		    t_is_fun/2,
+		    t_is_impossible/1,
 		    t_is_integer/2,
 		    t_is_nil/1, t_is_nil/2,
 		    t_is_none/1,
-		    t_is_none_or_unit/1,
 		    t_is_number/2,
 		    t_is_pid/2,
 		    t_is_port/2,
@@ -772,6 +773,10 @@ type(erlang, length, 1, Xs, Opaques) ->
 %% Guard bif, needs to be here.
 type(erlang, map_size, 1, Xs, Opaques) ->
   type(maps, size, 1, Xs, Opaques);
+type(erlang, max, 2, Xs, Opaques) ->
+  strict(erlang, max, 2, Xs,
+         fun([A, B]) -> t_sup(A, B) end,
+         Opaques);
 %% Guard bif, needs to be here.
 type(erlang, map_get, 2, Xs, Opaques) ->
   type(maps, get, 2, Xs, Opaques);
@@ -803,6 +808,10 @@ type(erlang, make_tuple, 3, Xs, Opaques) ->
 	       _Other -> t_tuple()
 	     end
 	 end, Opaques);
+type(erlang, min, 2, Xs, Opaques) ->
+  strict(erlang, min, 2, Xs,
+         fun([A, B]) -> t_sup(A, B) end,
+         Opaques);
 type(erlang, nif_error, 1, Xs, Opaques) ->
   %% this BIF and the next one are stubs for NIFs and never return
   strict(erlang, nif_error, 1, Xs, fun (_) -> t_any() end, Opaques);
@@ -813,6 +822,13 @@ type(erlang, node, 0, _, _Opaques) -> t_node();
 %% Guard bif, needs to be here.
 type(erlang, node, 1, Xs, Opaques) ->
   strict(erlang, node, 1, Xs, fun (_) -> t_node() end, Opaques);
+type(erlang, raise, 3, Xs, Opaques) ->
+  Ts = arg_types(erlang, raise, 3),
+  Xs1 = inf_lists(Xs, Ts, Opaques),
+  case any_is_none_or_unit(Xs1) of
+    true -> t_atom('badarg');
+    false -> t_none()
+  end;
 %% Guard bif, needs to be here.
 type(erlang, round, 1, Xs, Opaques) ->
   strict(erlang, round, 1, Xs, fun (_) -> t_integer() end, Opaques);
@@ -902,8 +918,10 @@ type(erlang, system_info, 1, Xs, Opaques) ->
 		     t_internal_cpu_topology();
 		   ['loaded'] ->
 		     t_binary();
-		   ['logical_processors'] ->
-		     t_non_neg_fixnum();
+		   [P] when P == 'logical_processors'
+		        orelse P == 'logical_processors_available'
+		        orelse P == 'logical_processors_online' ->
+		     t_sup([t_non_neg_fixnum(),t_atom('unknown')]);
 		   ['machine'] ->
 		     t_string();
 		   ['multi_scheduling'] ->
@@ -1665,7 +1683,7 @@ list_replace(1, E, [_X | Xs]) ->
   [E | Xs].
 
 any_is_none_or_unit(Ts) ->
-  lists:any(fun erl_types:t_is_none_or_unit/1, Ts).
+  lists:any(fun erl_types:t_is_impossible/1, Ts).
 
 check_guard([X], Test, Type, Opaques) ->
   check_guard_single(X, Test, Type, Opaques).
@@ -2290,6 +2308,8 @@ arg_types(erlang, is_tuple, 1) ->
 %% Guard bif, needs to be here.
 arg_types(erlang, length, 1) ->
   [t_list()];
+arg_types(erlang, max, 2) ->
+  [t_any(), t_any()];
 %% Guard bif, needs to be here.
 arg_types(erlang, map_size, 1) ->
   [t_map()];
@@ -2302,6 +2322,8 @@ arg_types(erlang, make_tuple, 2) ->
   [t_non_neg_fixnum(), t_any()];  % the value 0 is OK as first argument
 arg_types(erlang, make_tuple, 3) ->
   [t_non_neg_fixnum(), t_any(), t_list(t_tuple([t_pos_integer(), t_any()]))];
+arg_types(erlang, min, 2) ->
+  [t_any(), t_any()];
 arg_types(erlang, nif_error, 1) ->
   [t_any()];
 arg_types(erlang, nif_error, 2) ->
@@ -2312,6 +2334,20 @@ arg_types(erlang, node, 0) ->
 %% Guard bif, needs to be here.
 arg_types(erlang, node, 1) ->
   [t_identifier()];
+arg_types(erlang, raise, 3) ->
+  %% The types are based on the implementation of erlang:raise/3, not
+  %% the documention. That is, those are the types for which
+  %% erlang:raise/3 will raise an exception. If the arguments do not
+  %% conform to those types, erlang:raise/3 will return 'badarg'.
+  Class = t_sup([t_atom('error'), t_atom('exit'), t_atom('throw')]),
+  InfoList = t_list(),                 %erlang:raise/3 does not check the contents of the list.
+  ArityOrArgs = t_any(),               %erlang:raise/3 does not check the type.
+  MFA = t_tuple([t_module(), t_atom(), ArityOrArgs]),
+  MFAS = t_tuple([t_module(), t_atom(), ArityOrArgs, InfoList]),
+  Fun = t_tuple([t_fun(), ArityOrArgs]),
+  FunI = t_tuple([t_fun(), ArityOrArgs, InfoList]),
+  Stack = t_list(t_sup([MFA, MFAS, Fun, FunI])),
+  [Class, t_any(), Stack];
 %% Guard bif, needs to be here.
 arg_types(erlang, round, 1) ->
   [t_number()];
@@ -2532,7 +2568,7 @@ check_fun_application(Fun, Args, Opaques) ->
     true ->
       case t_fun_args(Fun, Opaques) of
 	unknown ->
-	  case t_is_none_or_unit(t_fun_range(Fun, Opaques)) of
+	  case t_is_impossible(t_fun_range(Fun, Opaques)) of
 	    true -> error;
 	    false -> ok
 	  end;
@@ -2540,7 +2576,7 @@ check_fun_application(Fun, Args, Opaques) ->
 	  case any_is_none_or_unit(inf_lists(FunDom, Args, Opaques)) of
 	    true -> error;
 	    false ->
-	      case t_is_none_or_unit(t_fun_range(Fun, Opaques)) of
+	      case t_is_impossible(t_fun_range(Fun, Opaques)) of
 		true -> error;
 		false -> ok
 	      end
@@ -2551,7 +2587,6 @@ check_fun_application(Fun, Args, Opaques) ->
       error
   end.
 
-
 %% =====================================================================
 %% Some basic types used in various parts of the system
 %% =====================================================================
