@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2010-2017. All Rights Reserved.
+%% Copyright Ericsson AB 2010-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -36,6 +36,7 @@
 %%
 
 -module(diameter_config).
+-moduledoc false.
 -behaviour(gen_server).
 
 -export([start_service/2,
@@ -46,6 +47,10 @@
          lookup/1,
          subscribe/2]).
 
+-export([
+         which_transports/0, which_transports/1
+        ]).
+
 %% server start
 -export([start_link/0,
          start_link/1]).
@@ -68,6 +73,7 @@
 -include_lib("diameter/include/diameter.hrl").
 -include("diameter_internal.hrl").
 
+
 %% Server state.
 -record(state, {id = diameter_lib:now(),
                 role :: server | transport}).
@@ -127,6 +133,7 @@
 %%% The return values below assume the server diameter_config is started.
 %%% The functions will exit if it isn't.
 
+
 %% --------------------------------------------------------------------------
 %% # start_service/2
 %% --------------------------------------------------------------------------
@@ -141,7 +148,7 @@ start_service(SvcName, Opts)
 
 start_rc({ok = T, _Pid}) ->
     T;
-start_rc({error, _} = No) ->
+start_rc({error, _R} = No) ->
     No;
 start_rc(timeout) ->
     {error, application_not_started}.
@@ -232,6 +239,7 @@ pred(_) ->
 subscribe(Ref, T) ->
     diameter_reg:subscribe(?TRANSPORT_KEY(Ref), T).
 
+
 %% --------------------------------------------------------------------------
 %% # have_transport/2
 %%
@@ -246,6 +254,32 @@ have_transport(SvcName, Ref) ->
                           {'=:=', '$2', {const, Ref}}}],
              [true]}]).
 
+
+%% --------------------------------------------------------------------------
+%% # which_transports/0,1
+%% --------------------------------------------------------------------------
+
+which_transports() ->
+    MatchHead = #transport{service = '$1',
+                           ref     = '$2',
+                           type    = '$3',
+                           _       = '_'},
+    Guard     = [],
+    Return    = [{{'$2', '$3', '$1'}}],
+    [#{ref => Ref, type => Type, service => Service} ||
+        {Ref, Type, Service} <- select([{MatchHead, Guard, Return}])].
+
+which_transports(SvcName) ->
+    MatchHead = #transport{service = '$1',
+                           ref     = '$2',
+                           type    = '$3',
+                           _       = '_'},
+    Guard     = [{'=:=', '$1', {const, SvcName}}],
+    Return    = [{{'$2', '$3'}}],
+    [#{ref => Ref, type => Type} ||
+        {Ref, Type} <- select([{MatchHead, Guard, Return}])].
+
+
 %% --------------------------------------------------------------------------
 %% # lookup/1
 %% --------------------------------------------------------------------------
@@ -261,6 +295,7 @@ lookup(SvcName) ->
              [{'=:=', '$1', {const, SvcName}}],
              [{{'$2', '$3', '$4'}}]}]).
 
+
 %% ---------------------------------------------------------
 %% EXPORTED INTERNAL FUNCTIONS
 %% ---------------------------------------------------------
@@ -291,21 +326,14 @@ init([]) ->
 
 %% Child start as a consequence of add_transport.
 init({SvcName, Type, Opts}) ->
-    Res = try
-              add(SvcName, Type, Opts)
-          catch
-              ?FAILURE(Reason) -> {error, Reason}
-          end,
-    proc_lib:init_ack({ok, self(), Res}),
-    loop(Res).
-
-%% loop/1
-
-loop({ok, _}) ->
-    gen_server:enter_loop(?MODULE, [], #state{role = transport});
-
-loop({error, _}) ->
-    ok.  %% die
+    try add(SvcName, Type, Opts) of
+        Ref ->
+            proc_lib:init_ack({ok, self(), Ref}),
+            gen_server:enter_loop(?MODULE, [], #state{role = transport})
+    catch
+        ?FAILURE(Reason) ->
+            proc_lib:init_fail({error, Reason}, {exit, normal})  %% die
+    end.
 
 %%% ----------------------------------------------------------
 %%% # handle_call/2
@@ -443,8 +471,12 @@ sync({stop_service, SvcName}) ->
 %% This is to provide a way for processes to to be notified when the
 %% configuration is removed (diameter_reg:subscribe/2).
 sync({add, SvcName, Type, Opts}) ->
-    {ok, _Pid, Res} = diameter_config_sup:start_child({SvcName, Type, Opts}),
-    Res;
+    case diameter_config_sup:start_child({SvcName, Type, Opts}) of
+        {ok, _Pid, Ref} ->
+            {ok, Ref};
+        {error, _} = Res ->
+            Res
+    end;
 
 sync({remove, SvcName, Pred}) ->
     Recs = select([{#transport{service = '$1', _ = '_'},
@@ -545,16 +577,12 @@ add(SvcName, Type, Opts0) ->
     %% The call to the service returns error if the service isn't
     %% started yet, which is harmless. The transport will be started
     %% when the service is in that case.
-    case start_transport(SvcName, T) of
-        ok ->
-            insert(#transport{service = SvcName,
-                              ref = Ref,
-                              type = Type,
-                              options = Opts}),
-            {ok, Ref};
-        {error, _} = No ->
-            No
-    end.
+    start_transport(SvcName, T),
+    insert(#transport{service = SvcName,
+                      ref = Ref,
+                      type = Type,
+                      options = Opts}),
+    Ref.
 
 transport_opts(Opts) ->
     [setopt(transport, T) || T <- Opts].
@@ -633,8 +661,8 @@ opt(service, {K, F})
         Nodes ->
             is_list(Nodes) orelse {error, Nodes}
     catch
-        E:R ->
-            {error, {E, R, ?STACK}}
+        E:R:Stack ->
+            {error, {E, R, Stack}}
     end;
 
 opt(service, {sequence, {H,N}}) ->
@@ -651,16 +679,22 @@ opt(service = S, {sequence = K, F}) ->
         V ->
             {error, V}
     catch
-        E:R ->
-            {error, {E, R, ?STACK}}
+        E:R:Stack ->
+            {error, {E, R, Stack}}
     end;
 
+opt(service, {bins_info, BI}) ->
+    is_boolean(BI) orelse (is_integer(BI) andalso (BI >= 0));
+
 opt(transport, {transport_module, M}) ->
     is_atom(M);
 
 opt(transport, {transport_config, _, Tmo}) ->
     ?IS_UINT32(Tmo) orelse Tmo == infinity;
 
+opt(transport, {transport_config, _}) ->
+    true;
+
 opt(transport, {applications, As}) ->
     is_list(As);
 
@@ -720,15 +754,16 @@ opt(_, {K, _})
   when K == disconnect_cb;
        K == capabilities_cb ->
     true;
-opt(transport, {K, _})
-  when K == transport_config;
-       K == private ->
+opt(transport, {private, _}) ->
     true;
 
-%% Anything else, which is ignored in transport config. This makes
-%% options sensitive to spelling mistakes, but arbitrary options are
-%% passed by some users as a way to identify transports so can't just
-%% do away with it.
+%% Anything else is ignored in transport config. This makes options
+%% sensitive to spelling mistakes and unintentionally passing service
+%% options, but arbitrary options are passed by some users as a way to
+%% identify transports (they can be returned by diameter:service_info/2
+%% for example) so can't just do away with it, although silently
+%% swallowing service options is at least debatable. The documentation
+%% says anything, so accept anything.
 opt(K, _) ->
     K == transport.
 
@@ -747,8 +782,8 @@ start_transport(SvcName, T) ->
             ok;
         {error, no_service} ->
             ok;
-        {error, _} = No ->
-            No
+        {error, Reason} ->
+            ?THROW(Reason)
     end.
 
 %% remove/2
@@ -788,6 +823,7 @@ stop_transport(SvcName, Refs) ->
 %% make_config/2
 
 make_config(SvcName, Opts) ->
+
     AppOpts = [T || {application, _} = T <- Opts],
     Apps = [init_app(T) || T <- AppOpts],
 
@@ -805,10 +841,14 @@ make_config(SvcName, Opts) ->
 
     D = proplists:get_value(string_decode, SvcOpts, true),
 
-    #service{name = SvcName,
-             rec = #diameter_service{applications = Apps,
-                                     capabilities = binary_caps(Caps, D)},
-             options = SvcOpts}.
+    Service =
+        #service{name = SvcName,
+                 rec = #diameter_service{applications = Apps,
+                                         capabilities = binary_caps(Caps, D)},
+                 options = SvcOpts},
+
+    Service.
+
 
 binary_caps(Caps, true) ->
     Caps;
@@ -866,6 +906,7 @@ init_app({application, Opts} = T) ->
     M = get_opt(call_mutates_state, Opts, false, [true]),
     A = get_opt(answer_errors, Opts, discard, [callback, report]),
     P = get_opt(request_errors, Opts, answer_3xxx, [answer, callback]),
+
     #diameter_app{alias = Alias,
                   dictionary = Dict,
                   id = cb(Dict, id),
@@ -932,8 +973,8 @@ cb(M,F) ->
     try M:F() of
         V -> V
     catch
-        E: Reason ->
-            ?THROW({callback, E, Reason, ?STACK})
+        E: Reason: Stack ->
+            ?THROW({callback, E, Reason, Stack})
     end.
 
 %% call/1
