@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2010-2017. All Rights Reserved.
+%% Copyright Ericsson AB 2010-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@
 %%
 
 -module(diameter_watchdog).
+-moduledoc false.
 -behaviour(gen_server).
 
 %% towards diameter_service
@@ -72,12 +73,11 @@
                      restrict := boolean(),
                      suspect := non_neg_integer(), %% OKAY -> SUSPECT
                      okay := non_neg_integer()},   %% REOPEN -> OKAY
-         codec :: #{string_decode := false,
+         codec :: #{decode_format := none,
+                    string_decode := false,
                     strict_mbit := boolean(),
-                    failed_avp := false,
                     rfc := 3588 | 6733,
-                    ordered_encode := false,
-                    incoming_maxlen := diameter:message_length()},
+                    ordered_encode := false},
          shutdown = false :: boolean()}).
 
 %% ---------------------------------------------------------------------------
@@ -135,13 +135,6 @@ i({Ack, T, Pid, {Opts,
     putr(restart, {T, Opts, Svc, SvcOpts}),  %% save seeing it in trace
     putr(dwr, dwr(Caps)),                    %%
     Nodes = restrict_nodes(Restrict),
-    CodecKeys = [string_decode,
-                 strict_mbit,
-                 incoming_maxlen,
-                 spawn_opt,
-                 rfc,
-                 ordered_encode],
-
     #watchdog{parent = Pid,
               transport = start(T, Opts, SvcOpts, Nodes, Dict0, Svc),
               tw = proplists:get_value(watchdog_timer,
@@ -149,14 +142,23 @@ i({Ack, T, Pid, {Opts,
                                        ?DEFAULT_TW_INIT),
               receive_data = RecvData,
               dictionary = Dict0,
-              config =
-                  maps:without(CodecKeys,
-                               config(SvcOpts#{restrict => restrict(Nodes),
-                                               suspect => 1,
-                                               okay => 3},
-                                      Opts)),
-              codec = maps:with(CodecKeys, SvcOpts#{string_decode := false,
-                                                    ordered_encode => false})}.
+              config = maps:with([sequence,
+                                  restrict_connections,
+                                  restrict,
+                                  suspect,
+                                  okay],
+                                 config(SvcOpts#{restrict => restrict(Nodes),
+                                                 suspect => 1,
+                                                 okay => 3},
+                                        Opts)),
+              codec = maps:with([decode_format,
+                                 strict_mbit,
+                                 string_decode,
+                                 rfc,
+                                 ordered_encode],
+                                SvcOpts#{decode_format := none,
+                                         string_decode := false,
+                                         ordered_encode => false})}.
 
 wait(Ref, Pid) ->
     receive
