@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  * 
- * Copyright Ericsson AB 1998-2018. All Rights Reserved.
+ * Copyright Ericsson AB 1998-2021. All Rights Reserved.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -100,7 +100,7 @@ typedef unsigned long AsciiWord;
 #elif SIZEOF_VOID_P == SIZEOF_LONG_LONG
 typedef unsigned long long AsciiWord;
 #else
-#  error "Uknown word type"
+#  error "Unknown word type"
 #endif
 
 #if SIZEOF_VOID_P == 4
