@@ -25,6 +25,7 @@
 %% @doc Test running functionality
 
 -module(eunit_test).
+-moduledoc false.
 
 -export([run_testfun/1, mf_wrapper/2, enter_context/4, multi_setup/1]).
 
@@ -164,7 +165,7 @@ macro_test_() ->
  		 {ok, ok} = run_testfun(F)
  	     end),
       ?_test(begin
- 		 {?LINE, F} = ?_assertEqual(3, 1+1),
+ 		 {?LINE, F} = ?_assertEqual(id(3), id(1+1)),
  		 {error,{error,{assertEqual_failed,
 				[{module,_},
 				 {line,_},
@@ -175,7 +176,7 @@ macro_test_() ->
 		     = run_testfun(F)
  	     end),
       ?_test(begin
-		 {?LINE, F} = ?_assertNotEqual(1, 0),
+		 {?LINE, F} = ?_assertNotEqual(id(1), id(0)),
 		 {ok, ok} = run_testfun(F)
 	     end),
       ?_test(begin
@@ -402,4 +403,7 @@ multi_setup([{Tag, S} | Es], CleanupPrev) ->
 multi_setup([], CleanupAll) ->
     {fun (Rs) -> Rs end, CleanupAll}.
 
+id(I) ->
+    I.
+
 ok(_) -> ok.
