@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2004-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2004-2023. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -131,8 +131,8 @@ pid_roundtrip(Config) when is_list(Config)->
 	       || Cr <- [1,2,3,4,16#adec0ded],
 		  {Num, Ser} <- [{4711,4711},{32767, 8191}]],
     do_echo([self(),
-	     mk_pid(ThisNode, 4711, 4711),
-	     mk_pid(ThisNode, 32767, 8191)
+	     mk_pid(ThisNode, 4711, 0),
+	     mk_pid(ThisNode, 1 bsl 27, 0)
 	     | RemPids],
 	    Config).
 
