@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1998-2022. All Rights Reserved.
+%% Copyright Ericsson AB 1998-2024. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -18,6 +18,54 @@
 %% %CopyrightEnd%
 %%
 -module(global_group).
+-moduledoc """
+Grouping nodes to global name registration groups.
+
+This module makes it possible to partition the nodes of a system into _global
+groups_. Each global group has its own global namespace, see `m:global`.
+
+The main advantage of dividing systems into global groups is that the background
+load decreases while the number of nodes to be updated is reduced when
+manipulating globally registered names.
+
+The Kernel configuration parameter [`global_groups`](kernel_app.md#global_groups)
+defines the global groups:
+
+```erlang
+{global_groups, [GroupTuple :: group_tuple()]}
+```
+
+For the processes and nodes to run smoothly using the global group
+functionality, the following criteria must be met:
+
+- An instance of the global group server, `global_group`, must be running on
+  each node. The processes are automatically started and synchronized when a
+  node is started.
+- All involved nodes must agree on the global group definition, otherwise the
+  behavior of the system is undefined.
+- _All_ nodes in the system must belong to exactly one global group.
+
+In the following descriptions, a _group node_ is a node belonging to the same
+global group as the local node.
+
+## Notes
+
+- In the situation where a node has lost its connections to other nodes in its
+  global group, but has connections to nodes in other global groups, a request
+  from another global group can produce an incorrect or misleading result. For
+  example, the isolated node can have inaccurate information about registered
+  names in its global group.
+- Function [`send/2,3`](`send/2`) is not secure.
+- Distribution of applications is highly dependent of the global group
+  definitions. It is not recommended that an application is distributed over
+  many global groups, as the registered names can be moved to another global
+  group at failover/takeover. Nothing prevents this to be done, but the
+  application code must then handle the situation.
+
+## See Also
+
+`m:global`, [`erl`](`e:erts:erl_cmd.md`)
+""".
 
 %% Groups nodes into global groups with an own global name space.
 
@@ -62,10 +110,24 @@
 
 %%%====================================================================================
 
+-doc """
+A node started with command-line flag `-hidden` (see
+[`erl`](`e:erts:erl_cmd.md`)) is said to be a _hidden_ node. A hidden node
+establishes hidden connections to nodes not part of the same global group, but
+normal (visible) connections to nodes part of the same global group.
+
+A global group defined with `PublishType` equal to `hidden` is said to be a
+hidden global group. All nodes in a hidden global group are hidden nodes,
+whether they are started with command-line flag `-hidden` or not.
+""".
 -type publish_type() :: 'hidden' | 'normal'.
 -type sync_state()   :: 'no_conf' | 'synced'.
 
 -type group_name()  :: atom().
+-doc """
+A `GroupTuple` without `PublishType` is the same as a `GroupTuple` with
+`PublishType` equal to `normal`.
+""".
 -type group_tuple() :: {GroupName :: group_name(), [node()]}
                      | {GroupName :: group_name(),
                         PublishType :: publish_type(),
@@ -125,12 +187,27 @@
 %%% External exported
 %%%====================================================================================
 
+-doc """
+Returns a tuple containing the name of the global group that the local node
+belongs to, and the list of all other known group names.
+
+Returns `undefined` if no global groups are defined.
+""".
 -spec global_groups() -> {GroupName, GroupNames} | undefined when
       GroupName :: group_name(),
       GroupNames :: [GroupName].
 global_groups() ->
     request(global_groups).
 
+-doc """
+Alter the calling process' subscription of node status change messages.
+
+If `Flag` is equal to `true` the calling process starts subscribing to
+node status change messages. If equal to `false` it stops subscribing.
+
+A process that has subscribed receives the messages `{nodeup, Node}` and
+`{nodedown, Node}` when a group node connects or disconnects, respectively.
+""".
 -spec monitor_nodes(Flag) -> 'ok' when
       Flag :: boolean().
 monitor_nodes(Flag) -> 
@@ -140,26 +217,47 @@ monitor_nodes(Flag) ->
 	_ -> {error, not_boolean}
     end.
 
+-doc "Returns the names of all group nodes, regardless of their current status.".
 -spec own_nodes() -> Nodes when
       Nodes :: [Node :: node()].
 own_nodes() ->
     request(own_nodes).
 
+-doc "A registered name.".
 -type name()  :: atom().
 -type where() :: {'node', node()} | {'group', group_name()}.
 
+-doc """
+Returns a list of all names that are globally registered on the specified node
+or in the specified global group.
+""".
 -spec registered_names(Where) -> Names when
       Where :: where(),
       Names :: [Name :: name()].
 registered_names(Arg) ->
     request({registered_names, Arg}).
 
+-doc """
+Sends `Msg` to the pid represented by the globally registered name `Name`.
+
+`send/2` searches for `Name` any any global group. The global groups are searched
+in the order that they appear in the value of configuration parameter
+[`global_groups`](kernel_app.md#global_groups).
+
+If `Name` is found, message `Msg` is sent to the corresponding pid. The pid is
+also the return value of the function. If the name is not found, the function
+returns `{badarg, {Name, Msg}}`.
+""".
 -spec send(Name, Msg) -> pid() | {'badarg', {Name, Msg}} when
       Name :: name(),
       Msg :: term().
 send(Name, Msg) ->
     request({send, Name, Msg}).
 
+-doc """
+Equivalent to [`send(Name, Msg)`](`send/2`) except that he search is limited
+to the node or global group specified by `Where`.
+""".
 -spec send(Where, Name, Msg) -> pid() | {'badarg', {Name, Msg}} when
       Where :: where(),
       Name :: name(),
@@ -167,26 +265,53 @@ send(Name, Msg) ->
 send(Group, Name, Msg) ->
     request({send, Group, Name, Msg}).
 
+-doc """
+Searched for `Name` in any global group.
+
+The global groups are searched in the order that they appear in the value
+of configuration parameter `global_groups`.
+
+If `Name` is found, the corresponding pid is returned. If the name is not found,
+the function returns `undefined`.
+""".
 -spec whereis_name(Name) -> pid() | 'undefined' when
       Name :: name().
 whereis_name(Name) ->
     request({whereis_name, Name}).
 
+-doc """
+Equivalent to [`whereis_name(Name)`](`whereis_name/1`) except that he search is limited
+to the node or global group specified by `Where`.
+""".
 -spec whereis_name(Where, Name) -> pid() | 'undefined' when
       Where :: where(),
       Name :: name().
 whereis_name(Group, Name) ->
     request({whereis_name, Group, Name}).
 
+-doc false.
 global_groups_changed(NewPara) ->
     request({global_groups_changed, NewPara}).
 
+-doc false.
 global_groups_added(NewPara) ->
     request({global_groups_added, NewPara}).
 
+-doc false.
 global_groups_removed(NewPara) ->
     request({global_groups_removed, NewPara}).
 
+-doc """
+Synchronizes the group nodes, that is, the global name servers on the group
+nodes. Also checks the names globally registered in the current global group and
+unregisters them on any known node not part of the group.
+
+If synchronization is not possible, an error report is sent to the error logger
+(see also `m:error_logger`.
+
+Returns `{error, {'invalid global_groups definition', Bad}}` if configuration
+parameter `global_groups` has an invalid value `Bad`.
+""".
 -spec sync() -> 'ok'.
 sync() ->
     request(sync).
@@ -200,23 +325,58 @@ sync() ->
                    | {'other_groups', Groups :: [group_tuple()]}
                    | {'monitoring', Pids :: [pid()]}.
 
+-doc """
+Returns a list containing information about the global groups. Each list element
+is a tuple. The order of the tuples is undefined.
+
+- **`{state, State}`** - If the local node is part of a global group, `State` is
+  equal to `synced`. If no global groups are defined, `State` is equal to
+  `no_conf`.
+
+- **`{own_group_name, GroupName}`** - The name (atom) of the group that the
+  local node belongs to.
+
+- **`{own_group_nodes, Nodes}`** - A list of node names (atoms), the group
+  nodes.
+
+- **`{synced_nodes, Nodes}`** - A list of node names, the group nodes currently
+  synchronized with the local node.
+
+- **`{sync_error, Nodes}`** - A list of node names, the group nodes with which
+  the local node has failed to synchronize.
+
+- **`{no_contact, Nodes}`** - A list of node names, the group nodes to which
+  there are currently no connections.
+
+- **`{other_groups, Groups}`** - `Groups` is a list of tuples
+  `{GroupName, Nodes}`, specifying the name and nodes of the other global
+  groups.
+
+- **`{monitoring, Pids}`** - A list of pids, specifying the processes that have
+  subscribed to `nodeup` and `nodedown` messages.
+""".
 -spec info() -> [info_item()].
 info() ->
     request(info, 3000).
 
 %% global_group internal...
 
+-doc false.
 ng_add_check(Node, OthersNG) ->
     ng_add_check(Node, normal, OthersNG).
 
+-doc false.
 ng_add_check(Node, PubType, OthersNG) ->
     request({ng_add_check, Node, PubType, OthersNG}).
 
 %% ==== ONLY for test suites ====
+-doc false.
 registered_names_test(Arg) ->
     request({registered_names_test, Arg}).
+-doc false.
 send_test(Name, Msg) ->
     request({send_test, Name, Msg}).
+-doc false.
 whereis_name_test(Name) ->
     request({whereis_name_test, Name}).
 %% ==== ONLY for test suites ====
@@ -255,11 +415,16 @@ request(Req, Time) ->
 %%% are used to store information needed if the search process crashes. 
 %%% The search process is a help process to find registered names in the system.
 %%%====================================================================================
+-doc false.
 start() -> gen_server:start({local, global_group}, global_group, [], []).
+-doc false.
 start_link() -> gen_server:start_link({local, global_group}, global_group,[],[]).
+-doc false.
 stop() -> gen_server:call(global_group, stop, infinity).
 
+-doc false.
 init([]) ->
+    _ = process_flag(async_dist, true),
     process_flag(priority, max),
     ok = net_kernel:monitor_nodes(true, #{connection_id => true}),
     put(registered_names, [undefined]),
@@ -329,6 +494,7 @@ initial_group_setup(#gconf{node_name = NodeName,
 %%% a release upgrade. It can also be ordered if something has made the nodes
 %%% to disagree of the global_groups definition.
 %%%====================================================================================
+-doc false.
 handle_call(sync, _From, #state{nodes = OldNodes,
                                 connections = Conns} = S) ->
 %    io:format("~p sync ~p~n",[node(), application:get_env(kernel, global_groups)]),
@@ -767,6 +933,7 @@ handle_call(Call, _From, S) ->
 %%%
 %%% Get a list of nodes in the own global group
 %%%====================================================================================
+-doc false.
 handle_cast({registered_names, User}, S) ->
 %    io:format(">>>>> registered_names User ~p~n",[User]),
     Res = global:registered_names(),
@@ -907,6 +1074,7 @@ handle_cast(_Cast, S) ->
 %    io:format("***** handle_cast ~p~n",[_Cast]),
     {noreply, S}.
 
+-doc false.
 handle_info(Msg, #state{erpc_requests = Requests} = S) ->
     try erpc:check_response(Msg, Requests, true) of
         NoMatch when NoMatch == no_request; NoMatch == no_response ->
@@ -1108,10 +1276,12 @@ continue_handle_info(_Info, S) ->
 
 
 
+-doc false.
 terminate(_Reason, _S) ->
     ok.
     
 
+-doc false.
 code_change(_OldVsn, State, _Extra) ->
     {ok, State}.
 
@@ -1374,9 +1544,7 @@ make_group_conf(NodeName, KernParamValue) ->
             GMap = if OwnNodes == [] ->
                            all;
                       true ->
-                           maps:from_list(lists:map(fun (Node) ->
-                                                            {Node, ok}
-                                                    end, OwnNodes))
+                           #{Node => ok || Node <- OwnNodes}
                    end,
             #gconf{parameter_value = KernParamValue,
                    node_name = NodeName,
@@ -1591,6 +1759,7 @@ force_nodedown(DisconnectNodes, Conns) ->
 %%%====================================================================================
 %%% Get the current global_groups definition
 %%%====================================================================================
+-doc false.
 get_own_nodes_with_errors() ->
     case lookup_group_conf(false) of
         #gconf{state = {error, Error, _NodeGrps}} ->
@@ -1601,6 +1770,7 @@ get_own_nodes_with_errors() ->
             {ok, Nodes}
     end.
 
+-doc false.
 get_own_nodes() ->
     get_own_nodes(false).
 
@@ -1613,6 +1783,7 @@ get_own_nodes(#gconf{group_list = Nodes}) ->
 %%% Is a group configured?
 %%%====================================================================================
 
+-doc false.
 -spec group_configured() -> boolean().
 
 group_configured() ->
@@ -1633,6 +1804,7 @@ group_configured(GConf) ->
 %%% or there are no group configured (in which case all nodes are participants).
 %%%====================================================================================
 
+-doc false.
 -spec participant(Node::node()) -> boolean().
 
 participant(Node) ->
@@ -1649,6 +1821,7 @@ participant(Node) ->
 %%% Is node member of our configured group?
 %%%====================================================================================
 
+-doc false.
 -spec member(Node::node()) -> boolean().
 
 member(Node) ->
@@ -1666,6 +1839,7 @@ member(GG, Node) ->
 %%% Publish on node?
 %%%====================================================================================
 
+-doc false.
 -spec publish(OwnPublishType, Node) -> boolean() when
       OwnPublishType :: 'hidden' | 'normal',
       Node :: node().
