@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2018-2020. All Rights Reserved.
+%% Copyright Ericsson AB 2018-2023. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -343,7 +343,7 @@ cascade({PNode,PMFA,_PStatProcs},{CNode,CMFA,_CStatProcs},TestFun) ->
     after TO ->
             All = ets:lookup_element(Tab,producer,2),
             Written = ets:lookup_element(Tab,consumer,2),
-            dbg:stop_clear(),
+            dbg:stop(),
             ?COLLECT_STATS(All,
                            [{PNode,P,Id} || {Id,P} <- _PStatProcs] ++
                                [{CNode,P,Id} || {Id,P} <- _CStatProcs]),
