@@ -59,7 +59,10 @@
     group_leave/1,
     monitor_nonempty_scope/0, monitor_nonempty_scope/1,
     monitor_scope/0, monitor_scope/1,
-    monitor/1
+    monitor/1,
+    monitor_self/1,
+    multi_monitor/1,
+    protocol_upgrade/1
 ]).
 
 -include_lib("common_test/include/ct.hrl").
@@ -87,7 +90,8 @@ all() ->
 
 groups() ->
     [
-        {basic, [parallel], [errors, pg, leave_exit_race, single, overlay_missing]},
+        {basic, [parallel], [errors, pg, leave_exit_race, single, overlay_missing,
+                             protocol_upgrade, monitor_self, multi_monitor]},
         {performance, [], [thundering_herd]},
         {cluster, [parallel], [process_owner_check, two, initial, netsplit, trisplit, foursplit,
             exchange, nolocal, double, scope_restart, missing_scope_join, empty_group_by_remote_leave,
@@ -755,9 +759,75 @@ second_monitor(Msgs) ->
             second_monitor([Msg | Msgs])
     end.
 
+%% Test for GH-7625: monitor process that joined a group
+monitor_self(Config) when is_list(Config) ->
+    F = fun() ->
+        %% spawned process both monitor and group-joined
+        pg:monitor(?FUNCTION_NAME, ?FUNCTION_NAME),
+        pg:join(?FUNCTION_NAME, ?FUNCTION_NAME, self())
+        end,
+    {Pid, Mon} = spawn_monitor(F),
+    receive
+        {'DOWN', Mon, process, Pid, Reason} ->
+            ?assertEqual(normal, Reason)
+    end,
+    %% if pg crashes, next expression fails the test
+    sync(?FUNCTION_NAME).
+
+%% check same process monitoring several things at once,
+%% and also joining a few groups
+multi_monitor(Config) when is_list(Config) ->
+    F = fun() ->
+        Self = self(),
+        %% spawned process both monitor and group-joined
+        {RefOne, []} = pg:monitor(?FUNCTION_NAME, one),
+        {RefTwo, []} = pg:monitor(?FUNCTION_NAME, two),
+        {RefScope, _} = pg:monitor_scope(?FUNCTION_NAME),
+        ok = pg:join(?FUNCTION_NAME, one, Self),
+        ok = pg:join(?FUNCTION_NAME, two, Self),
+        sync(?FUNCTION_NAME),
+        %% ensure receiving 4 messages: two per group this process
+        [wait_message(Ref, join, Group, [Self], "Local") || {Ref, Group} <-
+            [{RefOne, one}, {RefScope, one}, {RefTwo, two}, {RefScope, two}]]
+        end,
+    {Pid, Mon} = spawn_monitor(F),
+    receive
+        {'DOWN', Mon, process, Pid, Reason} ->
+            ?assertEqual(normal, Reason)
+    end,
+    %% if pg crashes, next expression fails the test
+    sync(?FUNCTION_NAME),
+    %% white box: pg should not have any group or scope monitors
+    {state, _, _, _, SM, GM, _} = sys:get_state(?FUNCTION_NAME),
+    ?assertEqual(#{}, SM),
+    ?assertEqual(#{}, GM).
+
+protocol_upgrade(Config) when is_list(Config) ->
+    Scope = ?FUNCTION_NAME,
+    Group = ?FUNCTION_NAME,
+    {Peer, Node} = spawn_node(Scope, Config),
+    PgPid = rpc:call(Node, erlang, whereis, [Scope]),
+
+    RemotePid = erlang:spawn(Node, forever()),
+    ok = rpc:call(Node, pg, join, [Scope, Group, RemotePid]),
+
+    %% OTP 26:
+    %% Just do a white-box test and verify that pg accepts
+    %% a "future" discover message and replies with a sync.
+    PgPid ! {discover, self(), "Protocol version (ignore me)"},
+    {'$gen_cast', {sync, PgPid, [{Group, [RemotePid]}]}} = receive_any(),
+
+    %% stop the peer
+    peer:stop(Peer),
+    ok.
+
+
 %%--------------------------------------------------------------------
 %% Test Helpers - start/stop additional Erlang nodes
 
+receive_any() ->
+    receive M -> M end.
+
 %% flushes GS (GenServer) queue, ensuring that all prior
 %%  messages have been processed
 sync(GS) ->
