@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2018. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -48,7 +48,7 @@
 %%   with the same interface as mnesia_backup and configure
 %%   Mnesia so the alternate module performs the actual accesses
 %%   to the backup media. This means that the user may put the
-%%   backup on medias that Mnesia does not know about possibly on
+%%   backup on media that Mnesia does not know about possibly on
 %%   hosts where Erlang is not running.
 %%
 %% All these logs have to some extent a common structure.
@@ -93,7 +93,7 @@
 %% The log file structure for the mnesia_down log is as follows.
 %%
 %%    After the mnesia log section follows a mnesia_down section
-%%    containg lists with yoyo records as single element.
+%%    containing lists with yoyo records as single element.
 %%
 %%    +-----------------+
 %%    | mnesia log head |
@@ -124,6 +124,7 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 -module(mnesia_log).
+-moduledoc false.
 
 -export([
 	 append/2,
