@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1996-2016. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2024. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -22,6 +22,7 @@
 %% Supervisor for the entire Mnesia application
 
 -module(mnesia_sup).
+-moduledoc false.
 
 -behaviour(supervisor).
 
@@ -88,13 +89,25 @@ add_event_handler() ->
 
 kill() ->
     Mnesia = [mnesia_fallback | mnesia:ms()],
-    Kill = fun(Name) -> catch exit(whereis(Name), kill) end,
+    Kill = fun
+               (mnesia_sup) ->
+                   try  %% Avoid crash reports
+                       Sup = whereis(mnesia_sup),
+                       {_,Dict} = process_info(Sup, dictionary),
+                       [App] = proplists:get_value('$ancestors', Dict),
+                       unlink(App),
+                       exit(Sup, kill),
+                       App ! {'EXIT', Sup, normal}
+                   catch _:_ -> ok end;
+               (Name) ->
+                   try exit(whereis(Name), kill) catch _:_ -> ok end
+           end,
     lists:foreach(Kill, Mnesia),
     lists:foreach(fun ensure_dead/1, Mnesia),
     timer:sleep(10),
     case lists:keymember(mnesia, 1, application:which_applications()) of
-	true -> kill();
-	false -> ok
+        true -> kill();
+        false -> ok
     end.
 
 ensure_dead(Name) ->
