@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1997-2022. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2023. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -27,7 +27,6 @@
          all/0, groups/0]).
 
 -export([consistency_after_change_table_copy_type/1,
-         consistency_after_rename_of_node/1,
          consistency_after_restart_1_ram/1,
          consistency_after_restart_1_disc/1,
          consistency_after_restart_1_disc_only/1,
@@ -110,9 +109,8 @@ all() ->
      {group, consistency_after_del_replica},
      {group, consistency_after_move_replica},
      {group, consistency_after_transform_table},
-     consistency_after_change_table_copy_type,
+     %% consistency_after_change_table_copy_type,
      {group, consistency_after_restore},
-     consistency_after_rename_of_node,
      {group, checkpoint_retainer_consistency},
      {group, backup_consistency}].
 
@@ -381,7 +379,7 @@ consistency_after_restart(ReplicaType, NodeConfig, Config) ->
     TpcbConfig = tpcb_config(ReplicaType, NodeConfig, Nodes, [Node1]),
     mnesia_tpcb:init(TpcbConfig),
     A ! fun () -> mnesia_tpcb:run(TpcbConfig) end,
-    timer:sleep(timer:seconds(10)),
+    timer:sleep(timer:seconds(3)),
     mnesia_test_lib:kill_mnesia([Node1]),
     %% Start and wait for tables to be loaded on all nodes
     timer:sleep(timer:seconds(3)),
@@ -408,7 +406,7 @@ consistency_after_dump_tables(ReplicaType, NodeConfig, Config) ->
     TpcbConfig = tpcb_config(ReplicaType, NodeConfig, Nodes, []),
     mnesia_tpcb:init(TpcbConfig),
     A ! fun() -> mnesia_tpcb:run(TpcbConfig) end,
-    timer:sleep(timer:seconds(10)),
+    timer:sleep(timer:seconds(3)),
     ?match({atomic, ok}, rpc:call(Node1, mnesia, dump_tables,
                         [[branch, teller, account, history]])),
     mnesia_tpcb:stop(),
@@ -459,7 +457,7 @@ consistency_after_add_replica(ReplicaType, NodeConfig, Config) ->
     TpcbConfig = tpcb_config(ReplicaType, NodeConfig, Nodes, []),
     mnesia_tpcb:init(TpcbConfig),
     A ! fun () -> mnesia_tpcb:run(TpcbConfig) end,
-    timer:sleep(timer:seconds(10)),
+    timer:sleep(timer:seconds(2)),
     ?match({atomic, ok}, mnesia:add_table_copy(account, AddNode, ReplicaType)),
     mnesia_tpcb:stop(),
     ?match(ok, mnesia_tpcb:verify_tabs()),
@@ -501,7 +499,7 @@ consistency_after_del_replica(ReplicaType, NodeConfig, Config) ->
     TpcbConfig = tpcb_config(ReplicaType, NodeConfig, Nodes, []),
     mnesia_tpcb:init(TpcbConfig),
     A ! fun () -> mnesia_tpcb:run(TpcbConfig) end,
-    timer:sleep(timer:seconds(10)),
+    timer:sleep(timer:seconds(3)),
     ?match({atomic, ok}, mnesia:del_table_copy(account, Node2)),
     mnesia_tpcb:stop(),
     ?match(ok, mnesia_tpcb:verify_tabs()),
@@ -543,7 +541,7 @@ consistency_after_move_replica(ReplicaType, NodeConfig, Config) ->
     TpcbConfig = tpcb_config(ReplicaType, NodeConfig, Nodes -- [Node2], []),
     mnesia_tpcb:init(TpcbConfig),
     A ! fun () -> mnesia_tpcb:run(TpcbConfig) end,
-    timer:sleep(timer:seconds(10)),
+    timer:sleep(timer:seconds(3)),
     ?match({atomic, ok}, mnesia:move_table_copy(account, Node1, Node2)),    
     ?log("First move completed from node ~p to ~p ~n", [Node1, Node2]),
     ?match({atomic, ok}, mnesia:move_table_copy(account, Node2, Node1)),
@@ -585,8 +583,8 @@ consistency_after_transform_table(Type, Config) ->
 	[k,a,n]) || Tab <- Tabs]),
     [?match([k,a,n], mnesia:table_info(Tab, attributes)) || Tab <- Tabs],
 
-    Filter = fun(Tab) -> mnesia:foldl(fun(A, Acc) when size(A) == 3 -> [A|Acc];
-					 (A, Acc) when size(A) == 4 -> Acc
+    Filter = fun(Tab) -> mnesia:foldl(fun(A, Acc) when tuple_size(A) == 3 -> [A|Acc];
+					 (A, Acc) when tuple_size(A) == 4 -> Acc
 				      end, [], Tab)
 	     end,    
 	
@@ -638,7 +636,7 @@ consistency_after_fallback_3_disc_only(Config) when is_list(Config) ->
 consistency_after_fallback(ReplicaType, NodeConfig, Config) ->
     put(mnesia_test_verbose, true),
     %%?verbose("Starting consistency_after_fallback2 at ~p~n", [self()]),
-    Delay = 5,
+    Delay = 3,
     Nodes = ?acquire_nodes(NodeConfig, [{tc_timeout, timer:minutes(10)} | Config]),
     Node1 = hd(Nodes),
     %%?verbose("Mnesia info: ~p~n", [mnesia:info()]),
@@ -821,11 +819,6 @@ restore_verify_tabs([Tab | R]) ->
 restore_verify_tabs([]) ->
     ok.
 
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-consistency_after_rename_of_node(doc) ->
-    ["Skipped because it is an unimportant case."].
-
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
@@ -867,7 +860,7 @@ updates_during_checkpoint_activation_3_disc_only(Config) when is_list(Config) ->
 
 updates_during_checkpoint_activation(ReplicaType,NodeConfig,Config) ->
     %%?verbose("updates_during_checkpoint_activation2 at ~p~n", [self()]),
-    Delay = 5,
+    Delay = 2,
     Nodes = ?acquire_nodes(NodeConfig, Config),
     Node1 = hd(Nodes),
     %%?verbose("Mnesia info: ~p~n", [mnesia:info()]),
@@ -922,7 +915,7 @@ updates_during_checkpoint_iteration_2_disc_only(Config) when is_list(Config) ->
 
 updates_during_checkpoint_iteration(ReplicaType,NodeConfig,Config) ->
    %?verbose("updates_during_checkpoint_iteration2 at ~p~n", [self()]),
-    Delay = 5,
+    Delay = 2,
     Nodes = ?acquire_nodes(NodeConfig, Config),
     Node1 = hd(Nodes),
    %?verbose("Mnesia info: ~p~n", [mnesia:info()]),
