@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2011-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2011-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@
 %% %CopyrightEnd%
 
 -module(observer_procinfo).
+-moduledoc false.
 
 -behaviour(wx_object).
 
@@ -54,12 +55,8 @@ start(Process, ParentFrame, Parent) ->
 init([Pid, ParentFrame, Parent]) ->
     try
 	Table = ets:new(observer_expand,[set,public]),
-	Title=case observer_wx:try_rpc(node(Pid), erlang, process_info, [Pid, registered_name]) of
-		  [] -> io_lib:format("~p",[Pid]);
-		  {registered_name, Registered} -> io_lib:format("~tp (~p)",[Registered, Pid]);
-		  undefined -> throw(process_undefined)
-	      end,
-    Scale = observer_wx:get_scale(),
+	Title = get_name(Pid),
+        Scale = observer_wx:get_scale(),
 	Frame=wxFrame:new(ParentFrame, ?wxID_ANY, [atom_to_list(node(Pid)), $:, Title],
 			  [{style, ?wxDEFAULT_FRAME_STYLE}, {size, {Scale * 850, Scale * 600}}]),
 	MenuBar = wxMenuBar:new(),
@@ -339,7 +336,7 @@ fetch_state_info2(Pid, M) ->
 	    %% Formatted status ?
 	    case lists:keyfind(format_status, 1, rpc:call(node(Pid), M, module_info, [exports])) of
 		false	-> Opt = {"Format", unknown};
-		_	-> Opt = {"Format", overriden}
+		_	-> Opt = {"Format", overridden}
 	    end,
 	    [{"Behaviour", B}, Opt, {"State",OtherFormat}];
 	{badrpc,{'EXIT',{timeout, _}}} -> []
@@ -449,6 +446,34 @@ filter_monitor_info() ->
 	    [Id || {_Type, Id} <- Ms] % Type is process or port
     end.
 
+%% NOTE: intentionally throws error
+get_name(Pid) ->
+    case observer_wx:try_rpc(node(Pid), erlang, process_info, [Pid, registered_name]) of
+        [] ->
+            case observer_wx:try_rpc(node(Pid), proc_lib, get_label, [Pid]) of
+                {error, _} ->
+                    io_lib:format("~w",[Pid]);
+                undefined ->
+                    io_lib:format("~w",[Pid]);
+                Label ->
+                    format_label(Label, Pid)
+            end;
+        {registered_name, Registered} ->
+            io_lib:format("~0.tp ~w",[Registered, Pid]);
+        undefined ->
+            throw(process_undefined)
+    end.
+
+format_label(Id, Pid) when is_list(Id); is_binary(Id) ->
+    case unicode:characters_to_binary(Id) of
+        {error, _, _} ->
+            io_lib:format("~0.tp ~w", [Id, Pid]);
+        BinString ->
+            io_lib:format("~ts ~w", [BinString, Pid])
+    end;
+format_label(Id, Pid) ->
+    io_lib:format("~0.tp ~w", [Id, Pid]).
+
 stringify_bins(Data) ->
     Bins = proplists:get_value(binary, Data),
     [lists:flatten(io_lib:format("<< ~s, refc ~w>>", [observer_lib:to_str({bytes,Sz}),Refc]))
