@@ -1,5 +1,5 @@
 -module(gs2).
--vsn(1).
+-vsn(2).
 -behaviour(gen_server).
 
 -export([is_operation_ok/1]).
@@ -14,7 +14,8 @@ init([Data]) ->
 
 handle_call({is_operation_ok, Op}, _From, State) ->
     Data = gs1:get_data(),
-    Reply = lists2:assoc(Op, Data),
+    Time = gs1:get_time(),
+    Reply = do_things(lists2:assoc(Op, Data), Time),
     {reply, Reply, State}.
 
 handle_cast(_Request, State) ->
@@ -29,3 +30,9 @@ terminate(_Reason, _State) ->
 code_change(_OldVsn, State, _Extra) ->
     {ok, State}.
 
+do_things({ok, Val}, Time) ->
+    Val;
+do_things(false, Time) ->
+    {false, Time}.
+
+    
