@@ -95,12 +95,8 @@ gen_server_crash(Config, Encoding, FormatterOpts, Min, Max) ->
     ok = file:write_file(ConfigFileName ++ ".config",
                          io_lib:format("[{kernel, ~p},~n{sasl, ~p}].",
                                        [KernelConfig,SaslConfig])),
-    {ok,Node} =
-        test_server:start_node(
-          TC, peer,
-          [{args, ["-pa ",filename:dirname(code:which(?MODULE)),
-                   " -boot start_sasl -kernel start_timer true "
-                   "-config ",ConfigFileName]}]),
+    {ok,Peer,Node} = ?CT_PEER(["-boot", "start_sasl", "-kernel", "start_timer", "true",
+                   "-config", ConfigFileName]),
 
     %% Set depth or chars_limit.
     ok = rpc:call(Node,logger,update_formatter_config,[default,FormatterOpts]),
@@ -126,7 +122,7 @@ gen_server_crash(Config, Encoding, FormatterOpts, Min, Max) ->
     ok = rpc:call(Node,logger_std_h,filesync,[default]),
     ok = rpc:call(Node,logger_std_h,filesync,[sasl]),
 
-    test_server:stop_node(Node),
+    peer:stop(Peer),
     ok = logger:remove_primary_filter(no_remote),
 
     check_file(KernelLog, utf8, Min, Max),
