@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1996-2022. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2024. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -18,6 +18,13 @@
 %% %CopyrightEnd%
 %%
 -module(snmpa_supervisor).
+-moduledoc """
+A supervisor for the SNMP agent Processes
+
+This is the top supervisor for the agent part of the SNMP application. There is
+always one supervisor at each node with an SNMP agent (master agent or
+sub-agent).
+""".
 
 -behaviour(supervisor).
 
@@ -29,6 +36,7 @@
 %% Internal exports
 -export([init/1, config/2]).
 
+-compile({no_auto_import,[erase/1]}).
 
 -define(SERVER, ?MODULE).
 
@@ -41,6 +49,10 @@
 %% Process structure
 %% =================
 %%
+%%                                "application"
+%%                                     |
+%%                                  app-sup
+%%                                     |
 %%             ___________________ supervisor __________________
 %%            /              |              |          \         \ 
 %%   ___misc_sup___    target_cache  symbolic_store   local_db   agent_sup
@@ -71,6 +83,7 @@
 %% in some way.
 %%-----------------------------------------------------------------
 
+-doc false.
 start_link(Type, Opts) ->
     ?d("start_link -> entry with"
       "~n   Type. ~p"
@@ -92,9 +105,11 @@ start_link(master, Opts, {takeover, Node}) ->
     end.
 
 
+-doc false.
 stop() ->
     stop(0).
 
+-doc false.
 stop(Timeout) ->
     case whereis(?SERVER) of
 	Pid when is_pid(Pid) ->
@@ -161,6 +176,23 @@ takeover_mib({_MibName, _Symbolic, FileName}) ->
 
 %% ----------------------------------------------------------------
 
+-doc """
+Starts a supervisor for the SNMP agent system without a master agent. The
+supervisor starts all involved SNMP agent processes, but no agent processes.
+Sub-agents should be started by calling
+[`start_sub_agent/3`](`start_sub_agent/3`).
+
+`db_dir` is mandatory.
+
+See [configuration parameters](snmp_config.md#configuration_params) for a
+description of the options.
+""".
+-spec start_sub_sup(Opts) -> {ok, Pid} | {error, Reason} when
+      Opts   :: [Opt],
+      Opt    :: {db_dir, snmp:dir()} | {atom(), term()},
+      Pid    :: pid(),
+      Reason :: {already_started, Pid} | term().
+
 start_sub_sup(Opts) ->
     ?d("start_sub_sup -> entry with"
       "~n   Opts: ~p", [Opts]),
@@ -171,12 +203,47 @@ do_start_sub_sup(Opts) ->
     ?d("do_start_sub_sup -> start (sub) supervisor",[]),
     supervisor:start_link({local, ?SERVER}, ?MODULE, [sub, Opts]).  
 
+-doc """
+Starts a supervisor for the SNMP agent system. The supervisor starts all
+involved SNMP processes, including the master agent. Sub-agents should be
+started by calling `start_subagent/3`.
+
+`db_dir` is mandatory.
+
+`dir` in config is mandatory.
+
+See [snmp config](snmp_config.md) for a description of the options.
+""".
+-spec start_master_sup(Opts) -> {ok, Pid} | {error, Reason} when
+      Opts     :: [Opt],
+      Opt      :: {db_dir, string()} | {config, ConfOpts} | {atom(), term()},
+      ConfOpts :: [ConfOpt],
+      ConfOpt  :: {dir, string()} | {atom(), term()},
+      Pid      :: pid(),
+      Reason   :: {already_started, Pid} | term().
+
 start_master_sup(Opts) ->
     (catch do_start_master_sup(Opts)).
 
 do_start_master_sup(Opts) ->
     verify_mandatory([db_dir], Opts),
-    supervisor:start_link({local, ?SERVER}, ?MODULE, [master, Opts]).  
+    case supervisor:start_link({local, ?SERVER}, ?MODULE, [master, Opts]) of
+        {ok, Pid} = OK ->
+            %% <HACKETI-HACK-HACK>
+            Key = master_agent_child_spec,
+            MasterAgentSpec = lookup(Key),
+            case snmpa_agent_sup:start_master_agent(MasterAgentSpec) of
+                {ok, MPid} when is_pid(MPid) ->
+                    erase(Key),
+                    OK;
+                {error, {Reason, _ChildSpec}} ->
+                    stop(Pid, 0),
+                    {error, Reason}
+            end;
+            %% </HACKETI-HACK-HACK>
+        Else ->
+            Else
+    end.
 
 verify_mandatory([], _) ->
     ok;
@@ -191,6 +258,21 @@ verify_mandatory([Key|Keys], Opts) ->
 
 %% ----------------------------------------------------------------
 
+-doc """
+Starts a sub-agent on the node where the function is called. The
+`snmpa_supervisor` must be running.
+
+If the supervisor is not running, the function fails with the reason `badarg`.
+""".
+-spec start_sub_agent(ParentAgent, Subtree, Mibs) ->
+          {ok, Pid} | {error, Reason} when
+      ParentAgent :: pid(),
+      Subtree     :: snmp:oid(),
+      Mibs        :: [MibName],
+      MibName     :: string(),
+      Pid         :: pid(),
+      Reason      :: term().
+
 start_sub_agent(ParentAgent, Subtree, Mibs) 
   when is_pid(ParentAgent) andalso is_list(Mibs) ->
     ?d("start_sub_agent -> entry with"
@@ -199,12 +281,22 @@ start_sub_agent(ParentAgent, Subtree, Mibs)
       "~n   Mibs:        ~p", [ParentAgent, Subtree, Mibs]),
     snmpa_agent_sup:start_subagent(ParentAgent, Subtree, Mibs).
 
+-doc """
+Stops the sub-agent on the node where the function is called. The
+`snmpa_supervisor` must be running.
+
+If the supervisor is not running, the function fails with the reason `badarg`.
+""".
+-spec stop_sub_agent(SubAgentPid) -> ok | no_such_child when
+      SubAgentPid :: pid().
+
 stop_sub_agent(SubAgentPid) ->
     snmpa_agent_sup:stop_subagent(SubAgentPid).
 
 
 %% ----------------------------------------------------------------
 
+-doc false.
 init([AgentType, Opts]) ->
     ?d("init -> entry with"
       "~n   AgentType: ~p"
@@ -503,9 +595,18 @@ init([AgentType, Opts]) ->
 		    worker_spec(snmpa_agent, 
 				[Prio, snmp_master_agent, none, Ref, AgentOpts],
 				Restart, 15000),
+                %% <HACKETI-HACK-HACK>
+                %% The point is to make start failure more quiet
+                %% Often the failure happens in the master agent,
+                %% so we move the start of that out of this function
+                %% and into the 'do_start_master_sup' function.
+                %% At some point we should rewrite this. Maybe start all
+                %% children the same way (explicitly).
+                store(master_agent_child_spec, AgentSpec),
 		AgentSupSpec = 
-		    sup_spec(snmpa_agent_sup, [AgentSpec], 
+		    sup_spec(snmpa_agent_sup, [], 
 			     Restart, infinity), 
+                %% </HACKETI-HACK-HACK>
 		[ConfigSpec, AgentSupSpec];
 	    _ ->
 		?vdebug("[sub agent] spec for the agent supervisor",[]),
@@ -521,6 +622,13 @@ init([AgentType, Opts]) ->
 store(Key, Value) ->
     ets:insert(snmp_agent_table, {Key, Value}).
 
+lookup(Key) ->
+    [{Key, Value}] = ets:lookup(snmp_agent_table, Key),
+    Value.
+
+erase(Key) ->
+    ets:delete(snmp_agent_table, Key).
+
 get_mibs(Mibs, Vsns) ->
     MibDir = filename:join(code:priv_dir(snmp), "mibs"),
     StdMib = 
@@ -552,6 +660,7 @@ add_mib(DefaultMib, [Mib | T], BaseNames) ->
     end.
 
 
+-doc false.
 config(Vsns, Opts) ->
     ?d("config -> entry with"
       "~n   Vsns. ~p"
