@@ -1,7 +1,7 @@
 %% 
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2004-2023. All Rights Reserved.
+%% Copyright Ericsson AB 2004-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 
 -ifndef(snmpm_net_if_mt).
 -module(snmpm_net_if).
+-moduledoc false.
 -endif.
 
 -behaviour(gen_server).
@@ -292,14 +293,26 @@ do_init(Server, NoteStore) ->
     ?vdebug("DomainAddresses: ~w", [DomainAddresses]),
     CommonSocketOpts = common_socket_opts(Opts),
     BindTo = get_opt(Opts, bind_to, false),
-    InetBackend = case get_opt(Opts, inet_backend, use_default) of
-                      use_default -> [];
-                      IB          -> [{inet_backend, IB}]
-                  end,
+    {RequireBind, InetBackend} =
+	case get_opt(Opts, inet_backend, use_default) of
+	    use_default ->
+		{false, []};
+	    IB when (IB =:= inet) ->
+		{false, [{inet_backend, IB}]};
+	    IB when (IB =:= socket) ->
+		{case os:type() of
+		     {win32, nt} ->
+			 true;
+		     _ ->
+			 false
+		 end,
+		 [{inet_backend, IB}]}
+	end,
     case
 	[begin
 	     {IpPort, SocketOpts} =
-		 socket_params(Domain, Address, BindTo, CommonSocketOpts),
+		 socket_params(Domain, Address,
+			       RequireBind, BindTo, CommonSocketOpts),
              %% The 'inet-backend' option has to be first,
              %% so we might as well add it last.
 	     Socket = socket_open(IpPort, InetBackend ++ SocketOpts),
@@ -356,7 +369,8 @@ socket_open(IpPort, SocketOpts) ->
 	    Socket
     end.
 
-socket_params(Domain, {IpAddr, IpPort} = Addr, BindTo, CommonSocketOpts) ->
+socket_params(Domain, {IpAddr, IpPort} = Addr,
+	      RequireBind, BindTo, CommonSocketOpts) ->
     Family = snmp_conf:tdomain_to_family(Domain),
     SocketOpts =
 	case Family of
@@ -370,22 +384,22 @@ socket_params(Domain, {IpAddr, IpPort} = Addr, BindTo, CommonSocketOpts) ->
 	    case init:get_argument(snmpm_fd) of
 		{ok, [[FdStr]]} ->
 		    Fd = list_to_integer(FdStr),
-		    case BindTo of
+		    case RequireBind orelse BindTo of
 			true ->
 			    {IpPort, [{ip, IpAddr}, {fd, Fd} | SocketOpts]};
 			_ ->
 			    {0, [{fd, Fd} | SocketOpts]}
 		    end;
 		error ->
-		    socket_params(SocketOpts, Addr, BindTo)
+		    socket_params(SocketOpts, Addr, RequireBind, BindTo)
 	    end;
 	_ ->
-	    socket_params(SocketOpts, Addr, BindTo)
+	    socket_params(SocketOpts, Addr, RequireBind, BindTo)
     end.
 
 %%
-socket_params(SocketOpts, {IpAddr, IpPort}, BindTo) ->
-    case BindTo of
+socket_params(SocketOpts, {IpAddr, IpPort}, RequireBind, BindTo) ->
+    case RequireBind orelse BindTo of
 	true ->
 	    {IpPort, [{ip, IpAddr} | SocketOpts]};
 	_ ->
@@ -603,7 +617,8 @@ handle_cast(filter_reset, State) ->
     {noreply, State};
 
 %% This is for debugging!!
-handle_cast({exec, F}, #state{allow_exec = true} = State) when is_function(F, 0) ->
+handle_cast({exec, F}, #state{allow_exec = true} = State)
+  when is_function(F, 0) ->
     ?vlog("[cast] exec", []),
     F(),
     {noreply, State};
@@ -626,8 +641,8 @@ handle_info(
     Size = byte_size(Bytes),
     case lists:keyfind(Socket, #transport.socket, Transports) of
 	#transport{socket = Socket, domain = Domain} ->
-	    ?vlog("received ~w bytes from ~p:~p [~w]",
-		  [Size, IpAddr, IpPort, Socket]),
+	    ?vlog("received ~w bytes from [~w] ~p:~p (~w)",
+		  [Size, Domain, IpAddr, IpPort, Socket]),
 	    maybe_handle_recv_msg(Domain, {IpAddr, IpPort}, Bytes, State),
 	    {noreply, State};
 	false ->
