@@ -1,7 +1,7 @@
 %% 
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2004-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2004-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@
 %% 
 
 -module(snmp_usm).
+-moduledoc false.
 
 %% Avoid warning for local function error/1 clashing with autoimported BIF.
 -compile({no_auto_import,[error/1]}).
@@ -29,6 +30,10 @@
 -export([des_encrypt/3, des_decrypt/3]).
 -export([aes_encrypt/5, aes_decrypt/5]).
 
+-export_type([
+              algorithm/0
+             ]).
+
 
 -define(SNMP_USE_V3, true).
 -include("snmp_types.hrl").
@@ -63,10 +68,12 @@
 
 -define(BLOCK_CIPHER_DES, des_cbc).
 
+-type algorithm() :: md5 | sha | sha224 | sha256 | sha384 | sha512.
+
 
 %%-----------------------------------------------------------------
 %% Func: passwd2localized_key/3
-%% Types: Alg      = md5 | sha
+%% Types: Alg      = algorithm()
 %%        Passwd   = string()
 %%        EngineID = string()
 %% Purpose: Generates a key that can be used as an authentication
@@ -76,8 +83,6 @@
 %%          rfc2274.
 %%-----------------------------------------------------------------
 
--type algorithm() :: md5 | sha | sha224 | sha256 | sha384 | sha512.
-
 -spec passwd2localized_key(Alg      :: algorithm(),
                            Passwd   :: string(),
                            EngineID :: string()) ->
@@ -411,7 +416,7 @@ des_decrypt(PrivKey, MsgPrivParams, EncData)
     DesKey = [A,B,C,D,E,F,G,H],
     Salt = MsgPrivParams,
     IV = list_to_binary(snmp_misc:str_xor(PreIV, Salt)),
-    %% Whatabout errors here???  E.g. not a mulitple of 8!
+    %% Whatabout errors here???  E.g. not a multiple of 8!
     Data = binary_to_list(crypto:crypto_one_time(?BLOCK_CIPHER_DES, 
                                                  DesKey, IV, EncData, false)),
     Data2 = snmp_pdus:strip_encrypted_scoped_pdu_data(Data),
@@ -438,7 +443,7 @@ aes_decrypt(PrivKey, MsgPrivParams, EncData, EngineBoots, EngineTime)
     AesKey = PrivKey,
     Salt = MsgPrivParams,
     IV = list_to_binary([?i32(EngineBoots), ?i32(EngineTime) | Salt]),
-    %% Whatabout errors here???  E.g. not a mulitple of 8!
+    %% Whatabout errors here???  E.g. not a multiple of 8!
     Data = binary_to_list(crypto:crypto_one_time(?BLOCK_CIPHER_AES(AesKey),
                                                  AesKey, IV, EncData, false)),
     Data2 = snmp_pdus:strip_encrypted_scoped_pdu_data(Data),
