@@ -1,7 +1,7 @@
 %% 
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2022. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2023. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -256,14 +256,12 @@ init({Options, CallerPid}) ->
 	    IpFamily = get_value(ipfamily, Options, inet),
 	    ?IPRINT("init -> IpFamily: ~p", [IpFamily]),
 	    AgIp = case snmp_misc:assq(agent, Options) of
-		       {value, Addr} when is_tuple(Addr) andalso 
-                                          (size(Addr) =:= 4) andalso
-                                          (IpFamily =:= inet) ->
+		       {value, Addr} when  (tuple_size(Addr) =:= 4) andalso
+                                   (IpFamily =:= inet) ->
                            ?IPRINT("init -> Addr: ~p", [Addr]),
 			   Addr;
-		       {value, Addr} when is_tuple(Addr) andalso 
-                                          (size(Addr) =:= 8) andalso
-                                          (IpFamily =:= inet6) ->
+		       {value, Addr} when  (tuple_size(Addr) =:= 8) andalso
+                                   (IpFamily =:= inet6) ->
                            ?IPRINT("init -> Addr: ~p", [Addr]),
 			   Addr;
 		       {value, Host} when is_list(Host) ->
@@ -1304,7 +1302,7 @@ cast(Msg) ->
 sizeOf(L) when is_list(L) ->
     length(lists:flatten(L));
 sizeOf(B) when is_binary(B) ->
-    size(B).
+    byte_size(B).
 
 d(F)    -> d(F, []).
 d(F, A) -> d(get(debug), F, A).
