@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2005-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2005-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -149,7 +149,7 @@ check_mktemp(Config) ->
     end.
 
 extract_pubkey(PrivKey) ->
-    PubKey = ssh_transport:extract_public_key(PrivKey),
+    PubKey = ssh_file:extract_public_key(PrivKey),
     ssh_message:ssh2_pubkey_encode(PubKey).
 
 sig_format('ssh-rsa') -> <<"ssh-rsa">>;
