@@ -1,18 +1,19 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2016-2022. All Rights Reserved.
+%% Copyright Ericsson AB 2016-2024. All Rights Reserved.
 %%
-%% The contents of this file are subject to the Erlang Public License,
-%% Version 1.1, (the "License"); you may not use this file except in
-%% compliance with the License. You should have received a copy of the
-%% Erlang Public License along with this software. If not, it can be
-%% retrieved online at http://www.erlang.org/.
+%% Licensed under the Apache License, Version 2.0 (the "License");
+%% you may not use this file except in compliance with the License.
+%% You may obtain a copy of the License at
 %%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-%% the License for the specific language governing rights and limitations
-%% under the License.
+%%     http://www.apache.org/licenses/LICENSE-2.0
+%%
+%% Unless required by applicable law or agreed to in writing, software
+%% distributed under the License is distributed on an "AS IS" BASIS,
+%% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+%% See the License for the specific language governing permissions and
+%% limitations under the License.
 %%
 %% %CopyrightEnd%
 %%
@@ -20,6 +21,7 @@
 %%
 
 -module(dtls_packet_demux).
+-moduledoc false.
 
 -behaviour(gen_server).
 
@@ -27,12 +29,14 @@
 -include_lib("kernel/include/logger.hrl").
 
 %% API
--export([start_link/5,
+-export([start_link/6,
          active_once/3,
          accept/2,
          sockname/1,
          close/1,
-         new_owner/1,
+         new_owner/2,
+         new_connection/2,
+         connection_setup/2,
          get_all_opts/1,
          set_all_opts/2,
          get_sock_opts/2,
@@ -47,7 +51,7 @@
 	 terminate/2,
          code_change/3]).
 
--record(state, 
+-record(state,
 	{active_n,
          port,
 	 listener,
@@ -55,9 +59,9 @@
 	 dtls_options,
 	 emulated_options,
 	 dtls_msq_queues = kv_new(),
-	 clients = set_new(),
 	 dtls_processes = kv_new(),
 	 accepters  = queue:new(),
+         owner, %% Listen process owner
 	 first,
          close,
          session_id_tracker
@@ -67,8 +71,8 @@
 %%% API
 %%%===================================================================
 
-start_link(Port, TransportInfo, EmOpts, InetOptions, DTLSOptions) ->
-    gen_server:start_link(?MODULE, [Port, TransportInfo, EmOpts, InetOptions, DTLSOptions], []).
+start_link(Owner, Port, TransportInfo, EmOpts, InetOptions, DTLSOptions) ->
+    gen_server:start_link(?MODULE, [Owner, Port, TransportInfo, EmOpts, InetOptions, DTLSOptions], []).
 
 active_once(PacketSocket, Client, Pid) ->
     gen_server:cast(PacketSocket, {active_once, Client, Pid}).
@@ -82,8 +86,14 @@ sockname(PacketSocket) ->
 close(PacketSocket) ->
     call(PacketSocket, close).
 
-new_owner(PacketSocket) ->
-    call(PacketSocket, new_owner).
+new_owner(PacketSocket, Owner) ->
+    call(PacketSocket, {new_owner, Owner}).
+
+new_connection(PacketSocket, Client) ->
+    call(PacketSocket, {new_connection, Client, self()}).
+
+connection_setup(PacketSocket, Client) ->
+    gen_server:cast(PacketSocket, {connection_setup, Client}).
 
 get_sock_opts(PacketSocket, SplitSockOpts) ->
     call(PacketSocket,  {get_sock_opts, SplitSockOpts}).
@@ -102,9 +112,11 @@ getstat(PacketSocket, Opts) ->
 %%% gen_server callbacks
 %%%===================================================================
 
-init([Port0, TransportInfo, EmOpts, DTLSOptions, Socket]) ->
+init([Owner, Port0, TransportInfo, EmOpts, DTLSOptions, Socket]) ->
     InternalActiveN = get_internal_active_n(),
+    erlang:monitor(process, Owner),
     {ok, SessionIdHandle} = session_id_tracker(Socket, DTLSOptions),
+    proc_lib:set_label({dtls_server_packet_demultiplexer, Port0}),
     {ok, #state{active_n = InternalActiveN,
                 port = Port0,
                 first = true,
@@ -112,6 +124,7 @@ init([Port0, TransportInfo, EmOpts, DTLSOptions, Socket]) ->
                 dtls_options = DTLSOptions,
                 emulated_options = EmOpts,
                 listener = Socket,
+                owner = Owner,
                 close = false,
                 session_id_tracker = SessionIdHandle}}.
 
@@ -133,21 +146,34 @@ handle_call({accept, Accepter}, From, #state{accepters = Accepters} = State0) ->
 handle_call(sockname, _, #state{listener = Socket} = State) ->
     Reply = inet:sockname(Socket),
     {reply, Reply, State};
-handle_call(close, _, #state{dtls_processes = Processes,
-                             accepters = Accepters} = State) ->
-    case kv_empty(Processes) of
-        true ->
-            {stop, normal, ok, State#state{close=true}};
-        false -> 
-            lists:foreach(fun({_, From}) ->
-                                  gen_server:reply(From, {error, closed})
-                          end, queue:to_list(Accepters)),
-            {reply, ok,  State#state{close = true, accepters = queue:new()}}
+handle_call(close, _, State0) ->
+    case do_close(State0) of
+        {stop, State} ->
+            {stop, normal, stop, State};
+        {wait, State} ->
+            {reply, waiting, State}
     end;
-handle_call(new_owner, _, State) ->
-    {reply, ok,  State#state{close = false, first = true}};
+handle_call({new_owner, Owner}, _, State) ->
+    {reply, ok,  State#state{close = false, first = true, owner = Owner}};
+handle_call({new_connection, Old, _Pid}, _,
+            #state{accepters = Accepters, dtls_msq_queues = MsgQs0} = State) ->
+    case queue:is_empty(Accepters) of
+        false ->
+            case kv_lookup(Old, MsgQs0) of
+                {value, OldQueue} ->
+                    MsgQs1 = kv_delete(Old, MsgQs0),
+                    MsgQs = kv_insert({old,Old}, OldQueue, MsgQs1),
+                    {reply, true, State#state{dtls_msq_queues = MsgQs}};
+                none ->
+                    %% Already set as old
+                    {reply, true, State}
+            end;
+        true ->
+            {reply, false, State}
+    end;
+
 handle_call({get_sock_opts, {SocketOptNames, EmOptNames}}, _, #state{listener = Socket,
-                                                               emulated_options = EmOpts} = State) ->
+                                                                     emulated_options = EmOpts} = State) ->
     case get_socket_opts(Socket, SocketOptNames) of
         {ok, Opts} ->
             {reply, {ok, emulated_opts_list(EmOpts, EmOptNames, []) ++ Opts}, State};
@@ -170,7 +196,16 @@ handle_call({getstat, Options}, _,  #state{listener = Socket, transport =  {Tran
 
 handle_cast({active_once, Client, Pid}, State0) ->
     State = handle_active_once(Client, Pid, State0),
-    {noreply, State}.
+    {noreply, State};
+handle_cast({connection_setup, Client}, #state{dtls_msq_queues = MsgQueues} = State) ->
+    case kv_lookup({old, Client}, MsgQueues) of
+        none ->
+            {noreply, State};
+        {value, {Pid, _}} ->
+            Pid ! {socket_reused, Client},
+            %% Will be deleted when handling DOWN message
+            {noreply, State}
+    end.
 
 handle_info({Transport, Socket, IP, InPortNo, _} = Msg, #state{listener = Socket, transport = {_,Transport,_,_,_}} = State0) ->
     State = handle_datagram({IP, InPortNo}, Msg, State0),
@@ -190,24 +225,47 @@ handle_info({udp_error, Socket, econnreset = Error}, #state{listener = Socket, t
     ?LOG_NOTICE(Report),
     {noreply, State};
 handle_info({ErrorTag, Socket, Error}, #state{listener = Socket, transport = {_,_,_, ErrorTag,_}} = State) ->
-    Report = io_lib:format("SSL Packet muliplxer shutdown: Socket error: ~p ~n", [Error]),
+    Report = io_lib:format("SSL Packet muliplexer shutdown: Socket error: ~p ~n", [Error]),
     ?LOG_NOTICE(Report),
     {noreply, State#state{close=true}};
 
-handle_info({'DOWN', _, process, Pid, _}, #state{clients = Clients,
-						 dtls_processes = Processes0,
-                                                 dtls_msq_queues = MsgQueues0,
-                                                 close = ListenClosed} = State) ->
+handle_info({'DOWN', _, process, Owner, _}, #state{owner = Owner} = State0) ->
+    case do_close(State0) of
+        {stop, State} ->
+            {stop, normal, State};
+        {wait, State} ->
+            {noreply, State}
+    end;
+handle_info({'DOWN', _, process, Pid, _},
+            #state{dtls_processes = Processes0,
+                   dtls_msq_queues = MsgQueues0,
+                   close = ListenClosed} = State0) ->
     Client = kv_get(Pid, Processes0),
     Processes = kv_delete(Pid, Processes0),
-    MsgQueues = kv_delete(Client, MsgQueues0),
+    State = case kv_lookup(Client, MsgQueues0) of
+                none ->
+                    MsgQueues1 = kv_delete({old, Client}, MsgQueues0),
+                    State0#state{dtls_processes = Processes, dtls_msq_queues = MsgQueues1};
+                {value, {Pid, _}} ->
+                    MsgQueues1 = kv_delete(Client, MsgQueues0),
+                    %% Restore old process if exists
+                    case kv_lookup({old, Client}, MsgQueues1) of
+                        none ->
+                            State0#state{dtls_processes = Processes, dtls_msq_queues = MsgQueues1};
+                        {value, Old} ->
+                            MsgQueues2 = kv_delete({old, Client}, MsgQueues1),
+                            MsgQueues = kv_insert(Client, Old, MsgQueues2),
+                            State0#state{dtls_processes = Processes, dtls_msq_queues = MsgQueues}
+                    end;
+                {value, _} -> %% Old process died (just delete its queue)
+                    MsgQueues1 = kv_delete({old, Client}, MsgQueues0),
+                    State0#state{dtls_processes = Processes, dtls_msq_queues = MsgQueues1}
+            end,
     case ListenClosed andalso kv_empty(Processes) of
         true ->
             {stop, normal, State};
         false ->
-            {noreply, State#state{clients = set_delete(Client, Clients),
-                                  dtls_processes = Processes,
-                                  dtls_msq_queues = MsgQueues}}
+            {noreply, State}
     end.
 
 terminate(_Reason, _State) ->
@@ -219,55 +277,68 @@ code_change(_OldVsn, State, _Extra) ->
 %%%===================================================================
 %%% Internal functions
 %%%===================================================================
-handle_datagram(Client, Msg, #state{clients = Clients,
-				    accepters = AcceptorsQueue0} = State) ->
-    case set_is_member(Client, Clients) of
-	false ->
+
+do_close(#state{dtls_processes = Processes, accepters = Accepters} = State) ->
+    case kv_empty(Processes) of
+        true ->
+            {stop, State#state{close=true}};
+        false ->
+            lists:foreach(fun({_, From}) -> gen_server:reply(From, {error, closed}) end,
+                          queue:to_list(Accepters)),
+            {wait, State#state{close = true, accepters = queue:new()}}
+    end.
+
+handle_datagram(Client, Msg, #state{dtls_msq_queues = MsgQueues, accepters = AcceptorsQueue0} = State) ->
+    case kv_lookup(Client, MsgQueues) of
+	none ->
 	    case queue:out(AcceptorsQueue0) of
-		{{value, {UserPid, From}}, AcceptorsQueue} ->	
-		    setup_new_connection(UserPid, From, Client, Msg, 
+		{{value, {UserPid, From}}, AcceptorsQueue} ->
+		    setup_new_connection(UserPid, From, Client, Msg,
 					 State#state{accepters = AcceptorsQueue});
 		{empty, _} ->
 		    %% Drop packet client will resend
 		    State
 	    end;
-	true -> 
-	    dispatch(Client, Msg, State)
+	{value, Queue} ->
+	    dispatch(Queue, Client, Msg, State)
     end.
 
-dispatch(Client, Msg, #state{dtls_msq_queues = MsgQueues} = State) ->
-    case kv_lookup(Client, MsgQueues) of
-	{value, Queue0} ->
-	    case queue:out(Queue0) of
-		{{value, Pid}, Queue} when is_pid(Pid) ->
-		    Pid ! Msg,
-		    State#state{dtls_msq_queues = 
-				    kv_update(Client, Queue, MsgQueues)};
-		{{value, _UDP}, _Queue} ->
-		    State#state{dtls_msq_queues = 
-				    kv_update(Client, queue:in(Msg, Queue0), MsgQueues)};
-		{empty, Queue} ->
-		    State#state{dtls_msq_queues = 
-				    kv_update(Client, queue:in(Msg, Queue), MsgQueues)}
-	    end
+dispatch({Pid, Queue0}, Client, Msg, #state{dtls_msq_queues = MsgQueues} = State) ->
+    case queue:out(Queue0) of
+        {{value, Pid}, Queue} when is_pid(Pid) ->
+            Pid ! Msg,
+            State#state{dtls_msq_queues =
+                            kv_update(Client, {Pid, Queue}, MsgQueues)};
+        {{value, _UDP}, _Queue} ->
+            State#state{dtls_msq_queues =
+                            kv_update(Client, {Pid, queue:in(Msg, Queue0)}, MsgQueues)};
+        {empty, Queue} ->
+            State#state{dtls_msq_queues =
+                            kv_update(Client, {Pid, queue:in(Msg, Queue)}, MsgQueues)}
     end.
+
 next_datagram(Socket, N) ->
     inet:setopts(Socket, [{active, N}]).
 
 handle_active_once(Client, Pid, #state{dtls_msq_queues = MsgQueues} = State0) ->
-    Queue0 = kv_get(Client, MsgQueues),
+    {Key, Queue0} = case kv_lookup(Client, MsgQueues) of
+                        {value, {Pid, Q0}} -> {Client, Q0};
+                        _ ->
+                            OldKey = {old, Client},
+                            {Pid, Q0} = kv_get(OldKey, MsgQueues),
+                            {OldKey, Q0}
+                    end,
     case queue:out(Queue0) of
-	{{value, Pid}, _} when is_pid(Pid) ->
-	    State0;
-	{{value, Msg}, Queue} ->	      
-	    Pid ! Msg,
-	    State0#state{dtls_msq_queues = kv_update(Client, Queue, MsgQueues)};
-	{empty, Queue0} ->
-	    State0#state{dtls_msq_queues = kv_update(Client, queue:in(Pid, Queue0), MsgQueues)}
+        {{value, Pid}, _} when is_pid(Pid) ->
+            State0;
+        {{value, Msg}, Queue} ->
+            Pid ! Msg,
+            State0#state{dtls_msq_queues = kv_update(Key, {Pid, Queue}, MsgQueues)};
+        {empty, Queue0} ->
+            State0#state{dtls_msq_queues = kv_update(Key, {Pid, queue:in(Pid, Queue0)}, MsgQueues)}
     end.
 
 setup_new_connection(User, From, Client, Msg, #state{dtls_processes = Processes,
-						     clients = Clients,
 						     dtls_msq_queues = MsgQueues,
 						     dtls_options = DTLSOpts,
 						     port = Port,
@@ -281,8 +352,7 @@ setup_new_connection(User, From, Client, Msg, #state{dtls_processes = Processes,
 	    erlang:monitor(process, Pid),
 	    gen_server:reply(From, {ok, Pid, {Client, Socket}}),
 	    Pid ! Msg,
-	    State#state{clients = set_insert(Client, Clients), 
-			dtls_msq_queues = kv_insert(Client, queue:new(), MsgQueues),
+	    State#state{dtls_msq_queues = kv_insert(Client, {Pid, queue:new()}, MsgQueues),
 			dtls_processes = kv_insert(Pid, Client, Processes)};
 	{error, Reason} ->
 	    gen_server:reply(From, {error, Reason}),
@@ -295,7 +365,7 @@ kv_lookup(Key, Store) ->
     gb_trees:lookup(Key, Store).
 kv_insert(Key, Value, Store) ->
     gb_trees:insert(Key, Value, Store).
-kv_get(Key, Store) -> 
+kv_get(Key, Store) ->
     gb_trees:get(Key, Store).
 kv_delete(Key, Store) ->
     gb_trees:delete(Key, Store).
@@ -304,15 +374,6 @@ kv_new() ->
 kv_empty(Store) ->
     gb_trees:is_empty(Store).
 
-set_new() ->
-    gb_sets:empty().
-set_insert(Item, Set) ->
-    gb_sets:insert(Item, Set).
-set_delete(Item, Set) ->
-    gb_sets:delete(Item, Set).
-set_is_member(Item, Set) ->
-    gb_sets:is_member(Item, Set).
-
 call(Server, Msg) ->
     try
         gen_server:call(Server, Msg, infinity)
