@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2007-2022. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -27,7 +27,8 @@
 %% %%----------------------------------------------------------------------
 
 -module(ssl_alert).
-
+-moduledoc false.
+-feature(maybe_expr, enable).
 -include("ssl_alert.hrl").
 -include("ssl_record.hrl").
 -include("ssl_internal.hrl").
@@ -113,9 +114,20 @@ own_alert_format_depth(#alert{reason = Reason} = Alert) ->
             {" ~s\n ~P", [Txt, Reason, ?DEPTH]}
     end.
 
-own_alert_txt(#alert{level = Level, description = Description, where = #{line := Line, file := Mod}, role = Role}) ->
-    "at " ++ Mod ++ ":" ++ integer_to_list(Line) ++ " generated " ++ string:uppercase(atom_to_list(Role)) ++ " ALERT: " ++
-        level_txt(Level) ++ description_txt(Description).
+own_alert_txt(#alert{level = Level, description = Description,
+                     where = #{line := Line, file := Mod} = Where,
+                     role = Role}) ->
+    DefaultLeft = "at " ++ Mod ++ ":" ++ integer_to_list(Line),
+    DefaultRight = " generated " ++ string:uppercase(atom_to_list(Role)) ++ " ALERT: " ++
+        level_txt(Level) ++ description_txt(Description),
+    maybe
+        debug ?= get(log_level),
+        {current_stacktrace, Stacktrace} ?= maps:get(st, Where, undefined),
+        DefaultLeft ++ io_lib:format("~n~p~n", [Stacktrace]) ++ DefaultRight
+    else
+        _ ->
+            DefaultLeft ++ DefaultRight
+    end.
 
 alert_format(Alert) ->
     Txt = alert_txt(Alert),
@@ -155,8 +167,6 @@ description_txt(?DECRYPTION_FAILED_RESERVED) ->
     "Decryption Failed Reserved";
 description_txt(?RECORD_OVERFLOW) ->
     "Record Overflow";
-description_txt(?DECOMPRESSION_FAILURE) ->
-    "Decompression Failure";
 description_txt(?HANDSHAKE_FAILURE) ->
     "Handshake Failure";
 description_txt(?NO_CERTIFICATE_RESERVED) ->
@@ -226,8 +236,6 @@ description_atom(?DECRYPTION_FAILED_RESERVED) ->
     decryption_failed_reserved;
 description_atom(?RECORD_OVERFLOW) ->
     record_overflow;
-description_atom(?DECOMPRESSION_FAILURE) ->
-    decompression_failure;
 description_atom(?HANDSHAKE_FAILURE) ->
     handshake_failure;
 description_atom(?NO_CERTIFICATE_RESERVED) ->
