@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2022. All Rights Reserved.
+%% Copyright Ericsson AB 2008-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -112,7 +112,7 @@ init_per_suite(Config) ->
 		    {skip, io_lib:format("Bad openssl version: ~p",[OpenSSL_version])};
 		_ ->
 		    end_per_suite(Config),
-		    try crypto:start() of
+		    case application:ensure_started(crypto) of
 			ok ->
 			    {ok, Hostname0} = inet:gethostname(),
 			    IPfamily =
@@ -120,8 +120,8 @@ init_per_suite(Config) ->
 				    true -> inet6;
 				    false -> inet
 				end,
-			    [{ipfamily,IPfamily}, {openssl_version,OpenSSL_version} | Config]
-		    catch _:_ ->
+			    [{ipfamily,IPfamily}, {openssl_version,OpenSSL_version} | Config];
+                        _ ->
 			    {skip, "Crypto did not start"}
 		    end
 	    end
@@ -576,18 +576,18 @@ delete_crl_with_path(Config) ->
     PemEntries = public_key:pem_decode(PemBin),
     CRLs = [CRL || {'CertificateList', CRL, not_encrypted}
                        <- PemEntries],
-
     {status, _, _, StatusInfo} = sys:get_status(whereis(ssl_manager)),
     [_, _,_, _, Prop] = StatusInfo,
     State = ssl_test_lib:state(Prop),
+    #'DistributionPoint'{distributionPoint = {fullName, Names}} = DP,
+    {_, URI} = lists:keyfind(uniformResourceIdentifier, 1, Names),
     case element(5, State) of
         [_, _, _, {CRLCache, _}] ->
-            URI = "http://localhost/otpCA/crl.pem",
-            not_available = ssl_crl_cache:lookup(DP, issuer, {{CRLCache, unused}, unused}),
+            not_available = ssl_crl_cache:lookup(URI, issuer, {{CRLCache, unused}, unused}),
             ok = ssl_crl_cache:insert(URI, {der, CRLs}),
             CRLs = ssl_crl_cache:lookup(DP, issuer, {{CRLCache, unused}, unused}),
             ok = ssl_crl_cache:delete(URI),
-            not_available = ssl_crl_cache:lookup(DP, issuer, {{CRLCache, unused}, unused}),
+            not_available = ssl_crl_cache:lookup(URI, issuer, {{CRLCache, unused}, unused}),
             ok
     end.
 
