@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2017-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2017-2023. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@
 
 -behaviour(ct_suite).
 
+-include("ssl_test_lib.hrl").
 -include_lib("common_test/include/ct.hrl").
 -include_lib("public_key/include/public_key.hrl").
 
@@ -70,7 +71,7 @@ init_per_suite(Config) ->
                                 {ok, Engine} ->
                                     [{engine, Engine} |Config];
                                 {error, Reason} ->
-                                    ct:pal("Reason ~p", [Reason]),
+                                    ?CT_PAL("Reason ~p", [Reason]),
                                     {skip, "No dynamic engine support"}
                             catch error:notsup ->
                                     {skip, "No engine support in OpenSSL"}    
@@ -159,10 +160,10 @@ private_key(Config) when is_list(Config) ->
     test_tls_connection([{ciphers, RSASuites}, {versions, ['tlsv1.2']} | EngineServerConf], 
                         [{ciphers, RSASuites}, {versions, ['tlsv1.2']} | EngineClientConf], Config),
     
-    %% Test with engine and present file arugments
+    %% Test with engine and present file arguments
     test_tls_connection(EngineFileServerConf, EngineFileClientConf, Config),
     
-    %% Test that sofware fallback is available
+    %% Test that software fallback is available
     test_tls_connection(ServerConf, [{reuse_sessions, false} |ClientConf], Config).
     
 engine_key(Conf) ->
