@@ -1,19 +1,20 @@
 %%
 %% %CopyrightBegin%
-%% 
-%% Copyright Ericsson AB 2015-2022. All Rights Reserved.
-%% 
-%% The contents of this file are subject to the Erlang Public License,
-%% Version 1.1, (the "License"); you may not use this file except in
-%% compliance with the License. You should have received a copy of the
-%% Erlang Public License along with this software. If not, it can be
-%% retrieved online at http://www.erlang.org/.
-%% 
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-%% the License for the specific language governing rights and limitations
-%% under the License.
-%% 
+%%
+%% Copyright Ericsson AB 2015-2023. All Rights Reserved.
+%%
+%% Licensed under the Apache License, Version 2.0 (the "License");
+%% you may not use this file except in compliance with the License.
+%% You may obtain a copy of the License at
+%%
+%%     http://www.apache.org/licenses/LICENSE-2.0
+%%
+%% Unless required by applicable law or agreed to in writing, software
+%% distributed under the License is distributed on an "AS IS" BASIS,
+%% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+%% See the License for the specific language governing permissions and
+%% limitations under the License.
+%%
 %% %CopyrightEnd%
 %%
 %%
@@ -41,7 +42,8 @@
          tls_extraneous_chain/1,
          tls_extraneous_chains/1,
          tls_extraneous_and_unorder_chains/1,
-         tls_client_cert_auth/1
+         tls_client_cert_auth/1,
+         tls_eccs/1
          ]).
 
 %%--------------------------------------------------------------------
@@ -60,7 +62,8 @@ all() ->
      tls_extraneous_chain,
      tls_extraneous_chains,
      tls_extraneous_and_unorder_chains,
-     tls_client_cert_auth
+     tls_client_cert_auth,
+     tls_eccs
     ].
 
 %%--------------------------------------------------------------------
@@ -148,3 +151,8 @@ tls_client_cert_auth(Config) when is_list(Config) ->
     ssl:start(),
     true = ct_property_test:quickcheck(ssl_eqc_chain:prop_client_cert_auth(),
                                        Config).
+tls_eccs(Config) when is_list(Config) ->
+    %% manual test:  proper:quickcheck(ssl_eqc_ecc:prop_tls_ecc)
+    ssl:start(),
+    true = ct_property_test:quickcheck(ssl_eqc_ecc:prop_tls_ecc(),
+                                       Config).
