@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2007-2023. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@
 
 -behaviour(ct_suite).
 
+-include("ssl_test_lib.hrl").
 -include("tls_handshake.hrl").
 -include("ssl_record.hrl").
 
@@ -185,8 +186,14 @@ reuse_session_expired() ->
 reuse_session_expired(Config) when is_list(Config) -> 
     ClientOpts = ssl_test_lib:ssl_options(client_rsa_verify_opts, Config),
     ServerOpts = ssl_test_lib:ssl_options(server_rsa_verify_opts, Config),
+    TestVersion = ssl_test_lib:protocol_version(Config),
     {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
-    
+    Ciphers = ssl:filter_cipher_suites(ssl:cipher_suites(all, TestVersion),
+                                       [{key_exchange, fun(srp_rsa) -> false;
+                                                          (srp_dss) -> false;
+                                                          (_) -> true
+                                                       end}]),
+
     Server0 =
 	ssl_test_lib:start_server([{node, ServerNode}, {port, 0},
 				   {from, self()},
@@ -198,13 +205,14 @@ reuse_session_expired(Config) when is_list(Config) ->
     Client0 = ssl_test_lib:start_client([{node, ClientNode},
                                          {port, Port0}, {host, Hostname},
                                          {mfa, {ssl_test_lib, session_id, []}},
-                                         {from, self()},  {options, [{reuse_sessions, save} | ClientOpts]}]),
+                                         {from, self()},  {options, [{reuse_sessions, save},
+                                                                     {ciphers, Ciphers}| ClientOpts]}]),
     Server0 ! listen,
     
     Client1 = ssl_test_lib:start_client([{node, ClientNode},
                                          {port, Port0}, {host, Hostname},
                                          {mfa, {ssl_test_lib, session_id, []}},
-                                         {from, self()},  {options, ClientOpts}]),    
+                                         {from, self()},  {options,  [{ciphers, Ciphers} | ClientOpts]}]),
     
     SID = receive
               {Client0, Id0} ->
@@ -464,6 +472,8 @@ no_reuses_session_server_restart_new_cert() ->
 no_reuses_session_server_restart_new_cert(Config) when is_list(Config) ->
     ClientOpts = ssl_test_lib:ssl_options(client_rsa_der_opts, Config),
     ServerOpts = ssl_test_lib:ssl_options(server_rsa_der_verify_opts, Config),
+    POpts = proplists:get_value(protocol_opts, Config, []),
+
     #{client_config := NewCOpts,
       server_config := NewSOpts} = ssl_test_lib:make_cert_chains_der(rsa,
                                                                      [[{key, ssl_test_lib:hardcode_rsa_key(4)}],
@@ -493,19 +503,19 @@ no_reuses_session_server_restart_new_cert(Config) when is_list(Config) ->
     Server1 = ssl_test_lib:start_server([{node, ServerNode}, {port, Port},
                                          {from, self()},
                                          {mfa, {ssl_test_lib, session_info_result, []}},
-                                         {options, [{reuseaddr, true} | NewSOpts]}]),
+                                         {options, [{reuseaddr, true} | NewSOpts ++ POpts]}]),
 
     Client1 = ssl_test_lib:start_client([{node, ClientNode},
                                          {port, Port}, {host, Hostname},
                                          {mfa, {ssl_test_lib, session_info_result, []}},
-                                         {from, self()},  {options, NewCOpts}]),
+                                         {from, self()},  {options, NewCOpts ++ POpts}]),
     Info1 = receive {Server1, Info10} -> Info10 end,
 
     receive
 	{Client1, Info0} ->
 	    ct:fail(session_reused_when_server_has_new_cert);
 	{Client1, Info1} ->
-            ct:pal("First: ~p~nSecond ~p~n",[Info0, Info1]);
+            ?CT_LOG("First: ~p~nSecond ~p~n",[Info0, Info1]);
         Unexpected ->
             ct:fail({unexpected, Unexpected, {Client1, Info1}})
     end,
@@ -520,6 +530,8 @@ no_reuses_session_server_restart_new_cert_file() ->
 no_reuses_session_server_restart_new_cert_file(Config) when is_list(Config) ->
     ClientOpts = ssl_test_lib:ssl_options(client_rsa_opts, Config),
     ServerOpts = ssl_test_lib:ssl_options(server_rsa_verify_opts, Config),
+    POpts = proplists:get_value(protocol_opts, Config, []),
+
     #{client_config := NewCOpts,
       server_config := NewSOpts} = ssl_test_lib:make_cert_chains_pem(rsa,
                                                                      [[{key, ssl_test_lib:hardcode_rsa_key(4)}],
@@ -534,14 +546,14 @@ no_reuses_session_server_restart_new_cert_file(Config) when is_list(Config) ->
     Server =
 	ssl_test_lib:start_server([{node, ServerNode}, {port, 0},
 				   {from, self()},
-		      {mfa, {ssl_test_lib, session_info_result, []}},
+                                   {mfa, {ssl_test_lib, session_info_result, []}},
 				   {options, NewServerOpts0}]),
     Port = ssl_test_lib:inet_port(Server),
     Client0 =
 	ssl_test_lib:start_client([{node, ClientNode},
-		      {port, Port}, {host, Hostname},
-			    {mfa, {ssl_test_lib, no_result, []}},
-		      {from, self()},  {options, ClientOpts}]),
+                                   {port, Port}, {host, Hostname},
+                                   {mfa, {ssl_test_lib, no_result, []}},
+                                   {from, self()},  {options, ClientOpts}]),
     SessionInfo =
 	receive
 	    {Server, Info} ->
@@ -561,13 +573,13 @@ no_reuses_session_server_restart_new_cert_file(Config) when is_list(Config) ->
     Server1 =
 	ssl_test_lib:start_server([{node, ServerNode}, {port, Port},
 				   {from, self()},
-		      {mfa, {ssl_test_lib, no_result, []}},
-				   {options,  [{reuseaddr, true} | NewServerOpts1]}]),
+                                   {mfa, {ssl_test_lib, no_result, []}},
+				   {options,  [{reuseaddr, true} | NewServerOpts1 ++ POpts]}]),
     Client1 =
 	ssl_test_lib:start_client([{node, ClientNode},
-		      {port, Port}, {host, Hostname},
-		      {mfa, {ssl_test_lib, session_info_result, []}},
-				   {from, self()},  {options, NewCOpts}]),
+                                   {port, Port}, {host, Hostname},
+                                   {mfa, {ssl_test_lib, session_info_result, []}},
+				   {from, self()},  {options, NewCOpts ++ POpts}]),
     receive
 	{Client1, SessionInfo} ->
 	    ct:fail(session_reused_when_server_has_new_cert);
@@ -649,7 +661,7 @@ session_cachce_info(SessionCache) ->
 
 check_table_did_not_grow(SessionCachePid, N) ->
     {SessionCacheCb, SessionCacheDb} = session_cachce_info(SessionCachePid),
-    ct:pal("Run ~p ~p", [SessionCacheCb, SessionCacheDb]),
+    ?CT_LOG("Run ~p ~p", [SessionCacheCb, SessionCacheDb]),
     case catch SessionCacheCb:size(SessionCacheDb) of
         N ->
             ok;
@@ -668,9 +680,9 @@ faulty_client(Host, Port) ->
 
 
 encode_client_hello(CH, Random) ->
-    HSBin = tls_handshake:encode_handshake(CH, {3,3}),
+    HSBin = tls_handshake:encode_handshake(CH, ?TLS_1_2),
     CS = connection_states(Random),
-    {Encoded, _} = tls_record:encode_handshake(HSBin, {3,3}, CS),
+    {Encoded, _} = tls_record:encode_handshake(HSBin, ?TLS_1_2, CS),
     Encoded.
 
 client_hello(Random) ->
@@ -746,19 +758,22 @@ client_hello(Random) ->
 		   srp =>
 		       undefined},
 
-    #client_hello{client_version = {3,3},
+    #client_hello{client_version = ?TLS_1_2,
 		  random = Random,
 		  session_id = crypto:strong_rand_bytes(32),
 		  cipher_suites = CipherSuites,
-		  compression_methods = [0],
 		  extensions = Extensions
 		 }.
 
 connection_states(Random) ->
     #{current_write =>
-          #{beast_mitigation => one_n_minus_one,cipher_state => undefined,
-		 client_verify_data => undefined,compression_state => undefined,
-		 mac_secret => undefined,secure_renegotiation => undefined,
+          #{beast_mitigation => one_n_minus_one,
+            cipher_state => undefined,
+            mac_secret => undefined,
+            reneg => #{secure_renegotiation => undefined,
+                       client_verify_data => undefined,
+                       server_verify_data => undefined
+                      },
             security_parameters =>
                 #security_parameters{
                   cipher_suite = <<0,0>>,
@@ -766,19 +781,16 @@ connection_states(Random) ->
                    bulk_cipher_algorithm = 0,
                    cipher_type = 0,
                    iv_size = 0,
-                   key_size = 0,
                    key_material_length = 0,
-                   expanded_key_material_length = 0,
                    mac_algorithm = 0,
                    prf_algorithm = 0,
                    hash_size = 0,
-                   compression_algorithm = 0,
                    master_secret = undefined,
                    resumption_master_secret = undefined,
                    client_random = Random,
-                   server_random = undefined,
-                   exportable = undefined},
-            sequence_number => 0,server_verify_data => undefined,max_fragment_length => undefined}}.
+                   server_random = undefined},
+            sequence_number => 0,
+            max_fragment_length => undefined}}.
 
 
 
