@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2010-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2010-2023. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@
 
 -behaviour(ct_suite).
 
+-include("ssl_test_lib.hrl").
 -include_lib("common_test/include/ct.hrl").
 
 %% Callback functions
@@ -217,7 +218,7 @@ client_unique_session(Config) when is_list(Config) ->
 				   {options, ServerOpts}]),
     Port = ssl_test_lib:inet_port(Server),
     LastClient = clients_start(Server, ClientNode, Hostname, Port, ClientOpts, 20, []),
-    receive 
+    receive
 	{LastClient, {ok, _}} ->
 	    ok
     end,
@@ -227,12 +228,12 @@ client_unique_session(Config) when is_list(Config) ->
     ClientCache = element(2, State),
 
     1 = ?CLIENT_CB:size(ClientCache),
-  
+
     ssl_test_lib:close(Server, 500),
     ssl_test_lib:close(LastClient).
-		
+
 session_cleanup() ->
-    [{doc, "Test that sessions are cleand up eventually, so that the session table "
+    [{doc, "Test that sessions are cleaned up eventually, so that the session table "
      "does not grow and grow ..."}].
 session_cleanup(Config) when is_list(Config) ->
     process_flag(trap_exit, true),
@@ -380,7 +381,7 @@ max_table_size(Config) when is_list(Config) ->
     State = ssl_test_lib:state(Prop),
     ClientCache = element(2, State),	
     M = ?CLIENT_CB:size(ClientCache),
-    ct:pal("Cache size ~p",[M]),
+    ?CT_LOG("Cache size ~p",[M]),
     ssl_test_lib:close(Server, 500),
     ssl_test_lib:close(LastClient),
     true = M =< ?MAX_TABLE_SIZE.
@@ -560,8 +561,11 @@ clients_start(Server, ClientNode, Hostname, Port, ClientOpts, N, Opts) ->
     spawn_link(ssl_test_lib, start_client, 
 	       [[{node, ClientNode},
 		 {port, Port}, {host, Hostname},
-		 {mfa, {ssl_test_lib, no_result, []}},
+		 {mfa, {?MODULE, connection_info_result, []}},
 		 {from, self()},  {options, Opts ++ ClientOpts}]]),
+    receive  %% Sync client connect
+        {_, {ok, _}} -> ok
+    end,
     Server ! listen,
     wait_for_server(),
     clients_start(Server, ClientNode, Hostname, Port, ClientOpts, N-1, Opts).
