@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2019-2024. All Rights Reserved.
+%% Copyright Ericsson AB 2019-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -79,7 +79,9 @@
          client_cert_fail_alert_active/0,
          client_cert_fail_alert_active/1,
          client_cert_fail_alert_passive/0,
-         client_cert_fail_alert_passive/1
+         client_cert_fail_alert_passive/1,
+         keylog_on_alert/0,
+         keylog_on_alert/1
         ]).
 
 
@@ -117,7 +119,8 @@ tls_1_3_1_2_tests() ->
      middle_box_client_tls_v2_session_reused,
      renegotiate_error,
      client_cert_fail_alert_active,
-     client_cert_fail_alert_passive   
+     client_cert_fail_alert_passive,
+     keylog_on_alert
     ].
 legacy_tests() ->
     [tls_client_tls10_server,
@@ -134,8 +137,7 @@ legacy_tests() ->
     ].
 
 init_per_suite(Config) ->
-    catch crypto:stop(),
-    try crypto:start() of
+    case application:ensure_started(crypto) of
 	ok ->
             case ssl_test_lib:sufficient_crypto_support('tlsv1.3') of                
                 true ->
@@ -144,8 +146,8 @@ init_per_suite(Config) ->
                      Config];
                 false ->
                     {skip, "Insufficient crypto support for TLS-1.3"}
-            end
-    catch _:_ ->
+            end;
+        _ ->
 	    {skip, "Crypto did not start"}
     end.
 
@@ -533,7 +535,7 @@ client_cert_fail_alert_active(Config) when is_list(Config) ->
     ServerOpts0 = ssl_test_lib:ssl_options(extra_server, server_cert_opts, Config),
     PrivDir = proplists:get_value(priv_dir, Config),
 
-    NewClientCertFile = filename:join(PrivDir, "clinet_invalid_cert.pem"),
+    NewClientCertFile = filename:join(PrivDir, "client_invalid_cert.pem"),
     create_bad_client_certfile(NewClientCertFile, ClientOpts0),
 
     ClientOpts = [{active, true},
@@ -573,9 +575,9 @@ client_cert_fail_alert_passive(Config) when is_list(Config) ->
                   {certfile, NewClientCertFile} | proplists:delete(certfile, ClientOpts0)],
     ServerOpts = [{verify, verify_peer}, {fail_if_no_peer_cert, true}| ServerOpts0],
     alert_passive(ServerOpts, ClientOpts, recv,
-                  ServerNode, Hostname),
+                  ServerNode, Hostname, unknown_ca),
     alert_passive(ServerOpts, ClientOpts, setopts,
-                  ServerNode, Hostname).
+                  ServerNode, Hostname, unknown_ca).
 
 tls13_client_tls11_server() ->
     [{doc,"Test that a TLS 1.3 client gets old server alert from TLS 1.0 server."}].
@@ -584,7 +586,77 @@ tls13_client_tls11_server(Config) when is_list(Config) ->
     ServerOpts =  [{versions, ['tlsv1']} | ssl_test_lib:ssl_options(server_cert_opts, Config)],
     ssl_test_lib:basic_alert(ClientOpts, ServerOpts, Config, insufficient_security).
 
+keylog_on_alert() ->
+    [{doc,"Test that keep_secrets keylog_hs callback, if specified, "
+      "is called with keylog info when handshake alert is raised"}].
 
+keylog_on_alert(Config) when is_list(Config) ->
+    ssl:clear_pem_cache(),
+    {_, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+    ClientOpts0 = ssl_test_lib:ssl_options(extra_client, client_cert_opts, Config),
+    ServerOpts0 = ssl_test_lib:ssl_options(extra_server, server_cert_opts, Config),
+    PrivDir = proplists:get_value(priv_dir, Config),
+    NewClientCertFile = filename:join(PrivDir, "client_invalid_cert.pem"),
+    create_bad_client_certfile(NewClientCertFile, ClientOpts0),
+    ClientOpts = [{versions, ['tlsv1.3']}, {active, false}, {certfile, NewClientCertFile}|
+                  proplists:delete(certfile, ClientOpts0)],
+    ServerOpts =  [{versions, ['tlsv1.3']},
+                   {verify, verify_peer}, {fail_if_no_peer_cert, true}
+                  | ServerOpts0],
+
+    Me = self(),
+    Fun = fun(AlertInfo) ->
+                  Me ! {alert_info, AlertInfo}
+          end,
+    alert_passive([{keep_secrets, {keylog_hs, Fun}} | ServerOpts], ClientOpts, recv,
+                  ServerNode, Hostname, unknown_ca),
+
+    receive_server_keylog_for_client_cert_alert(),
+
+    alert_passive(ServerOpts, [{keep_secrets, {keylog_hs, Fun}} | ClientOpts], recv,
+                  ServerNode, Hostname, unknown_ca),
+
+    receive_client_keylog_for_client_cert_alert(),
+
+    ClientNoCert = proplists:delete(keyfile, proplists:delete(certfile, ClientOpts0)),
+    alert_passive([{keep_secrets, {keylog_hs, Fun}} | ServerOpts], [{active, false} | ClientNoCert], recv,
+                  ServerNode, Hostname, certificate_required),
+
+    receive_server_keylog_for_client_cert_alert().
+
+receive_server_keylog_for_client_cert_alert() ->
+    %% This alert will be decrypted with application secrets
+    %% as client is already in connection
+    receive
+        {alert_info, #{items := SKeyLog1}} ->
+            case SKeyLog1 of
+                ["SERVER_TRAFFIC_SECRET_0"++_] ->
+                    ok;
+                S1Other ->
+                    ct:fail({server_received, S1Other})
+            end
+    end,
+
+    receive
+        {alert_info, #{items := SKeyLog}} ->
+            case SKeyLog of
+                ["CLIENT_HANDSHAKE_TRAFFIC_SECRET"++_,_|_] ->
+                    ok;
+                SOther ->
+                    ct:fail({server_received, SOther})
+            end
+    end.
+
+receive_client_keylog_for_client_cert_alert() ->
+    receive
+        {alert_info, #{items := CKeyLog}} ->
+            case CKeyLog of
+                ["CLIENT_HANDSHAKE_TRAFFIC_SECRET"++_,_,_,_|_] ->
+                    ok;
+                COther ->
+                    ct:fail({client_received, COther})
+            end
+    end.
 %%--------------------------------------------------------------------
 %% Internal functions and callbacks -----------------------------------
 %%--------------------------------------------------------------------
@@ -616,7 +688,7 @@ check_session_id(Socket, Expected) ->
     end.
 
 alert_passive(ServerOpts, ClientOpts, Function,
-              ServerNode, Hostname) ->
+              ServerNode, Hostname, AlertAtom) ->
     Server = ssl_test_lib:start_server([{node, ServerNode}, {port, 0},
                                         {from, self()},
                                         {mfa, {ssl_test_lib, no_result, []}},
@@ -626,7 +698,7 @@ alert_passive(ServerOpts, ClientOpts, Function,
     ct:sleep(500),
     case Function of
         recv ->
-            {error, {tls_alert, {unknown_ca,_}}} = ssl:recv(Socket, 0);
+            {error, {tls_alert, {AlertAtom,_}}} = ssl:recv(Socket, 0);
         setopts ->
             {error, {tls_alert, {unknown_ca,_}}} = ssl:setopts(Socket, [{active, once}])
     end.
@@ -645,15 +717,15 @@ create_bad_client_certfile(NewClientCertFile, ClientOpts) ->
 
 test_rsa_pcks1_cert(SHA, COpts, SOpts, Config) ->
     #{client_config := ClientOpts,
-      server_config := ServerOpts} = public_key:pkix_test_data(#{server_chain => #{root => root_key(SHA),
-                                                                                   intermediates => intermediates(SHA, 1),
-                                                                                   peer => peer_key(SHA)},
-                                                                 client_chain => #{root => root_key(SHA),
-                                                                                   intermediates => intermediates(SHA, 1),
-                                                                                   peer => peer_key(SHA)}}),
+      server_config := ServerOpts} =
+        public_key:pkix_test_data(#{server_chain => #{root => root_key(SHA),
+                                                      intermediates => intermediates(SHA, 1),
+                                                      peer => peer_key(SHA)},
+                                    client_chain => #{root => root_key(SHA),
+                                                      intermediates => intermediates(SHA, 1),
+                                                      peer => peer_key(SHA)}}),
     ssl_test_lib:basic_test(COpts ++ ClientOpts, SOpts ++ ServerOpts, Config).
 
-
 root_key(SHA) ->
     root_key(SHA, undefined).
 
