@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1997-2021. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2024. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@
 %% %CopyrightEnd%
 %%
 -module(erl_posix_msg).
+-moduledoc false.
 
 %% Converts from errno identifiers to error messages.
 
@@ -167,9 +168,9 @@ message_1(eunatch) -> <<"protocol driver not attached">>;
 message_1(eusers) -> <<"too many users">>;
 message_1(eversion) -> <<"version mismatch">>;
 message_1(ewouldblock) -> <<"operation would block">>;
-message_1(exdev) -> <<"cross-domain link">>;
+message_1(exdev) -> <<"cross-device link">>;
 message_1(exfull) -> <<"message tables full">>;
 message_1(nxdomain) -> <<"non-existing domain">>;
 message_1(exbadport) -> <<"inet_drv bad port state">>;
 message_1(exbadseq) -> <<"inet_drv bad request sequence">>;
-message_1(_) -> <<"unknown POSIX error">>.
+message_1(Other) -> <<"unknown POSIX error: ", (atom_to_binary(Other))/binary>>.
