@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1997-2022. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -18,6 +18,49 @@
 %% %CopyrightEnd%
 %%
 -module(filename).
+-moduledoc """
+Filename manipulation functions.
+
+This module provides functions for analyzing and manipulating filenames. These
+functions are designed so that the Erlang code can work on many different
+platforms with different filename formats. With filename is meant all strings
+that can be used to denote a file. The filename can be a short relative name
+like `foo.erl`, a long absolute name including a drive designator, a directory
+name like `D:\usr/local\bin\erl/lib\tools\foo.erl`, or any variations in
+between.
+
+In Windows, all functions return filenames with forward slashes only, even if
+the arguments contain backslashes. To normalize a filename by removing redundant
+directory separators, use `join/1`.
+
+The module supports [raw filenames](unicode_usage.md#notes-about-raw-filenames)
+in the way that if a binary is present, or the filename cannot be interpreted
+according to the return value of `file:native_name_encoding/0`, a raw filename
+is also returned. For example, [`join/1`](`join/1`) provided with a path
+component that is a binary (and cannot be interpreted under the current native
+filename encoding) results in a raw filename that is returned (the join
+operation is performed of course). For more information about raw filenames, see
+the `m:file` module.
+
+> #### Note {: .info }
+>
+> Functionality in this module generally assumes valid input and does not
+> necessarily fail on input that does not use a valid encoding, but may instead
+> very likely produce invalid output.
+>
+> File operations used to accept filenames containing null characters (integer
+> value zero). This caused the name to be truncated and in some cases arguments
+> to primitive operations to be mixed up. Filenames containing null characters
+> inside the filename are now _rejected_ and will cause primitive file
+> operations to fail.
+
+> #### Warning {: .warning }
+>
+> Currently null characters at the end of the filename will be accepted by
+> primitive file operations. Such filenames are however still documented as
+> invalid. The implementation will also change in the future and reject such
+> filenames.
+""".
 
 -removed([{find_src,'_',"use filelib:find_source/1,3 instead"}]).
 -removed([{safe_relative_path,1,"use filelib:safe_relative_path/2 instead"}]).
@@ -78,8 +121,10 @@
 
 -include_lib("kernel/include/file.hrl").
 
--define(IS_DRIVELETTER(Letter),(((Letter >= $A) andalso (Letter =< $Z)) orelse
-				((Letter >= $a) andalso (Letter =< $z)))). 
+-define(IS_DRIVELETTER(Letter),
+        (is_integer(Letter)
+         andalso (($A =< Letter andalso Letter =< $Z)
+                  orelse ($a =< Letter andalso Letter =< $z)))).
 
 %% Converts a relative filename to an absolute filename
 %% or the filename itself if it already is an absolute filename
@@ -98,12 +143,47 @@
 %% (for WIN32): absname("/") -> "D:/"
 
 
+-doc """
+Converts a relative `Filename` and returns an absolute name. No attempt is made
+to create the shortest absolute name, as this can give incorrect results on file
+systems that allow links.
+
+_Unix examples:_
+
+```erlang
+1> pwd().
+"/usr/local"
+2> filename:absname("foo").
+"/usr/local/foo"
+3> filename:absname("../x").
+"/usr/local/../x"
+4> filename:absname("/").
+"/"
+```
+
+_Windows examples:_
+
+```erlang
+1> pwd().
+"D:/usr/local"
+2> filename:absname("foo").
+"D:/usr/local/foo"
+3> filename:absname("../x").
+"D:/usr/local/../x"
+4> filename:absname("/").
+"D:/"
+```
+""".
 -spec absname(Filename) -> file:filename_all() when
       Filename :: file:name_all().
 absname(Name) ->
     {ok, Cwd} = file:get_cwd(),
     absname(Name, Cwd).
 
+-doc """
+Same as `absname/1`, except that the directory to which the filename is to be
+made relative is specified in argument `Dir`.
+""".
 -spec absname(Filename, Dir) -> file:filename_all() when
       Filename :: file:name_all(),
       Dir :: file:name_all().
@@ -159,6 +239,14 @@ absname_vr([[X, $:]|Name], _, _AbsBase) ->
 %% This is just a join/2, but assumes that 
 %% AbsBase must be absolute and Name must be relative.
 
+-doc """
+Joins an absolute directory with a relative filename.
+
+Similar to `join/2`, but on platforms with tight restrictions on raw filename length
+and no support for symbolic links, leading parent directory components in `Filename` are matched
+against trailing directory components in `Dir` so they can be removed from the
+result - minimizing its length.
+""".
 -spec absname_join(Dir, Filename) -> file:filename_all() when
       Dir :: file:name_all(),
       Filename :: file:name_all().
@@ -173,6 +261,21 @@ absname_join(AbsBase, Name) ->
 %%           basename("/usr/foo/") -> "foo"  (trailing slashes ignored)
 %%           basename("/") -> []
 
+-doc """
+Returns the last component of `Filename`, or `Filename` itself if it does not
+contain any directory separators.
+
+_Examples:_
+
+```erlang
+5> filename:basename("foo").
+"foo"
+6> filename:basename("/usr/foo").
+"foo"
+7> filename:basename("/").
+[]
+```
+""".
 -spec basename(Filename) -> file:filename_all() when
       Filename :: file:name_all().
 
@@ -239,6 +342,28 @@ skip_prefix(Name, _) ->
 %%	    rootname(basename("xxx.jam")) -> "xxx"
 %%	    rootname(basename("xxx.erl")) -> "xxx"
 
+-doc """
+Returns the last component of `Filename` with extension `Ext` stripped.
+
+This function is to be used to remove a (possible) specific extension.
+To remove an existing extension when you are unsure which one it is, use
+[`rootname(basename(Filename))`](`rootname/1`).
+
+_Examples:_
+
+```erlang
+8> filename:basename("~/src/kalle.erl", ".erl").
+"kalle"
+9> filename:basename("~/src/kalle.beam", ".erl").
+"kalle.beam"
+10> filename:basename("~/src/kalle.old.erl", ".erl").
+"kalle.old"
+11> filename:rootname(filename:basename("~/src/kalle.erl")).
+"kalle"
+12> filename:rootname(filename:basename("~/src/kalle.beam")).
+"kalle"
+```
+""".
 -spec basename(Filename, Ext) -> file:filename_all() when
       Filename :: file:name_all(),
       Ext :: file:name_all().
@@ -289,6 +414,23 @@ basename([], _Ext, Tail, _DrvSep2) ->
 %% Example: dirname("/usr/src/kalle.erl") -> "/usr/src",
 %%	    dirname("kalle.erl") -> "."
 
+-doc """
+Returns the directory part of `Filename`.
+
+_Examples:_
+
+```erlang
+13> filename:dirname("/usr/src/kalle.erl").
+"/usr/src"
+14> filename:dirname("kalle.erl").
+"."
+```
+
+```erlang
+5> filename:dirname("\\usr\\src/kalle.erl"). % Windows
+"/usr/src"
+```
+""".
 -spec dirname(Filename) -> file:filename_all() when
       Filename :: file:name_all().
 dirname(Name) when is_binary(Name) ->
@@ -333,8 +475,7 @@ dirname([$/|Rest], Dir, File, Seps) ->
 dirname([DirSep|Rest], Dir, File, {DirSep,_}=Seps) when is_integer(DirSep) ->
     dirname(Rest, File++Dir, [$/], Seps);
 dirname([Dl,DrvSep|Rest], [], [], {_,DrvSep}=Seps)
-  when is_integer(DrvSep), ((($a =< Dl) and (Dl =< $z)) or
-			    (($A =< Dl) and (Dl =< $Z))) ->
+  when is_integer(DrvSep), ?IS_DRIVELETTER(Dl) ->
     dirname(Rest, [DrvSep,Dl], [], Seps);
 dirname([Char|Rest], Dir, File, Seps) when is_integer(Char) ->
     dirname(Rest, Dir, [Char|File], Seps);
@@ -382,6 +523,19 @@ dirjoin1([H|T],Acc,Sep) ->
 %%
 %% On Windows:  fn:dirname("\\usr\\src/kalle.erl") -> "/usr/src"
 
+-doc """
+Returns the file extension of `Filename`, including the period. Returns an empty
+string if no extension exists.
+
+_Examples:_
+
+```erlang
+15> filename:extension("foo.erl").
+".erl"
+16> filename:extension("beam.src/kalle").
+[]
+```
+""".
 -spec extension(Filename) -> file:filename_all() when
       Filename :: file:name_all().
 extension(Name) when is_binary(Name) ->
@@ -434,6 +588,31 @@ extension([], Result, _OsType) ->
 
 %% Joins a list of filenames with directory separators.
 
+-doc """
+Joins a list of filename `Components` with directory separators. If one of the
+elements of `Components` includes an absolute path, such as `"/xxx"`, the
+preceding elements, if any, are removed from the result.
+
+The result is "normalized":
+
+- Redundant directory separators are removed.
+- In Windows, all directory separators are forward slashes and the drive letter
+  is in lower case.
+
+_Examples:_
+
+```erlang
+17> filename:join(["/usr", "local", "bin"]).
+"/usr/local/bin"
+18> filename:join(["a/b///c/"]).
+"a/b/c"
+```
+
+```erlang
+6> filename:join(["B:a\\b///c/"]). % Windows
+"b:a/b/c"
+```
+""".
 -spec join(Components) -> file:filename_all() when
       Components :: [file:name_all()].
 join([Name1, Name2|Rest]) ->
@@ -447,6 +626,10 @@ join([Name]) when is_atom(Name) ->
 
 %% Joins two filenames with directory separators.
 
+-doc """
+Joins two filename components with directory separators. Equivalent to
+[`join([Name1, Name2])`](`join/1`).
+""".
 -spec join(Name1, Name2) -> file:filename_all() when
       Name1 :: file:name_all(),
       Name2 :: file:name_all().
@@ -551,6 +734,7 @@ maybe_remove_dirsep(Name, _) ->
 %% a given base directory, which is is assumed to be normalised
 %% by a previous call to join/{1,2}.
 
+-doc false.
 -spec append(file:filename_all(), file:name_all()) -> file:filename_all().
 append(Dir, Name) when is_binary(Dir), is_binary(Name) ->
     <<Dir/binary,$/:8,Name/binary>>;
@@ -573,6 +757,26 @@ append(Dir, Name) ->
 %%		current working volume.  (Windows only)
 %%		Example: a:bar.erl, /temp/foo.erl
 
+-doc """
+Returns the path type, which is one of the following:
+
+- **`absolute`** - The path name refers to a specific file on a specific volume.
+
+  Unix example: `/usr/local/bin`
+
+  Windows example: `D:/usr/local/bin`
+
+- **`relative`** - The path name is relative to the current working directory on
+  the current volume.
+
+  Example: `foo/bar, ../src`
+
+- **`volumerelative`** - The path name is relative to the current working
+  directory on a specified volume, or it is a specific file on the current
+  working volume.
+
+  Windows example: `D:bar.erl, /bar/foo.erl`
+""".
 -spec pathtype(Path) -> 'absolute' | 'relative' | 'volumerelative' when
       Path :: file:name_all().
 pathtype(Atom) when is_atom(Atom) ->
@@ -629,6 +833,18 @@ win32_pathtype(_) 		  -> relative.
 %% Examples: rootname("/jam.src/kalle") -> "/jam.src/kalle"
 %%           rootname("/jam.src/foo.erl") -> "/jam.src/foo"
 
+-doc """
+Removes the filename extension.
+
+_Examples:_
+
+```erlang
+1> filename:rootname("/beam.src/kalle").
+"/beam.src/kalle"
+2> filename:rootname("/beam.src/foo.erl").
+"/beam.src/foo"
+```
+""".
 -spec rootname(Filename) -> file:filename_all() when
       Filename :: file:name_all().
 rootname(Name) when is_binary(Name) ->
@@ -659,6 +875,18 @@ rootname([], Root, _Ext, _OsType) ->
 %% Examples: rootname("/jam.src/kalle.jam", ".erl") -> "/jam.src/kalle.jam"
 %%           rootname("/jam.src/foo.erl", ".erl") -> "/jam.src/foo"
 
+-doc """
+Removes the filename extension `Ext` from `Filename`.
+
+_Examples:_
+
+```erlang
+1> filename:rootname("/beam.src/foo.erl", ".erl").
+"/beam.src/foo"
+2> filename:rootname("/beam.src/foo.beam", ".erl").
+"/beam.src/foo.beam"
+```
+""".
 -spec rootname(Filename, Ext) -> file:filename_all() when
       Filename :: file:name_all(),
       Ext :: file:name_all().
@@ -691,6 +919,20 @@ rootname2([Char|Rest], Ext, Result, OsType) when is_integer(Char) ->
 %% split("foo/bar") -> ["foo", "bar"]
 %% split("a:\\msdev\\include") -> ["a:/", "msdev", "include"]
 
+-doc """
+Returns a list whose elements are the path components of `Filename`.
+
+_Examples:_
+
+```erlang
+24> filename:split("/usr/local/bin").
+["/","usr","local","bin"]
+25> filename:split("foo/bar").
+["foo","bar"]
+26> filename:split("a:\\msdev\\include").
+["a:/","msdev","include"]
+```
+""".
 -spec split(Filename) -> Components when
       Filename :: file:name_all(),
       Components :: [file:name_all()].
@@ -757,7 +999,8 @@ win32_split([X, $\\|Rest]) when is_integer(X) ->
     win32_split([X, $/|Rest]);
 win32_split([X, Y, $\\|Rest]) when is_integer(X), is_integer(Y) ->
     win32_split([X, Y, $/|Rest]);
-win32_split([UcLetter, $:|Rest]) when UcLetter >= $A, UcLetter =< $Z ->
+win32_split([UcLetter, $:|Rest])
+  when is_integer(UcLetter), $A =< UcLetter, UcLetter =< $Z ->
     win32_split([UcLetter+$a-$A, $:|Rest]);
 win32_split([Letter, $:, $/|Rest]) ->
     split(Rest, [], [[Letter, $:, $/]], win32);
@@ -791,6 +1034,23 @@ split([], Comp, Components, OsType) ->
 %% will be converted to backslashes.  On all platforms, the
 %% name will be normalized as done by join/1.
 
+-doc """
+Converts `Path` to a form accepted by the command shell and native applications
+on the current platform. On Windows, forward slashes are converted to backward
+slashes. On all platforms, the name is normalized as done by `join/1`.
+
+_Examples:_
+
+```erlang
+19> filename:nativename("/usr/local/bin/"). % Unix
+"/usr/local/bin"
+```
+
+```erlang
+7> filename:nativename("/usr/local/bin/"). % Windows
+"\\usr\\local\\bin"
+```
+""".
 -spec nativename(Path) -> file:filename_all() when
       Path :: file:name_all().
 nativename(Name0) ->
@@ -822,6 +1082,10 @@ major_os_type() ->
 %% flatten(List)
 %%  Flatten a list, also accepting atoms.
 
+-doc """
+Converts a possibly deep list filename consisting of characters and atoms into
+the corresponding flat string filename.
+""".
 -spec flatten(Filename) -> file:filename_all() when
       Filename :: file:name_all().
 flatten(Bin) when is_binary(Bin) ->
@@ -860,6 +1124,11 @@ filename_string_to_binary(List) ->
                           os => 'windows' | 'darwin' | 'linux',
                           version => string() | binary()}.
 
+-doc """
+Equivalent to [basedir(PathType, Application, #\{\})](`m:filename#basedir_3_1`)
+or [basedir(PathsType, Application, #\{\})](`m:filename#basedir_3_2`).
+""".
+-doc(#{since => <<"OTP 19.0">>}).
 -spec basedir(PathType,Application) -> file:filename_all() when
       PathType :: basedir_path_type(),
       Application :: string() | binary();
@@ -871,6 +1140,185 @@ basedir(Type,Application) when is_atom(Type), is_list(Application) orelse
                                               is_binary(Application) ->
     basedir(Type, Application, #{}).
 
+-doc """
+[](){: #basedir_3_1 } [](){: #basedir_3_2 }
+
+Returns a suitable path, or paths, for a given type. If `os` is not set in
+`Opts` the function will default to the native option, that is `'linux'`,
+`'darwin'` or `'windows'`, as understood by `os:type/0`. Anything not recognized
+as `'darwin'` or `'windows'` is interpreted as `'linux'`.
+
+The options `'author'` and `'version'` are only used with `'windows'` option
+mode.
+
+- `user_cache`{: #user_cache }
+
+  The path location is intended for transient data files on a local machine.
+
+  On Linux: Respects the os environment variable `XDG_CACHE_HOME`.
+
+  ```erlang
+  1> filename:basedir(user_cache, "my_application", #{os=>linux}).
+  "/home/otptest/.cache/my_application"
+  ```
+
+  On Darwin:
+
+  ```erlang
+  1> filename:basedir(user_cache, "my_application", #{os=>darwin}).
+  "/home/otptest/Library/Caches/my_application"
+  ```
+
+  On Windows:
+
+  ```erlang
+  1> filename:basedir(user_cache, "My App").
+  "c:/Users/otptest/AppData/Local/My App/Cache"
+  2> filename:basedir(user_cache, "My App").
+  "c:/Users/otptest/AppData/Local/My App/Cache"
+  3> filename:basedir(user_cache, "My App", #{author=>"Erlang"}).
+  "c:/Users/otptest/AppData/Local/Erlang/My App/Cache"
+  4> filename:basedir(user_cache, "My App", #{version=>"1.2"}).
+  "c:/Users/otptest/AppData/Local/My App/1.2/Cache"
+  5> filename:basedir(user_cache, "My App", #{author=>"Erlang",version=>"1.2"}).
+  "c:/Users/otptest/AppData/Local/Erlang/My App/1.2/Cache"
+  ```
+
+- `user_config`{: #user_config }
+
+  The path location is intended for persistent configuration files.
+
+  On Linux: Respects the os environment variable `XDG_CONFIG_HOME`.
+
+  ```erlang
+  2> filename:basedir(user_config, "my_application", #{os=>linux}).
+  "/home/otptest/.config/my_application"
+  ```
+
+  On Darwin:
+
+  ```erlang
+  2> filename:basedir(user_config, "my_application", #{os=>darwin}).
+  "/home/otptest/Library/Application Support/my_application"
+  ```
+
+  On Windows:
+
+  ```erlang
+  1> filename:basedir(user_config, "My App").
+  "c:/Users/otptest/AppData/Roaming/My App"
+  2> filename:basedir(user_config, "My App", #{author=>"Erlang", version=>"1.2"}).
+  "c:/Users/otptest/AppData/Roaming/Erlang/My App/1.2"
+  ```
+
+- `user_data`{: #user_data }
+
+  The path location is intended for persistent data files.
+
+  On Linux: Respects the os environment variable `XDG_DATA_HOME`.
+
+  ```erlang
+  3> filename:basedir(user_data, "my_application", #{os=>linux}).
+  "/home/otptest/.local/my_application"
+  ```
+
+  On Darwin:
+
+  ```erlang
+  3> filename:basedir(user_data, "my_application", #{os=>darwin}).
+  "/home/otptest/Library/Application Support/my_application"
+  ```
+
+  On Windows:
+
+  ```erlang
+  8> filename:basedir(user_data, "My App").
+  "c:/Users/otptest/AppData/Local/My App"
+  9> filename:basedir(user_data, "My App",#{author=>"Erlang",version=>"1.2"}).
+  "c:/Users/otptest/AppData/Local/Erlang/My App/1.2"
+  ```
+
+- `user_log`{: #user_log }
+
+  The path location is intended for transient log files on a local machine.
+
+  On Linux: Respects the os environment variable `XDG_CACHE_HOME`.
+
+  ```erlang
+  4> filename:basedir(user_log, "my_application", #{os=>linux}).
+  "/home/otptest/.cache/my_application/log"
+  ```
+
+  On Darwin:
+
+  ```erlang
+  4> filename:basedir(user_log, "my_application", #{os=>darwin}).
+  "/home/otptest/Library/Logs/my_application"
+  ```
+
+  On Windows:
+
+  ```erlang
+  12> filename:basedir(user_log, "My App").
+  "c:/Users/otptest/AppData/Local/My App/Logs"
+  13> filename:basedir(user_log, "My App",#{author=>"Erlang",version=>"1.2"}).
+  "c:/Users/otptest/AppData/Local/Erlang/My App/1.2/Logs"
+  ```
+
+- `site_config`{: #site_config }
+
+  On Linux: Respects the os environment variable `XDG_CONFIG_DIRS`.
+
+  ```erlang
+  5> filename:basedir(site_config, "my_application", #{os=>linux}).
+  ["/usr/local/share/my_application",
+   "/usr/share/my_application"]
+  6> os:getenv("XDG_CONFIG_DIRS").
+  "/etc/xdg/xdg-ubuntu:/usr/share/upstart/xdg:/etc/xdg"
+  7> filename:basedir(site_config, "my_application", #{os=>linux}).
+  ["/etc/xdg/xdg-ubuntu/my_application",
+   "/usr/share/upstart/xdg/my_application",
+   "/etc/xdg/my_application"]
+  8> os:unsetenv("XDG_CONFIG_DIRS").
+  true
+  9> filename:basedir(site_config, "my_application", #{os=>linux}).
+  ["/etc/xdg/my_application"]
+  ```
+
+  On Darwin:
+
+  ```erlang
+  5> filename:basedir(site_config, "my_application", #{os=>darwin}).
+  ["/Library/Application Support/my_application"]
+  ```
+
+- `site_data`{: #site_data }
+
+  On Linux: Respects the os environment variable `XDG_DATA_DIRS`.
+
+  ```erlang
+  10> os:getenv("XDG_DATA_DIRS").
+  "/usr/share/ubuntu:/usr/share/gnome:/usr/local/share/:/usr/share/"
+  11> filename:basedir(site_data, "my_application", #{os=>linux}).
+  ["/usr/share/ubuntu/my_application",
+   "/usr/share/gnome/my_application",
+   "/usr/local/share/my_application",
+   "/usr/share/my_application"]
+  12> os:unsetenv("XDG_DATA_DIRS").
+  true
+  13> filename:basedir(site_data, "my_application", #{os=>linux}).
+  ["/usr/local/share/my_application",
+   "/usr/share/my_application"]
+  ```
+
+  On Darwin:
+
+  ```erlang
+  5> filename:basedir(site_data, "my_application", #{os=>darwin}).
+  ["/Library/Application Support/my_application"]
+  ```
+""".
+-doc(#{since => <<"OTP 19.0">>}).
 -spec basedir(PathType,Application,Opts) -> file:filename_all() when
       PathType :: basedir_path_type(),
       Application :: string() | binary(),
@@ -1039,6 +1487,7 @@ basedir_os_type() ->
 %% validate/1
 %%
 
+-doc false.
 -spec validate(FileName) -> boolean() when
       FileName :: file:name_all().
 
