@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1997-2017. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2022. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -23,13 +23,12 @@
 -export([absname/1, absname_2/1, 
 	 basename_1/1, basename_2/1,
 	 dirname/1, extension/1, join/1, t_nativename/1]).
--export([pathtype/1,rootname/1,split/1,find_src/1]).
+-export([pathtype/1,rootname/1,split/1]).
 -export([absname_bin/1, absname_bin_2/1, 
 	 basename_bin_1/1, basename_bin_2/1,
 	 dirname_bin/1, extension_bin/1, join_bin/1, t_nativename_bin/1]).
 -export([pathtype_bin/1,rootname_bin/1,split_bin/1]).
 -export([t_basedir_api/1, t_basedir_xdg/1, t_basedir_windows/1]).
--export([safe_relative_path/1]).
 
 -include_lib("common_test/include/ct.hrl").
 
@@ -39,11 +38,9 @@ suite() ->
 
 all() -> 
     [absname, absname_2,
-     find_src,
      absname_bin, absname_bin_2,
      {group,p},
-     t_basedir_xdg, t_basedir_windows,
-     safe_relative_path].
+     t_basedir_xdg, t_basedir_windows].
 
 groups() -> 
     [{p, [parallel],
@@ -313,11 +310,18 @@ extension(Config) when is_list(Config) ->
     ".erl" = filename:extension("A:/usr.bar/foo.nisse.erl"),
     "" = filename:extension("A:/usr.bar/foo"),
     "" = filename:extension("A:/usr/foo"),
+    "" = filename:extension(".gitignore"),
+    "" = filename:extension("A:/usr/.gitignore"),
+    ".ignore" = filename:extension("A:/usr/..ignore"),
+    ".ignore" = filename:extension("A:/usr/.git.ignore"),
     case os:type() of
         {win32, _} ->
             "" = filename:extension("A:\\usr\\foo"),
             ".erl" = filename:extension("A:/usr.bar/foo.nisse.erl"),
             "" = filename:extension("A:/usr.bar/foo"),
+            "" = filename:extension("A:\\usr\\.gitignore"),
+            ".ignore" = filename:extension("A:\\usr\\..ignore"),
+            ".ignore" = filename:extension("A:\\usr\\.git.ignore"),
             ok;
         _ -> ok
     end.
@@ -441,8 +445,14 @@ pathtype(Config) when is_list(Config) ->
 rootname(Config) when is_list(Config) ->
     "/jam.src/kalle" = filename:rootname("/jam.src/kalle"),
     "/jam.src/foo" = filename:rootname("/jam.src/foo.erl"),
+    "/jam.src/.gitignore" = filename:rootname("/jam.src/.gitignore"),
+    "/jam.src/.git" = filename:rootname("/jam.src/.git.ignore"),
+    "/jam.src/." = filename:rootname("/jam.src/..gitignore"),
     "/jam.src/foo" = filename:rootname(["/ja",'m.sr',"c/foo.erl"]),
     "/jam.src/foo" = filename:rootname("/jam.src/foo.erl", ".erl"),
+    "/jam.src/.gitignore" = filename:rootname("/jam.src/.gitignore", ".gitignore"),
+    "/jam.src/.git" = filename:rootname("/jam.src/.git.ignore", ".ignore"),
+    "/jam.src/." = filename:rootname("/jam.src/..gitignore", ".gitignore"),
     "/jam.src/foo.jam" = filename:rootname("/jam.src/foo.jam", ".erl"),
     "/jam.src/foo.jam" = filename:rootname(["/jam.sr",'c/foo.j',"am"],".erl"),
     "/jam.src/foo.jam" = filename:rootname(["/jam.sr",'c/foo.j'|am],".erl"),
@@ -496,33 +506,6 @@ t_nativename(Config) when is_list(Config) ->
                 filename:nativename("/usr/tmp//arne/")
     end.
 
-find_src(Config) when is_list(Config) ->
-    {Source,_} = filename:find_src(file),
-    ["file"|_] = lists:reverse(filename:split(Source)),
-    {Source,_} = filename:find_src(file, [{"",""}, {"ebin","src"}]),
-    {Source,_} = filename:find_src(Source),
-    {Source,_} = filename:find_src(Source ++ ".erl"),
-
-    %% Try to find the source for a preloaded module.
-    {error,{preloaded,init}} = filename:find_src(init),
-
-    %% Make sure that find_src works for a slim BEAM file.
-    OldPath = code:get_path(),
-    try
-        PrivDir = proplists:get_value(priv_dir, Config),
-        code:add_patha(PrivDir),
-        Src = "simple",
-        SrcPath = filename:join(PrivDir, Src) ++ ".erl",
-        SrcContents = "-module(simple).\n",
-        ok = file:write_file(SrcPath, SrcContents),
-        {ok,simple} = compile:file(SrcPath, [slim,{outdir,PrivDir}]),
-        BeamPath = filename:join(PrivDir, Src),
-        {BeamPath,[]} = filename:find_src(simple)
-    after
-        code:set_path(OldPath)
-    end,
-    ok.
-
 %%
 %%
 %% With binaries
@@ -693,11 +676,18 @@ extension_bin(Config) when is_list(Config) ->
     <<".erl">> = filename:extension(<<"A:/usr.bar/foo.nisse.erl">>),
     <<"">> = filename:extension(<<"A:/usr.bar/foo">>),
     <<"">> = filename:extension(<<"A:/usr/foo">>),
+    <<"">> = filename:extension(<<".gitignore">>),
+    <<"">> = filename:extension(<<"A:/usr/.gitignore">>),
+    <<".ignore">> = filename:extension(<<"A:/usr/..ignore">>),
+    <<".ignore">> = filename:extension(<<"A:/usr/.git.ignore">>),
     case os:type() of
         {win32, _} ->
             <<"">> = filename:extension(<<"A:\\usr\\foo">>),
             <<".erl">> = filename:extension(<<"A:/usr.bar/foo.nisse.erl">>),
             <<"">> = filename:extension(<<"A:/usr.bar/foo">>),
+            <<"">> = filename:extension(<<"A:/usr/.gitignore">>),
+            <<".ignore">> = filename:extension(<<"A:/usr/..ignore">>),
+            <<".ignore">> = filename:extension(<<"A:/usr/.git.ignore">>),
             ok;
         _ -> ok
     end.
@@ -884,71 +874,6 @@ t_nativename_bin(Config) when is_list(Config) ->
                 filename:nativename(<<"/usr/tmp//arne/">>)
     end.
 
-safe_relative_path(Config) ->
-    PrivDir = proplists:get_value(priv_dir, Config),
-    Root = filename:join(PrivDir, "filename_SUITE_safe_relative_path"),
-    ok = file:make_dir(Root),
-    ok = file:set_cwd(Root),
-
-    ok = file:make_dir("a"),
-    ok = file:set_cwd("a"),
-    ok = file:make_dir("b"),
-    ok = file:set_cwd("b"),
-    ok = file:make_dir("c"),
-
-    ok = file:set_cwd(Root),
-
-    "a" = test_srp("a"),
-    "a/b" = test_srp("a/b"),
-    "a/b" = test_srp("a/./b"),
-    "a/b" = test_srp("a/./b/."),
-
-    "" = test_srp("a/.."),
-    "" = test_srp("a/./.."),
-    "" = test_srp("a/../."),
-    "a" = test_srp("a/b/.."),
-    "a" = test_srp("a/../a"),
-    "a" = test_srp("a/../a/../a"),
-    "a/b/c" = test_srp("a/../a/b/c"),
-
-    unsafe = test_srp("a/../.."),
-    unsafe = test_srp("a/../../.."),
-    unsafe = test_srp("a/./../.."),
-    unsafe = test_srp("a/././../../.."),
-    unsafe = test_srp("a/b/././../../.."),
-
-    unsafe = test_srp(PrivDir),                 %Absolute path.
-
-    ok.
-
-test_srp(RelPath) ->
-    Res = do_test_srp(RelPath),
-    Res = case do_test_srp(list_to_binary(RelPath)) of
-              Bin when is_binary(Bin) ->
-                  binary_to_list(Bin);
-              Other ->
-                  Other
-          end.
-
-do_test_srp(RelPath) ->
-    {ok,Root} = file:get_cwd(),
-    ok = file:set_cwd(RelPath),
-    {ok,Cwd} = file:get_cwd(),
-    ok = file:set_cwd(Root),
-    case filename:safe_relative_path(RelPath) of
-        unsafe ->
-            true = length(Cwd) < length(Root),
-            unsafe;
-        "" ->
-            "";
-        SafeRelPath ->
-            ok = file:set_cwd(SafeRelPath),
-            {ok,Cwd} = file:get_cwd(),
-            true = length(Cwd) >= length(Root),
-            ok = file:set_cwd(Root),
-            SafeRelPath
-    end.
-
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% basedirs
 t_basedir_api(Config) when is_list(Config) ->
