@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2003-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2003-2023. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -582,6 +582,8 @@ autoimported(Config) when is_list(Config) ->
 	       {element,2},
 	       {hd,1},
 	       {length,1},
+               {max,2},
+               {min,2},
 	       {node,0},
 	       {node,1},
 	       {round,1},
@@ -857,6 +859,16 @@ action_function(Config) when is_list(Config) ->
 	    "silent(true), "
 	    "trace([send], [procs]), "
 	    "trace(Y, [procs], [send])  end)">>),
+    [{['$1','$2'],
+             [],
+             [{caller_line},
+              {current_stacktrace},
+              {current_stacktrace,3}]}] =
+        compile_and_run
+          (<<"dbg:fun2ms(fun([X,Y]) -> "
+             "caller_line(),"
+             "current_stacktrace(),"
+             "current_stacktrace(3) end)">>),
     ok.
 
 
