@@ -24,13 +24,13 @@
 %% @end
 %% =====================================================================
 
-%% @doc Pretty printing of abstract Erlang syntax trees.
-%%
-%% This module is a front end to the pretty-printing library module
-%% `prettypr', for text formatting of abstract syntax trees defined by
-%% the module `erl_syntax'.
-
 -module(erl_prettypr).
+-moduledoc """
+Pretty printing of abstract Erlang syntax trees.
+
+This module is a front end to the pretty-printing library module `prettypr`, for
+text formatting of abstract syntax trees defined by the module `erl_syntax`.
+""".
 
 -export([format/1, format/2, best/1, best/2, layout/1, layout/2,
 	 get_ctxt_precedence/1, set_ctxt_precedence/2,
@@ -52,9 +52,9 @@
 -define(NOHOOK, none).
 
 -type hook() :: 'none'
-              | fun((erl_syntax:syntaxTree(), _, _) -> prettypr:document()).
--type clause_t() :: 'case_expr' | 'cond_expr' | 'fun_expr'
-                  | 'if_expr' | 'receive_expr' | 'try_expr'
+              | fun((syntaxTree(), _, _) -> prettypr:document()).
+-type clause_t() :: 'case_expr' | 'fun_expr'
+                  | 'if_expr' | 'maybe_expr' | 'receive_expr' | 'try_expr'
                   | {'function', prettypr:document()}
                   | 'spec'.
 
@@ -66,32 +66,29 @@
 	       paper = ?PAPER     :: integer(),
 	       ribbon = ?RIBBON   :: integer(),
 	       user = ?NOUSER     :: term(),
-               encoding = epp:default_encoding() :: epp:source_encoding()}).
+               encoding = epp:default_encoding() :: epp:source_encoding(),
+	       empty_lines = sets:new() :: sets:set(integer())}).
 
 -type context() :: #ctxt{}.
 
-%% =====================================================================
-%% The following functions examine and modify contexts:
-
-%% @spec (context()) -> integer()
-%% @doc Returns the operator precedence field of the prettyprinter
-%% context.
-%%
-%% @see set_ctxt_precedence/2
+-doc """
+Returns the operator precedence field of the prettyprinter context.
 
+_See also: _`set_ctxt_precedence/2`.
+""".
 -spec get_ctxt_precedence(context()) -> integer().
 
 get_ctxt_precedence(Ctxt) ->
     Ctxt#ctxt.prec.
 
-%% @spec (context(), integer()) -> context()
-%%
-%% @doc Updates the operator precedence field of the prettyprinter
-%% context. See the {@link //stdlib/erl_parse} module for operator precedences.
-%%
-%% @see //stdlib/erl_parse
-%% @see get_ctxt_precedence/1
+-doc """
+Updates the operator precedence field of the prettyprinter context.
 
+See the [`//stdlib/erl_parse`](`m:erl_parse`) module for operator
+precedences.
+
+_See also: _[//stdlib/erl_parse](`m:erl_parse`), `get_ctxt_precedence/1`.
+""".
 -spec set_ctxt_precedence(context(), integer()) -> context().
 
 set_ctxt_precedence(Ctxt, Prec) ->
@@ -103,183 +100,174 @@ set_prec(Ctxt, Prec) ->
 reset_prec(Ctxt) ->
     set_prec(Ctxt, 0).    % used internally
 
-%% @spec (context()) -> integer()
-%% @doc Returns the paper widh field of the prettyprinter context.
-%% @see set_ctxt_paperwidth/2
+-doc """
+Returns the paper widh field of the prettyprinter context.
 
+_See also: _`set_ctxt_paperwidth/2`.
+""".
 -spec get_ctxt_paperwidth(context()) -> integer().
 
 get_ctxt_paperwidth(Ctxt) ->
     Ctxt#ctxt.paper.
 
-%% @spec (context(), integer()) -> context()
-%%
-%% @doc Updates the paper widh field of the prettyprinter context.
-%%
-%% Note: changing this value (and passing the resulting context to a
-%% continuation function) does not affect the normal formatting, but may
-%% affect user-defined behaviour in hook functions.
-%%
-%% @see get_ctxt_paperwidth/1
+-doc """
+Updates the paper widh field of the prettyprinter context.
 
+> #### Note {: .info }
+>
+> Changing this value (and passing the resulting context to a
+> continuation function) does not affect the normal formatting, but may
+> affect user-defined behaviour in hook functions.
+
+_See also: _`get_ctxt_paperwidth/1`.
+""".
 -spec set_ctxt_paperwidth(context(), integer()) -> context().
 
 set_ctxt_paperwidth(Ctxt, W) ->
     Ctxt#ctxt{paper = W}.
 
-%% @spec (context()) -> integer()
-%% @doc Returns the line widh field of the prettyprinter context.
-%% @see set_ctxt_linewidth/2
+-doc """
+Returns the line widh field of the prettyprinter context.
 
+_See also: _`set_ctxt_linewidth/2`.
+""".
 -spec get_ctxt_linewidth(context()) -> integer().
 
 get_ctxt_linewidth(Ctxt) ->
     Ctxt#ctxt.ribbon.
 
-%% @spec (context(), integer()) -> context()
-%%
-%% @doc Updates the line widh field of the prettyprinter context.
-%%
-%% Note: changing this value (and passing the resulting context to a
-%% continuation function) does not affect the normal formatting, but may
-%% affect user-defined behaviour in hook functions.
-%%
-%% @see get_ctxt_linewidth/1
+-doc """
+Updates the line widh field of the prettyprinter context.
 
+> #### Note {: .info }
+>
+> Changing this value (and passing the resulting context to a
+> continuation function) does not affect the normal formatting, but may
+> affect user-defined behaviour in hook functions.
+
+_See also: _`get_ctxt_linewidth/1`.
+""".
 -spec set_ctxt_linewidth(context(), integer()) -> context().
 
 set_ctxt_linewidth(Ctxt, W) ->
     Ctxt#ctxt{ribbon = W}.
 
-%% @spec (context()) -> hook()
-%% @doc Returns the hook function field of the prettyprinter context.
-%% @see set_ctxt_hook/2
+-doc """
+Returns the hook function field of the prettyprinter context.
 
+_See also: _`set_ctxt_hook/2`.
+""".
 -spec get_ctxt_hook(context()) -> hook().
 
 get_ctxt_hook(Ctxt) ->
     Ctxt#ctxt.hook.
 
-%% @spec (context(), hook()) -> context()
-%% @doc Updates the hook function field of the prettyprinter context.
-%% @see get_ctxt_hook/1
+-doc """
+Updates the hook function field of the prettyprinter context.
 
+_See also: _`get_ctxt_hook/1`.
+""".
 -spec set_ctxt_hook(context(), hook()) -> context().
 
 set_ctxt_hook(Ctxt, Hook) ->
     Ctxt#ctxt{hook = Hook}.
 
-%% @spec (context()) -> term()
-%% @doc Returns the user data field of the prettyprinter context.
-%% @see set_ctxt_user/2
+-doc """
+Returns the user data field of the prettyprinter context.
 
+_See also: _`set_ctxt_user/2`.
+""".
 -spec get_ctxt_user(context()) -> term().
 
 get_ctxt_user(Ctxt) ->
     Ctxt#ctxt.user.
 
-%% @spec (context(), term()) -> context()
-%% @doc Updates the user data field of the prettyprinter context.
-%% @see get_ctxt_user/1
+-doc """
+Updates the user data field of the prettyprinter context.
 
+_See also: _`get_ctxt_user/1`.
+""".
 -spec set_ctxt_user(context(), term()) -> context().
 
 set_ctxt_user(Ctxt, X) ->
     Ctxt#ctxt{user = X}.
 
 
-%% =====================================================================
-%% @spec format(Tree::syntaxTree()) -> string()
-%% @equiv format(Tree, [])
-
--spec format(erl_syntax:syntaxTree()) -> string().
+-doc #{equiv => format(Tree, [])}.
+-spec format(syntaxTree()) -> string().
 
 format(Node) ->
     format(Node, []).
 
+-doc """
+An abstract syntax tree.
 
-%% =====================================================================
-%% @spec format(Tree::syntaxTree(), Options::[term()]) -> string()
-%%
-%% @type syntaxTree() = erl_syntax:syntaxTree().
-%%
-%% An abstract syntax tree. See the {@link erl_syntax} module for
-%% details.
-%%
-%% @type hook() = (syntaxTree(), context(), Continuation) ->
-%%                            prettypr:document()
-%%	    Continuation = (syntaxTree(), context()) ->
-%%                            prettypr:document().
-%%
-%% A call-back function for user-controlled formatting. See {@link
-%% format/2}.
-%%
-%% @type context(). A representation of the current context of the
-%% pretty-printer. Can be accessed in hook functions.
-%%
-%% @doc Prettyprint-formats an abstract Erlang syntax tree as text. For
-%% example, if you have a `.beam' file that has been compiled with
-%% `debug_info', the following should print the source code for the
-%% module (as it looks in the debug info representation):
-%% ```{ok,{_,[{abstract_code,{_,AC}}]}} =
-%%            beam_lib:chunks("myfile.beam",[abstract_code]),
-%%    io:put_chars(erl_prettypr:format(erl_syntax:form_list(AC)))
-%% '''
-%%
-%% Available options:
-%% <dl>
-%%   <dt>{hook, none | {@link hook()}}</dt>
-%%       <dd>Unless the value is `none', the given function is called
-%%       for each node whose list of annotations is not empty; see below
-%%       for details. The default value is `none'.</dd>
-%%
-%%   <dt>{paper, integer()}</dt>
-%%       <dd>Specifies the preferred maximum number of characters on any
-%%       line, including indentation. The default value is 80.</dd>
-%%
-%%   <dt>{ribbon, integer()}</dt>
-%%       <dd>Specifies the preferred maximum number of characters on any
-%%       line, not counting indentation. The default value is 65.</dd>
-%%
-%%   <dt>{user, term()}</dt>
-%%       <dd>User-specific data for use in hook functions. The default
-%%       value is `undefined'.</dd>
-%%   <dt>{encoding, epp:source_encoding()}</dt>
-%%       <dd>Specifies the encoding of the generated file.</dd>
-%% </dl>
-%%
-%% A hook function (cf. the {@link hook()} type) is passed the current
-%% syntax tree node, the context, and a continuation. The context can be
-%% examined and manipulated by functions such as `get_ctxt_user/1' and
-%% `set_ctxt_user/2'. The hook must return a "document" data structure
-%% (see {@link layout/2} and {@link best/2}); this may be constructed in
-%% part or in whole by applying the continuation function. For example,
-%% the following is a trivial hook:
-%% ```
-%%     fun (Node, Ctxt, Cont) -> Cont(Node, Ctxt) end
-%% '''
-%% which yields the same result as if no hook was given.
-%% The following, however:
-%% ```
-%%     fun (Node, Ctxt, Cont) ->
-%%         Doc = Cont(Node, Ctxt),
-%%         prettypr:beside(prettypr:text("<b>"),
-%%                         prettypr:beside(Doc,
-%%                                         prettypr:text("</b>")))
-%%     end
-%% '''
-%% will place the text of any annotated node (regardless of the
-%% annotation data) between HTML "boldface begin" and "boldface end"
-%% tags.
-%%
-%% @see erl_syntax
-%% @see format/1
-%% @see layout/2
-%% @see best/2
-%% @see get_ctxt_user/1
-%% @see set_ctxt_user/2
+See the `m:erl_syntax` module for details.
+""".
+-type syntaxTree() :: erl_syntax:syntaxTree().
 
--spec format(erl_syntax:syntaxTree(), [term()]) -> string().
+
+-doc """
+Prettyprint-formats an abstract Erlang syntax tree as text.
+
+For example, if you have a `.beam` file that has been compiled with
+`debug_info`, the following should print the source code for the
+module (as it looks in the debug info representation):
+
+```text
+     {ok,{_,[{abstract_code,{_,AC}}]}} =
+             beam_lib:chunks("myfile.beam",[abstract_code]),
+     io:put_chars(erl_prettypr:format(erl_syntax:form_list(AC)))
+```
+
+Available options:
+
+- **`{hook, none | hook()}`** - Unless the value is `none`, the
+  given function is called for each node whose list of annotations is
+  not empty. The default value is `none`.
+
+- **`{paper, integer()}`** - Specifies the preferred maximum number of
+  characters on any line, including indentation. The default value is 80.
+
+- **`{ribbon, integer()}`** - Specifies the preferred maximum number of
+  characters on any line, not counting indentation. The default value is 65.
+
+- **`{user, term()}`** - User-specific data for use in hook functions. The
+  default value is `undefined`.
+
+- **`{encoding, epp:source_encoding()}`** - Specifies the encoding of the
+  generated file.
+
+A hook function (see the [`hook()`](`t:hook/0`) type) is passed the current
+syntax tree node, the context, and a continuation. The context can be examined
+and manipulated by functions such as [`get_ctxt_user/1`](`get_ctxt_user/1`) and
+[`set_ctxt_user/2`](`set_ctxt_user/2`). The hook must return a "document" data
+structure (see `layout/2` and `best/2`); this may be constructed in part or in
+whole by applying the continuation function. For example, the following is a
+trivial hook:
+
+```text
+      fun (Node, Ctxt, Cont) -> Cont(Node, Ctxt) end
+```
+
+which yields the same result as if no hook was given. The following, however:
+
+```text
+      fun (Node, Ctxt, Cont) ->
+          Doc = Cont(Node, Ctxt),
+          prettypr:beside(prettypr:text("<b>"),
+                          prettypr:beside(Doc,
+                                          prettypr:text("</b>")))
+      end
+```
+
+will place the text of any annotated node (regardless of the annotation data)
+between HTML "boldface begin" and "boldface end" tags.
+
+_See also: _`m:erl_syntax`, `best/2`, `format/1`, `get_ctxt_user/1`, `layout/2`,
+`set_ctxt_user/2`.
+""".
+-spec format(syntaxTree(), [term()]) -> string().
 
 format(Node, Options) ->
     W = proplists:get_value(paper, Options, ?PAPER),
@@ -287,32 +275,25 @@ format(Node, Options) ->
     prettypr:format(layout(Node, Options), W, L).
 
 
-%% =====================================================================
-%% @spec best(Tree::syntaxTree()) -> empty | prettypr:document()
-%% @equiv best(Tree, [])
-
--spec best(erl_syntax:syntaxTree()) -> 'empty' | prettypr:document().
+-doc #{equiv => best(Tree, [])}.
+-spec best(syntaxTree()) -> 'empty' | prettypr:document().
 
 best(Node) ->
     best(Node, []).
 
 
-%% =====================================================================
-%% @spec best(Tree::syntaxTree(), Options::[term()]) ->
-%%           empty | prettypr:document()
-%%
-%% @doc Creates a fixed "best" abstract layout for a syntax tree. This
-%% is similar to the `layout/2' function, except that here, the final
-%% layout has been selected with respect to the given options. The atom
-%% `empty' is returned if no such layout could be produced. For
-%% information on the options, see the `format/2' function.
-%%
-%% @see best/1
-%% @see layout/2
-%% @see format/2
-%% @see prettypr:best/3
+-doc """
+Creates a fixed "best" abstract layout for a syntax tree.
 
--spec best(erl_syntax:syntaxTree(), [term()]) -> 'empty' | prettypr:document().
+This is similar to the [`layout/2`](`layout/2`) function, except that
+here, the final layout has been selected with respect to the given
+options. The atom `empty` is returned if no such layout could be
+produced. For information on the options, see the
+[`format/2`](`format/2`) function.
+
+_See also: _`best/1`, `format/2`, `layout/2`, `prettypr:best/3`.
+""".
+-spec best(syntaxTree(), [term()]) -> 'empty' | prettypr:document().
 
 best(Node, Options) ->
     W = proplists:get_value(paper, Options, ?PAPER),
@@ -320,36 +301,30 @@ best(Node, Options) ->
     prettypr:best(layout(Node, Options), W, L).
 
 
-%% =====================================================================
-%% @spec layout(Tree::syntaxTree()) -> prettypr:document()
-%% @equiv layout(Tree, [])
-
--spec layout(erl_syntax:syntaxTree()) -> prettypr:document().
+-doc #{equiv => layout(Tree, [])}.
+-spec layout(syntaxTree()) -> prettypr:document().
 
 layout(Node) ->
     layout(Node, []).
 
 
-%% =====================================================================
-%% @spec layout(Tree::syntaxTree(), Options::[term()]) -> prettypr:document()
-%%
-%% @doc Creates an abstract document layout for a syntax tree. The
-%% result represents a set of possible layouts (cf. module `prettypr').
-%% For information on the options, see {@link format/2}; note, however,
-%% that the `paper' and `ribbon' options are ignored by this function.
-%%
-%% This function provides a low-level interface to the pretty printer,
-%% returning a flexible representation of possible layouts, independent
-%% of the paper width eventually to be used for formatting. This can be
-%% included as part of another document and/or further processed
-%% directly by the functions in the `prettypr' module, or used in a hook
-%% function (see `format/2' for details).
-%%
-%% @see prettypr
-%% @see format/2
-%% @see layout/1
+-doc """
+Creates an abstract document layout for a syntax tree.
 
--spec layout(erl_syntax:syntaxTree(), [term()]) -> prettypr:document().
+The result represents a set of possible layouts (see module
+`m:prettypr`). For information on the options, see `format/2`;
+however, note that the `paper` and `ribbon` options are ignored by
+this function.
+
+This function provides a low-level interface to the pretty printer, returning a
+flexible representation of possible layouts, independent of the paper width
+eventually to be used for formatting. This can be included as part of another
+document and/or further processed directly by the functions in the `prettypr`
+module, or used in a hook function (see [`format/2`](`format/2`) for details).
+
+_See also: _`m:prettypr`, `format/2`, `layout/1`.
+""".
+-spec layout(syntaxTree(), [term()]) -> prettypr:document().
 
 layout(Node, Options) ->
     lay(Node,
@@ -358,7 +333,8 @@ layout(Node, Options) ->
 	      ribbon = proplists:get_value(ribbon, Options, ?RIBBON),
 	      user = proplists:get_value(user, Options),
               encoding = proplists:get_value(encoding, Options,
-                                             epp:default_encoding())}).
+                                             epp:default_encoding()),
+              empty_lines = proplists:get_value(empty_lines, Options, sets:new())}).
 
 lay(Node, Ctxt) ->
     case erl_syntax:get_ann(Node) of
@@ -474,13 +450,13 @@ lay_2(Node, Ctxt) ->
 		     floating(text(",")), reset_prec(Ctxt),
 		     fun lay/2),
 	    beside(floating(text("{")),
-		   beside(par(Es),
+		   beside(sep(Es),
 			  floating(text("}"))));
 	
 	list ->
 	    Ctxt1 = reset_prec(Ctxt),
 	    Node1 = erl_syntax:compact_list(Node),
-	    D1 = par(seq(erl_syntax:list_prefix(Node1),
+	    D1 = sep(seq(erl_syntax:list_prefix(Node1),
 			 floating(text(",")), Ctxt1,
 			 fun lay/2)),
 	    D = case erl_syntax:list_suffix(Node1) of
@@ -513,7 +489,7 @@ lay_2(Node, Ctxt) ->
 	    D2 = lay(Operator, reset_prec(Ctxt)),
 	    D3 = lay(erl_syntax:infix_expr_right(Node),
 		     set_prec(Ctxt, PrecR)),
-	    D4 = par([D1, D2, D3], Ctxt#ctxt.sub_indent),
+	    D4 = par([D1, D2, D3], Ctxt#ctxt.break_indent),
 	    maybe_parentheses(D4, Prec, Ctxt);
 	
 	prefix_expr ->
@@ -535,7 +511,7 @@ lay_2(Node, Ctxt) ->
 		     '-' ->
 			 beside(D1, D2);
 		     _ ->
-			 par([D1, D2], Ctxt#ctxt.sub_indent)
+			 par([D1, D2], Ctxt#ctxt.break_indent)
 		 end,
 	    maybe_parentheses(D3, Prec, Ctxt);
 	
@@ -547,7 +523,7 @@ lay_2(Node, Ctxt) ->
 		     floating(text(",")), reset_prec(Ctxt),
 		     fun lay/2),
 	    D1 = beside(D, beside(text("("),
-				  beside(par(As),
+				  beside(sep(As),
 					 floating(text(")"))))),
 	    maybe_parentheses(D1, Prec, Ctxt);
 	
@@ -576,9 +552,7 @@ lay_2(Node, Ctxt) ->
 		     G ->
 			 lay(G, Ctxt1)
 		 end,
-	    D3 = sep(seq(erl_syntax:clause_body(Node),
-			 floating(text(",")), Ctxt1,
-			 fun lay/2)),
+	    D3 = lay_clause_expressions(erl_syntax:clause_body(Node), Ctxt1),
 	    case Ctxt#ctxt.clause of
 		fun_expr ->
 		    make_fun_clause(D1, D2, D3, Ctxt);
@@ -586,10 +560,10 @@ lay_2(Node, Ctxt) ->
 		    make_fun_clause(N, D1, D2, D3, Ctxt);
 		if_expr ->
 		    make_if_clause(D1, D2, D3, Ctxt);
-		cond_expr ->
-		    make_if_clause(D1, D2, D3, Ctxt);
 		case_expr ->
 		    make_case_clause(D1, D2, D3, Ctxt);
+		maybe_expr ->
+		    make_case_clause(D1, D2, D3, Ctxt);
 		receive_expr ->
 		    make_case_clause(D1, D2, D3, Ctxt);
 		try_expr ->
@@ -614,32 +588,24 @@ lay_2(Node, Ctxt) ->
 	    D1 = lay(erl_syntax:case_expr_argument(Node), Ctxt1),
 	    D2 = lay_clauses(erl_syntax:case_expr_clauses(Node),
 			     case_expr, Ctxt1),
-	    sep([par([follow(text("case"), D1, Ctxt1#ctxt.sub_indent),
+	    sep([par([follow(text("case"), D1, Ctxt1#ctxt.break_indent),
 		      text("of")],
 		     Ctxt1#ctxt.break_indent),
-		 nest(Ctxt1#ctxt.sub_indent, D2),
+		 nest(Ctxt1#ctxt.break_indent, D2),
 		 text("end")]);
 	
 	if_expr ->
 	    Ctxt1 = reset_prec(Ctxt),
 	    D = lay_clauses(erl_syntax:if_expr_clauses(Node),
 			    if_expr, Ctxt1),
-	    sep([follow(text("if"), D, Ctxt1#ctxt.sub_indent),
-		 text("end")]);
-
-	cond_expr ->
-	    Ctxt1 = reset_prec(Ctxt),
-	    D = lay_clauses(erl_syntax:cond_expr_clauses(Node),
-			    cond_expr, Ctxt1),
-	    sep([text("cond"),
-		 nest(Ctxt1#ctxt.sub_indent, D),
+	    sep([follow(text("if"), D, Ctxt1#ctxt.break_indent),
 		 text("end")]);
 
 	fun_expr ->
 	    Ctxt1 = reset_prec(Ctxt),
 	    D = lay_clauses(erl_syntax:fun_expr_clauses(Node),
 			    fun_expr, Ctxt1),
-	    sep([follow(text("fun"), D, Ctxt1#ctxt.sub_indent),
+	    sep([follow(text("fun"), D, Ctxt1#ctxt.break_indent),
 		 text("end")]);
 
         named_fun_expr ->
@@ -647,7 +613,7 @@ lay_2(Node, Ctxt) ->
             D1 = lay(erl_syntax:named_fun_expr_name(Node), Ctxt1),
             D = lay_clauses(erl_syntax:named_fun_expr_clauses(Node),
                             {function,D1}, Ctxt1),
-            sep([follow(text("fun"), D, Ctxt1#ctxt.sub_indent),
+            sep([follow(text("fun"), D, Ctxt1#ctxt.break_indent),
                  text("end")]);
 
 	module_qualifier ->
@@ -658,6 +624,34 @@ lay_2(Node, Ctxt) ->
 		     set_prec(Ctxt, PrecR)),
 	    beside(D1, beside(text(":"), D2));
 
+        maybe_expr ->
+	    Ctxt1 = reset_prec(Ctxt),
+	    D1 = vertical(seq(erl_syntax:maybe_expr_body(Node),
+                              floating(text(",")), Ctxt1, fun lay/2)),
+            Es0 = [text("end")],
+            Es1 = case erl_syntax:maybe_expr_else(Node) of
+                      none -> Es0;
+                      ElseNode ->
+                          ElseCs = erl_syntax:else_expr_clauses(ElseNode),
+                          D3 = lay_clauses(ElseCs, maybe_expr, Ctxt1),
+                          [text("else"),
+                           nest(Ctxt1#ctxt.break_indent, D3)
+                          | Es0]
+                  end,
+	    sep([par([text("maybe"), nest(Ctxt1#ctxt.break_indent, D1),
+		      hd(Es1)]) | tl(Es1)]);
+
+	maybe_match_expr ->
+	    {PrecL, Prec, PrecR} = inop_prec('='),
+	    D1 = lay(erl_syntax:maybe_match_expr_pattern(Node),
+		     set_prec(Ctxt, PrecL)),
+	    D2 = lay(erl_syntax:maybe_match_expr_body(Node),
+		     set_prec(Ctxt, PrecR)),
+	    D3 = follow(beside(D1, floating(text(" ?="))), D2,
+			Ctxt#ctxt.break_indent),
+	    maybe_parentheses(D3, Prec, Ctxt);
+
+
 	%%
 	%% The rest is in alphabetical order (except map and types)
 	%%
@@ -734,7 +728,7 @@ lay_2(Node, Ctxt) ->
                     _ when Args =:= none ->
 			lay(N, Ctxt1);
                     _ ->
-                        D1 = par(seq(Args, floating(text(",")), Ctxt1,
+                        D1 = sep(seq(Args, text(","), Ctxt1,
                                      fun lay/2)),
 			beside(lay(N, Ctxt1),
 			       beside(text("("),
@@ -766,14 +760,14 @@ lay_2(Node, Ctxt) ->
 	    Es = seq(erl_syntax:block_expr_body(Node),
 		     floating(text(",")), Ctxt1, fun lay/2),
 	    sep([text("begin"),
-		 nest(Ctxt1#ctxt.sub_indent, sep(Es)),
+		 nest(Ctxt1#ctxt.break_indent, sep(Es)),
 		 text("end")]);
 
-	catch_expr ->
+	'catch_expr' ->                         %Quoted to help Emacs.
 	    {Prec, PrecR} = preop_prec('catch'),
 	    D = lay(erl_syntax:catch_expr_body(Node),
 		    set_prec(Ctxt, PrecR)),
-	    D1 = follow(text("catch"), D, Ctxt#ctxt.sub_indent),
+	    D1 = follow(text("catch"), D, Ctxt#ctxt.break_indent),
 	    maybe_parentheses(D1, Prec, Ctxt);
 
 	class_qualifier ->
@@ -811,7 +805,7 @@ lay_2(Node, Ctxt) ->
 	    sep(seq(erl_syntax:disjunction_body(Node),
 		    floating(text(";")), reset_prec(Ctxt),
 		    fun lay/2));
-	    
+
 	error_marker ->
 	    E = erl_syntax:error_marker_info(Node),
 	    beside(text("** "),
@@ -838,6 +832,12 @@ lay_2(Node, Ctxt) ->
 	    D2 = lay(erl_syntax:binary_generator_body(Node), Ctxt1),
 	    par([D1, beside(text("<= "), D2)], Ctxt1#ctxt.break_indent);
 
+	map_generator ->
+	    Ctxt1 = reset_prec(Ctxt),
+	    D1 = lay(erl_syntax:map_generator_pattern(Node), Ctxt1),
+	    D2 = lay(erl_syntax:map_generator_body(Node), Ctxt1),
+	    par([D1, beside(text("<- "), D2)], Ctxt1#ctxt.break_indent);
+
 	implicit_fun ->
 	    D = lay(erl_syntax:implicit_fun_name(Node),
 		    reset_prec(Ctxt)),
@@ -854,7 +854,7 @@ lay_2(Node, Ctxt) ->
 				   beside(D2, floating(text("]"))))]));
 
 	binary_comp ->
-	    Ctxt1 = reset_prec(Ctxt),
+	    Ctxt1 = set_prec(Ctxt, max_prec()),
 	    D1 = lay(erl_syntax:binary_comp_template(Node), Ctxt1),
 	    D2 = par(seq(erl_syntax:binary_comp_body(Node),
 			 floating(text(",")), Ctxt1,
@@ -863,6 +863,15 @@ lay_2(Node, Ctxt) ->
 		   par([D1, beside(floating(text(" || ")),
 				   beside(D2, floating(text(" >>"))))]));
 
+	map_comp ->
+	    Ctxt1 = set_prec(Ctxt, max_prec()),
+	    D1 = lay(erl_syntax:map_comp_template(Node), Ctxt1),
+	    D2 = par(seq(erl_syntax:map_comp_body(Node),
+			 floating(text(",")), Ctxt1,
+			 fun lay/2)),
+	    beside(floating(text("#{")),
+		   par([D1, beside(floating(text("|| ")),
+				   beside(D2, floating(text("}"))))]));
 	macro ->
 	    %% This is formatted similar to a normal function call, but
 	    %% prefixed with a "?".
@@ -903,10 +912,10 @@ lay_2(Node, Ctxt) ->
 			      follow(floating(text("after")),
 				     append_clause_body(D4, D3,
 							Ctxt1),
-				     Ctxt1#ctxt.sub_indent)])
+				     Ctxt1#ctxt.break_indent)])
 		 end,
 	    sep([text("receive"),
-		 nest(Ctxt1#ctxt.sub_indent, D2),
+		 nest(Ctxt1#ctxt.break_indent, D2),
 		 text("end")]);
 
 	record_access ->
@@ -1003,7 +1012,7 @@ lay_2(Node, Ctxt) ->
             D1 = lay(erl_syntax:typed_record_field_body(Node), Ctxt1),
             D2 = lay(erl_syntax:typed_record_field_type(Node),
                      set_prec(Ctxt, Prec)),
-            D3 = par([D1, floating(text(" ::")), D2],
+            D3 = par([D1, floating(text("::")), D2],
                      Ctxt1#ctxt.break_indent),
             maybe_parentheses(D3, Prec, Ctxt);
 
@@ -1018,7 +1027,7 @@ lay_2(Node, Ctxt) ->
 			  D2 = sep(seq(As, floating(text(",")), Ctxt1,
 				       fun lay/2)),
 			  [text("after"),
-			   nest(Ctxt1#ctxt.sub_indent, D2)
+			   nest(Ctxt1#ctxt.break_indent, D2)
 			   | Es0]
 		  end,
 	    Es2 = case erl_syntax:try_expr_handlers(Node) of
@@ -1026,7 +1035,7 @@ lay_2(Node, Ctxt) ->
 		      Hs ->
 			  D3 = lay_clauses(Hs, try_expr, Ctxt1),
 			  [text("catch"),
-			   nest(Ctxt1#ctxt.sub_indent, D3)
+			   nest(Ctxt1#ctxt.break_indent, D3)
 			   | Es1]
 		  end,
 	    Es3 = case erl_syntax:try_expr_clauses(Node) of
@@ -1034,10 +1043,10 @@ lay_2(Node, Ctxt) ->
 		      Cs ->
 			  D4 = lay_clauses(Cs, try_expr, Ctxt1),
 			  [text("of"),
-			   nest(Ctxt1#ctxt.sub_indent, D4)
+			   nest(Ctxt1#ctxt.break_indent, D4)
 			   | Es2]
 		  end,
-	    sep([par([follow(text("try"), D1, Ctxt1#ctxt.sub_indent),
+	    sep([par([follow(text("try"), D1, Ctxt1#ctxt.break_indent),
 		      hd(Es3)])
 		 | tl(Es3)]);
 
@@ -1213,12 +1222,12 @@ lay_2(Node, Ctxt) ->
                              floating(text(",")), reset_prec(Ctxt),
                              fun lay/2),
                     beside(floating(text("{")),
-                           beside(par(Es), floating(text("}"))))
+                           beside(sep(Es), floating(text("}"))))
             end;
 
         type_union ->
             {_, Prec, PrecR} = type_inop_prec('|'),
-            Es = par(seq(erl_syntax:type_union_types(Node),
+            Es = sep(seq(erl_syntax:type_union_types(Node),
                          floating(text(" |")), set_prec(Ctxt, PrecR),
                          fun lay/2)),
             maybe_parentheses(Es, Prec, Ctxt);
@@ -1504,5 +1513,31 @@ tidy_float_2([$e | Cs]) -> tidy_float_2([$e, $+ | Cs]);
 tidy_float_2([_C | Cs]) -> tidy_float_2(Cs);
 tidy_float_2([]) -> [].
 
+lay_clause_expressions([H], Ctxt) ->
+	lay(H, Ctxt);
+lay_clause_expressions([H | T], Ctxt) ->
+    Clause = beside(lay(H, Ctxt), floating(text(","))),
+    Next = lay_clause_expressions(T, Ctxt),
+    case is_last_and_before_empty_line(H, T, Ctxt) of
+	true ->
+	    above(above(Clause, text("")), Next);
+        false ->
+            above(Clause, Next)
+    end;
+lay_clause_expressions([], _) ->
+    empty().
+
+is_last_and_before_empty_line(H, [], #ctxt{empty_lines = EmptyLines}) ->
+    try sets:is_element(get_line(H) + 1, EmptyLines)
+    catch error:badarith -> false
+    end;
+is_last_and_before_empty_line(H, [H2 | _], #ctxt{empty_lines = EmptyLines}) ->
+    try ((get_line(H2) - get_line(H)) >= 2) and sets:is_element(get_line(H) + 1, EmptyLines)
+    catch error:badarith -> false
+    end.
+
+get_line(Tree) ->
+    Anno = erl_syntax:get_pos(Tree),
+    erl_anno:line(Anno).
 
 %% =====================================================================
