@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2016. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2021. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -108,13 +108,6 @@ basic(Config) when is_list(Config) ->
     ok.
 
 basic_option(Config) when is_list(Config) ->
-    %% Eprof is not supported on native-compile code.
-    case lists:module_info(native_addresses) of
-	[] -> basic_option_1(Config);
-	[_|_] -> {skip,"lists is native-compiled"}
-    end.
-
-basic_option_1(Config) ->
 
     %% load eprof_test and change directory
 
