@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2008-2023. All Rights Reserved.
+ * Copyright Ericsson AB 2008-2024. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -81,3 +81,9 @@
 #ifndef wxICON_DEFAULT_BITMAP_TYPE
   #define wxICON_DEFAULT_BITMAP_TYPE wxBITMAP_TYPE_ICO_RESOURCE
 #endif
+
+
+#if defined(wxSTC_DISABLE_MACRO_DEPRECATIONS) && defined(wxSTC_DEPRECATED_MACRO_VALUE)
+#undef wxSTC_DEPRECATED_MACRO_VALUE
+#define wxSTC_DEPRECATED_MACRO_VALUE(value, msg) value
+#endif
